/*
**  Face.m
**
**  Copyright 2001, 2002
** 
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "Face.h"

#include "../../NSUserDefaultsExtensions.h"
#import "GNUMailConstants.h"

//#include <compface.h>

@implementation Face

//
//
//
- (id) init
{
  self = [super init];

  buf = NULL;
  bitmap = nil;
  image = nil;
  

  return self;
}

//
//
//
- (id) initWithCString: (char *) theCString
{
  self = [self init];

  buf = (char *)malloc( sizeof(char) * 4096 );
  memcpy(buf, theCString, strlen(theCString));
  
  //if ( uncompface(buf) < 0 )
  //  {
  //    NSDebugLog(@"Fatal error occured when uncompressing the X-Face");
  //    bitmap = nil;
  //    return self;
  //  }

  NSDebugLog(@"Uncompressed the X-Face!");
  bitmap = [[NSString stringWithCString: buf] componentsSeparatedByString: @","];
  RETAIN(bitmap);

  return self;
}


//
//
//
- (id) initWithURL: (NSURL *) theURL
{
  self = [self init];

  image = [[NSImage alloc] initWithContentsOfURL: theURL];

  return self;
}


//
//
//
- (void) dealloc
{
  NSDebugLog(@"Face: -dealloc");

  if ( buf )
    {
      free(buf);
    }
  
  TEST_RELEASE(bitmap);
  TEST_RELEASE(image);

  [super dealloc];
}


//
//
//
- (NSImage *) image
{
  return image;
}


//
//
//
- (void) setImage: (NSImage *) theImage
{
  if ( theImage )
    {
      RETAIN(theImage);
      RELEASE(image);
      
      image = theImage;
    }
  else
    {
      NSDebugLog(@"Face: -setImage. attempted to set a nil image.");

      RELEASE(image);
      image = nil;
    }
}


//
// Implementation of the inherited method from NSView
//
- (void) drawRect: (NSRect) rect
{
  NSColor *aColor;

  // We first clear our view
  //[[NSColor controlBackgroundColor] set];
  aColor = [[NSUserDefaults standardUserDefaults] colorForKey: @"MAILHEADERCELL_COLOR"];
  
  if ( !aColor )
    {
      aColor = [NSColor colorWithCalibratedRed: 0.9 
			green: 0.9 
			blue: 1.0 
			alpha: 1.0];
    }

  [aColor set];
  NSRectFill( NSMakeRect(0,0,64,64) );
 
  // Safe guard.
  if ( !image )
    {
      return;
    }

  // We scale the image properly before compositing it
  [image setScalesWhenResized: YES];
  [image setSize: NSMakeSize(64,64)];


  [image compositeToPoint: NSMakePoint(0,0) 
	 operation: NSCompositeSourceOver];
  

  //
  // TO REACTIVATE
  // 
#if 0
  NSBitmapImageRep *imageRep;
  NSImage *image;
  
  unsigned int i, j, n, x, y;
 
  char revtable[256] = {
    0, -128,   64,  -64,   32,  -96,   96,  -32,
    16, -112,   80,  -48,   48,  -80,  112,  -16,
    8, -120,   72,  -56,   40,  -88,  104,  -24,
    24, -104,   88,  -40,   56,  -72,  120,   -8,
    4, -124,   68,  -60,   36,  -92,  100,  -28,
    20, -108,   84,  -44,   52,  -76,  116,  -12,
    12, -116,   76,  -52,   44,  -84,  108,  -20,
    28, -100,   92,  -36,   60,  -68,  124,   -4,
    2, -126,   66,  -62,   34,  -94,   98,  -30,
    18, -110,   82,  -46,   50,  -78,  114,  -14,
    10, -118,   74,  -54,   42,  -86,  106,  -22,
    26, -102,   90,  -38,   58,  -70,  122,   -6,
    6, -122,   70,  -58,   38,  -90,  102,  -26,
    22, -106,   86,  -42,   54,  -74,  118,  -10,
    14, -114,   78,  -50,   46,  -82,  110,  -18,
    30,  -98,   94,  -34,   62,  -66,  126,   -2,
    1, -127,   65,  -63,   33,  -95,   97,  -31,
    17, -111,   81,  -47,   49,  -79,  113,  -15,
    9, -119,   73,  -55,   41,  -87,  105,  -23,
    25, -103,   89,  -39,   57,  -71,  121,   -7,
    5, -123,   69,  -59,   37,  -91,  101,  -27,
    21, -107,   85,  -43,   53,  -75,  117,  -11,
    13, -115,   77,  -51,   45,  -83,  109,  -19,
    29,  -99,   93,  -35,   61,  -67,  125,   -3,
    3, -125,   67,  -61,   35,  -93,   99,  -29,
    19, -109,   83,  -45,   51,  -77,  115,  -13,
    11, -117,   75,  -53,   43,  -85,  107,  -21,
    27, -101,   91,  -37,   59,  -69,  123,   -5,
    7, -121,   71,  -57,   39,  -89,  103,  -25,
    23, -105,   87,  -41,   55,  -73,  119,   -9,
    15, -113,   79,  -49,   47,  -81,  111,  -17,
    31,  -97,   95,  -33,   63,  -65,  127,   -1,
  };

  char cb[1024];

  if ( !bitmap )
    {
      NSDebugLog(@"The X-Face buffer is null.");
      return;
    }
  
  printf("%s\n", buf);

  [[NSColor whiteColor] set];
  NSRectFill( NSMakeRect(0,0,48,48) );
  
  bzero(cb, 1024);

  for (i = n = 0; i < 1024;)
    {
      if (! sscanf(&buf[n], "%i%n", &x, &y))
	{
	  break;
	}
      
      cb[i++] = revtable[(x >> 8) & 0xFF];
      cb[i++] = revtable[x & 0xFF];
      n += y;
      
      while (buf[n] && (buf[n]==',' || buf[n]=='\n')) n++;
    }
  
  printf("i = %d n = %d\n", i, n);

  [[NSColor blackColor] set];

  for (j = x = y = 0; j < i; j++)
    {
      char chunk;
      int k;

      NSDebugLog(@"x = %d, y = %d", x, y);
      chunk = cb[j];
      
      for (k = 7; k >= 0; k--)
	{
	  if ( 1 & (chunk >> k) )
	    {
	      NSRectFill( NSMakeRect( (x*8)+(7-k),
				      y,
				      1,
				      1) );
	    }
	}
      
      x += 1;
      
      if (x == 6)
	{
	  x = 0;
	  y += 1;
	}
    }

//    imageRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: &cb
//  				       pixelsWide: 48
//  				       pixelsHigh: 48
//  				       bitsPerSample: 1
//  				       samplesPerPixel: 1
//  				       hasAlpha: NO
//  				       isPlanar: YES
//  				       colorSpaceName: NSDeviceBlackColorSpace
//  				       bytesPerRow: 6
//  				       bitsPerPixel: 1];
//    NSDebugLog(@"allocated the imageRep");

//    //image = [[NSImage alloc] initWithData: [imageRep TIFFRepresentation]];
//    //image = [[NSImage alloc] initWithData: [imageRep TIFFRepresentationUsingCompression:  NSTIFFCompressionNone
//    //						   factor: 0.0]];
  
//    image = [[NSImage alloc] initWithSize: NSMakeSize(48,48)];
  
//    if (! [image drawRepresentation: imageRep
//  	       inRect: NSMakeRect(0,0,48,48)] )
//      {
//        NSDebugLog(@"draw failed.");
//      }
  
//    if ( image )
//      {
//        [image compositeToPoint: NSMakePoint(0,0) 
//  	     operation: NSCompositeSourceOver];
//      }
//    else
//      {
//        NSDebugLog(@"image is nil!");
//      }

  //[[NSColor blackColor] set];
  //n = x = y = 0;

  //NSDebugLog(@"bitmap count = %d", [bitmap count]);
  
  

//    for (i = 0; i < ([bitmap count] - 1); i++)
//      {
//        unsigned int line, c1, c2;
//        char *str;
      
//        NSDebugLog(@"x = %d, y = %d", x, y);
      
//        if ( [[bitmap objectAtIndex: i] length] > 6 )
//  	{
//  	  str = (char *)[[[bitmap objectAtIndex: i] substringFromIndex: 3] cString];
//  	}
//        else
//  	{
//  	  str = (char *)[[[bitmap objectAtIndex: i] substringFromIndex: 2] cString];
//  	}

//        sscanf(str, "%04x", &line);
//        printf("str = |%s|, line = %d\n", str, line);

//        c1 = (line >> 8) & 0xFF;
//        c2 = line & 0xFF;
//        //c1 = revtable[(line >> 8) & 0xFF];
//        //c2 = revtable[line & 0xFF];
      
//        c1 = c1 << 8;
//        line = c1;
      
//        c2 = c2 << 8;
//        c2 = c2 >> 8;
//        line = line | c2;
      
//        printf("converted line = %d\n", line);

//        //line = c2;
//        //line = line << 8;
//        //line = line | c1;
      
//        // FIXME - We must take care of the endianess of the CPU
//        for (j = 0; j < 16; j++)
//        //for (j = 15; j >= 0; j--)
//  	{
//  	  //if ( 0x8000 & (line >> j) )
//  	  //if ( 0x8000 & (j << line) )
//  	  //if ( 0x8000 & (line << j) )
//  	  if ( 1 & (line >> j) )
//  	  //if ( 1 & (line << j) )
//  	  //if (!( 1 & (line >> j) ))
//  	  //if ( line & j )
//  	    {
//  	      NSRectFill( NSMakeRect(x+j,y,1,1) );
//  	    }
//  	}
      
//        n += 1;
//        x += 16;
      
//        if (x == 48) 
//  	{
//  	  x = 0;
//  	  y++;
//  	}
//      }

#endif

}

@end
