/*
**  FaceController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "FaceController.h"

#import "Face.h"
#import "GNUMailConstants.h"

#import <Pantomime/Message.h>

static FaceController *singleInstance = nil;

@implementation FaceController

//
//
//
- (id) initWithOwner: (id) theOwner
{
  NSBundle *aBundle;
  
  self = [super init];

  owner = theOwner;
 
  aBundle = [NSBundle bundleForClass: [self class]];
  
  resourcePath = [aBundle resourcePath];
  RETAIN(resourcePath);

  allFaceViews = [[NSMutableArray alloc] init];

  cache = [[NSMutableDictionary alloc] init];

  return self;
}


//
//
//
- (void) dealloc
{
  NSDebugLog(@"FaceController: -dealloc");

  RELEASE(resourcePath);
  RELEASE(allFaceViews);
  
  RELEASE(cache);

  [super dealloc];
}



//
//
//
+ (id) singleInstance
{
  NSDebugLog(@"FaceController: -singleInstance");

  if ( !singleInstance )
    {
      singleInstance = [[FaceController alloc] initWithOwner: nil];
    }

  return singleInstance;
}


//
// access / mutation methods
//
- (NSString *) name
{
  return @"Face";
}

- (NSString *) description
{
  return @"This is a simple Face bundle supporting X-Face and X-Image-URL.";
}

- (NSString *) version
{
  return @"v0.1.0";
}

- (void) setOwner: (id) theOwner
{
  owner = theOwner;
}

//
// UI elements
//
- (BOOL) hasPreferencesPanel
{
  return NO;
}

- (BOOL) hasComposeViewAccessory
{
  return NO;
}

- (BOOL) hasViewingViewAccessory
{
  return YES;
}


//
//
//
- (id) viewingViewAccessory
{  
  Face *aFace;
      
  // FIXME - we should init with the data of the xface
  //aFace = [[Face alloc] initWithCString: "?(@Y~qjBA}~8ZMh5gM4{Q{bE_*:sCJ3@Z?{B*Co=J!#8bb~-z?-0._vJjt~MM59!MjxG%>U5>MW^2-\7~z04buszR^=m^U|m66>FdR@cFwhb;.A(8*D.QmLkK]z,md0'HiOE\pyeiv_PACR+P:Cm.wq_%l':E:q]g-UCc>r&s@BVo'kFN;(\9PF22Myg5w%nUBWQ6MJJ#qL#w>2oxckP'H:\$9F\"mxsz]Dgk{1`fTcP'Y$CgGnc^paTV$dzhVX<SVN1RPb=\"gz3>+;(U$;Eb)P<>G)g)"];
  
  //aFace = [[Face alloc] initWithCString: "B6DESMpZ'<\"P]D0{Aaq$#h/PfN=wI.C{,&<2_;#|.(\"xO+oeVic@<V)\"<@)LNEH!K`UV3kt/ISu`j'S[wrB%F+$l3ikPu:PiOhP-ONv9/^!B@R1(w`<U~w2)l`V#fbRSJ(L_2FJ_\oT8^'BQ_tJ8|xKF47(>@=zI2zD#NR~S#l*OV}["];
  
  // this one was taken on http://members.tripod.com/~maltesh/X-Angel/Angel.html - it's the Ofanim, The Wheels
  //aFace = [[Face alloc] initWithCString: "%wy.vrQuU%-*=Q9:,gdY%O@X*UD+T}MdwA+Y&91,tV:Uo\]M^^rD|>6G_ZXhm 5<|n\"S>_H49Vb'%wj`(cmx#8`Qm>\"peyZDe>lxAE!e?H`E=7Z/\"~!\Im`Ze.,vT}7:0nM64g)p+Z1#@gQ%D"];
  

  //aFace = [[Face alloc] initWithURL: [NSURL URLWithString: @"http://www.xs4all.nl/~trh/images/Tom_Hageman.tiff"]];

  aFace = [[Face alloc] init];

  [allFaceViews addObject: aFace];

  return AUTORELEASE(aFace);
}


//
//
//
- (enum ViewingViewType) viewingViewAccessoryType
{
  return ViewingViewTypeHeaderCell;
}

//
//
//
- (void) viewingViewAccessoryWillBeRemovedFromSuperview: (NSView *) theView
{

}


//
//
//
- (void) setCurrentSuperview: (NSView *) theView
{
  superview = theView;
}


//
//
//
- (NSArray *) submenuForMenu: (NSMenu *) theMenu
{
  return nil;
}


//
//
//
- (NSArray *) menuItemsForMenu: (NSMenu *) theMenu
{
  return nil;
}


//
// Pantomime related methods
//
- (void) messageWillBeDisplayed: (Message *) theMessage
			 inView: (NSTextView *) theTextView
{
  Face *aFace;
  int i;
  
  for (i = 0; i < [allFaceViews count]; i++)
    {
      aFace = [allFaceViews objectAtIndex: i];
      
      if ( [theMessage headerValueForName: @"X-Image-URL"] )
	{
	  NSString *aString;
	  
	  aString = [theMessage headerValueForName: @"X-Image-URL"];
	  
	  // We search for X-Image-URL
	  if ( aString )
	    {
	      NSImage *aImage;
	        
	      // We verify if the image is in our cache. If Not, we create it and add it to our cache.
	      aImage = [cache objectForKey: aString];
	      
	      if ( !aImage )
		{
		  aImage = [[NSImage alloc] initWithData:
					      [NSData dataWithContentsOfURL: [NSURL URLWithString: aString]]];
		  if ( aImage )
		    {
		      [cache setObject: aImage
			     forKey: aString];
		      
		      RELEASE(aImage);
		    }
		}
	      
	      [aFace setImage: aImage];
	    }
	}
      else
	{
	  [aFace setImage: nil];
	}
	  
      [aFace setNeedsDisplay: YES];
    }
}

@end
