/*
**  FontsView.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**          Completely rewritten old version by Alexey I. Froloff <raorn@binec.ru>.
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#import <AppKit/AppKit.h>
#import "FontsView.h"

#import "GNUMailConstants.h"
#import "LabelWidget.h"

@implementation FontsView

- (id) initWithParent: (id) theParent
{
  self = [super init];

  parent = theParent;

  return self;
}

- (void) layoutView
{
  LabelWidget *headerNameLabel, *headerValueLabel, *messageLabel, *plainTextMessageLabel;
  LabelWidget *label, *headerNameSizeLabel, *headerValueSizeLabel, *messageSizeLabel, *plainTextMessageSizeLabel;
  
  label =  [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 325, 300, TextFieldHeight)
			label: _(@"Font used when displaying a message for:")];
  [self addSubview: label];
  
  //
  // Header name
  //
  headerNameLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 290, 125, TextFieldHeight)
			  label: _(@"Header name:")
			  alignment: NSRightTextAlignment];
  [self addSubview: headerNameLabel];

  
  headerNamePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(150, 290, 150, ButtonHeight)];
  [headerNamePopUp setAutoenablesItems: NO];
  [self addSubview: headerNamePopUp];

  headerNameSizeLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(310, 290, 40, TextFieldHeight)
				     label: _(@"Size:")];
  [self addSubview: headerNameSizeLabel];

  
  headerNameSizePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(350, 290, 50, ButtonHeight)];
  [headerNameSizePopUp setAutoenablesItems: NO];
  [headerNameSizePopUp addItemWithTitle: @"8"];
  [headerNameSizePopUp addItemWithTitle: @"9"];
  [headerNameSizePopUp addItemWithTitle: @"10"];
  [headerNameSizePopUp addItemWithTitle: @"11"];
  [headerNameSizePopUp addItemWithTitle: @"12"];
  [headerNameSizePopUp addItemWithTitle: @"14"];
  [headerNameSizePopUp addItemWithTitle: @"16"];
  [headerNameSizePopUp addItemWithTitle: @"18"];
  [headerNameSizePopUp addItemWithTitle: @"20"];
  [headerNameSizePopUp addItemWithTitle: @"24"];
  [headerNameSizePopUp addItemWithTitle: @"28"];
  [headerNameSizePopUp addItemWithTitle: @"32"];
  [headerNameSizePopUp addItemWithTitle: @"48"];
  [self addSubview: headerNameSizePopUp];

  

  //
  // Header value
  //
  headerValueLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 255, 125, TextFieldHeight)
				  label: _(@"Header value:")
				  alignment: NSRightTextAlignment];
  [self addSubview: headerValueLabel];

  headerValuePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(150, 255, 150, ButtonHeight)];
  [headerValuePopUp setAutoenablesItems: NO];
  [self addSubview: headerValuePopUp];

  headerValueSizeLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(310, 255, 40, TextFieldHeight)
				      label: _(@"Size:")];
  [self addSubview: headerValueSizeLabel];

  headerValueSizePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(350, 255, 50, ButtonHeight)];
  [headerValueSizePopUp setAutoenablesItems: NO];
  [headerValueSizePopUp addItemWithTitle: @"8"];
  [headerValueSizePopUp addItemWithTitle: @"9"];
  [headerValueSizePopUp addItemWithTitle: @"10"];
  [headerValueSizePopUp addItemWithTitle: @"11"];
  [headerValueSizePopUp addItemWithTitle: @"12"];
  [headerValueSizePopUp addItemWithTitle: @"14"];
  [headerValueSizePopUp addItemWithTitle: @"16"];
  [headerValueSizePopUp addItemWithTitle: @"18"];
  [headerValueSizePopUp addItemWithTitle: @"20"];
  [headerValueSizePopUp addItemWithTitle: @"24"];
  [headerValueSizePopUp addItemWithTitle: @"28"];
  [headerValueSizePopUp addItemWithTitle: @"32"];
  [headerValueSizePopUp addItemWithTitle: @"48"];
  [self addSubview: headerValueSizePopUp];


  //
  // Content of message
  //
  messageLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 220, 125, TextFieldHeight)
                              label: _(@"Content of message:")
			      alignment: NSRightTextAlignment];
  [self addSubview: messageLabel];


  messagePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(150, 220, 150, ButtonHeight)];
  [messagePopUp setAutoenablesItems: NO];
  [self addSubview: messagePopUp];

  messageSizeLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(310, 220, 40, TextFieldHeight)
				     label: _(@"Size:")];
  [self addSubview: messageSizeLabel];
  
  messageSizePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(350, 220, 50, ButtonHeight)];
  [messageSizePopUp setAutoenablesItems: NO];
  [messageSizePopUp addItemWithTitle: @"8"];
  [messageSizePopUp addItemWithTitle: @"9"];
  [messageSizePopUp addItemWithTitle: @"10"];
  [messageSizePopUp addItemWithTitle: @"11"];
  [messageSizePopUp addItemWithTitle: @"12"];
  [messageSizePopUp addItemWithTitle: @"14"];
  [messageSizePopUp addItemWithTitle: @"16"];
  [messageSizePopUp addItemWithTitle: @"18"];
  [messageSizePopUp addItemWithTitle: @"20"];
  [messageSizePopUp addItemWithTitle: @"24"];
  [messageSizePopUp addItemWithTitle: @"28"];
  [messageSizePopUp addItemWithTitle: @"32"];
  [messageSizePopUp addItemWithTitle: @"48"];
  [self addSubview: messageSizePopUp];


  // Fixed width for plain messages
  checkbox = [[NSButton alloc] initWithFrame: NSMakeRect(5,175,300,ButtonHeight)];
  [checkbox setButtonType: NSSwitchButton];
  [checkbox setBordered: NO];
  [checkbox setTitle: _(@"Use fixed-width font for plain text messages")];
  [checkbox setTarget: parent];
  [checkbox setAction: @selector(checkboxClicked:)];
  [self addSubview: checkbox];

  plainTextMessageLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 145, 125, TextFieldHeight)
				       label: _(@"Plain text font:")
				       alignment: NSRightTextAlignment];
  [self addSubview: plainTextMessageLabel];


  plainTextMessagePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(150, 145, 150, ButtonHeight)];
  [plainTextMessagePopUp setAutoenablesItems: NO];
  [self addSubview: plainTextMessagePopUp];

  plainTextMessageSizeLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(310, 145, 40, TextFieldHeight)
				     label: _(@"Size:")];
  [self addSubview: plainTextMessageSizeLabel];
  
  plainTextMessageSizePopUp = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(350, 145, 50, ButtonHeight)];
  [plainTextMessageSizePopUp setAutoenablesItems: NO];
  [plainTextMessageSizePopUp addItemWithTitle: @"8"];
  [plainTextMessageSizePopUp addItemWithTitle: @"9"];
  [plainTextMessageSizePopUp addItemWithTitle: @"10"];
  [plainTextMessageSizePopUp addItemWithTitle: @"11"];
  [plainTextMessageSizePopUp addItemWithTitle: @"12"];
  [plainTextMessageSizePopUp addItemWithTitle: @"14"];
  [plainTextMessageSizePopUp addItemWithTitle: @"16"];
  [plainTextMessageSizePopUp addItemWithTitle: @"18"];
  [plainTextMessageSizePopUp addItemWithTitle: @"20"];
  [plainTextMessageSizePopUp addItemWithTitle: @"24"];
  [plainTextMessageSizePopUp addItemWithTitle: @"28"];
  [plainTextMessageSizePopUp addItemWithTitle: @"32"];
  [plainTextMessageSizePopUp addItemWithTitle: @"48"];
  [self addSubview: plainTextMessageSizePopUp];
}

- (void) dealloc
{
  RELEASE(headerNamePopUp);
  RELEASE(headerNameSizePopUp);
  RELEASE(headerValuePopUp);
  RELEASE(headerValueSizePopUp);
  RELEASE(messagePopUp);
  RELEASE(messageSizePopUp);

  RELEASE(checkbox);
  RELEASE(plainTextMessagePopUp);
  RELEASE(plainTextMessageSizePopUp);

  [super dealloc];
}


//
// access methods
//

- (NSPopUpButton *) headerNamePopUp
{
  return headerNamePopUp;
}

- (NSPopUpButton *) headerNameSizePopUp
{
  return headerNameSizePopUp;
}

- (NSPopUpButton *) headerValuePopUp
{
  return headerValuePopUp;
}

- (NSPopUpButton *) headerValueSizePopUp
{
  return headerValueSizePopUp;
}

- (NSPopUpButton *) messagePopUp
{
  return messagePopUp;
}

- (NSPopUpButton *) messageSizePopUp
{
  return messageSizePopUp;
}

- (NSButton *) checkbox
{
  return checkbox;
}

- (NSPopUpButton *) plainTextMessagePopUp
{
  return plainTextMessagePopUp;
}

- (NSPopUpButton *) plainTextMessageSizePopUp
{
  return plainTextMessageSizePopUp;
}

@end
