/*
**  FontsViewController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**          Completely rewritten old version by Alexey I. Froloff <raorn@binec.ru>.
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "FontsViewController.h"

#import "GNUMailConstants.h"

#ifndef MACOSX
#import "FontsView.h"
#endif

static FontsViewController *singleInstance = nil;

@implementation FontsViewController

- (id) initWithNibName: (NSString *) theName
{
  self = [super init];

#ifdef MACOSX
  
  if ( ![NSBundle loadNibNamed: theName
		  owner: self] )
    {
      NSDebugLog(@"Fatal error occured while loading the FontsView nib file");
      AUTORELEASE(self);
      return nil;
    }

  RETAIN(view);

#else
  // We link our view
  view = [[FontsView alloc] initWithParent: self];
  [view layoutView];

  // We link our outlets
  headerNamePopUp = [view headerNamePopUp];
  headerNameSizePopUp = [view headerNameSizePopUp];
  headerValuePopUp = [view headerValuePopUp];
  headerValueSizePopUp = [view headerValueSizePopUp];
  messagePopUp = [view messagePopUp];
  messageSizePopUp = [view messageSizePopUp];
  checkbox = [view checkbox];
  plainTextMessagePopUp = [view plainTextMessagePopUp];
  plainTextMessageSizePopUp = [view plainTextMessageSizePopUp];
#endif

  // We initialize our popup buttons
  [self _initializePopUpButtons];
    
  // We get our defaults for this panel
  [self initializeFromDefaults];

  return self;
}


//
//
//
- (void) dealloc
{
  singleInstance = nil;
  RELEASE(view);

  [super dealloc];
}


//
// action methods
//
- (IBAction) checkboxClicked: (id) sender
{
  if ( [checkbox state] == NSOnState )
    {
      [plainTextMessagePopUp setEnabled: YES];
      [plainTextMessageSizePopUp setEnabled: YES];
    }
  else
    {
      [plainTextMessagePopUp setEnabled: NO];
      [plainTextMessageSizePopUp setEnabled: NO];
    }
}

//
// access methods
//
- (NSImage *) image
{
  NSBundle *aBundle;
  
  aBundle = [NSBundle bundleForClass: [self class]];
  
  return AUTORELEASE([[NSImage alloc] initWithContentsOfFile:
					[aBundle pathForResource: @"fonts" ofType: @"tiff"]]);
}

- (NSString *) name
{
  return _(@"Fonts");
}

- (NSView *) view
{
  return view;
}

- (BOOL) hasChangesPending
{
  return YES;
}


//
//
//
- (void) initializeFromDefaults
{
  int aFontSize;

#ifdef MACOSX
  aFontSize = 10;
#else
  aFontSize = 0;
#endif

  //
  // Header name
  //
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"HEADER_NAME_FONT_NAME"] )
    {
      [headerNamePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults] 
					      stringForKey: @"HEADER_NAME_FONT_NAME"]];
      
      [headerNameSizePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults]
						  stringForKey: @"HEADER_NAME_FONT_SIZE"]];
    }
  else
    {
      NSFont *aFont;

      aFont = [NSFont systemFontOfSize: aFontSize];
      
      [headerNamePopUp selectItemWithTitle: [aFont familyName]];

      [headerNameSizePopUp selectItemWithTitle: [NSString stringWithFormat: @"%d", (int)[aFont pointSize]]];
    }
  
  
  //
  // Header value
  //
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"HEADER_VALUE_FONT_NAME"] )
    {
      [headerValuePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults] 
					       stringForKey: @"HEADER_VALUE_FONT_NAME"]];

      [headerValueSizePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults]
						   stringForKey: @"HEADER_VALUE_FONT_SIZE"]];
    }
  else
    {
      NSFont *aFont;

      aFont = [NSFont systemFontOfSize: aFontSize];
      
      [headerValuePopUp selectItemWithTitle: [aFont familyName]];

      [headerValueSizePopUp selectItemWithTitle: [NSString stringWithFormat: @"%d", (int)[aFont pointSize]]];
    }

  

  //
  // Message font
  //
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"MESSAGE_FONT_NAME"] )
    {
      [messagePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults] 
					   stringForKey: @"MESSAGE_FONT_NAME"]];
      
      [messageSizePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults]
					       stringForKey: @"MESSAGE_FONT_SIZE"]];
    }
  else
    {
      NSFont *aFont;
      
      aFont = [NSFont systemFontOfSize: aFontSize];
      
      [messagePopUp selectItemWithTitle: [aFont familyName]];

      [messageSizePopUp selectItemWithTitle: [NSString stringWithFormat: @"%d", (int)[aFont pointSize]]];
    }

  
  //
  // Plain text message font
  //
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"USE_FIXED_FONT_FOR_TEXT_PLAIN_MESSAGES"] &&
       [[NSUserDefaults standardUserDefaults] integerForKey: @"USE_FIXED_FONT_FOR_TEXT_PLAIN_MESSAGES"] == NSOnState )
    {
      [checkbox setState: NSOnState];
      [plainTextMessagePopUp setEnabled: YES];
      [plainTextMessageSizePopUp setEnabled: YES];
    }
  else
    {
      [checkbox setState: NSOffState];
      [plainTextMessagePopUp setEnabled: NO];
      [plainTextMessageSizePopUp setEnabled: NO];
    }

  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"PLAIN_TEXT_MESSAGE_FONT_NAME"] )
    {
      [plainTextMessagePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults] 
					   stringForKey: @"PLAIN_TEXT_MESSAGE_FONT_NAME"]];
      
      [plainTextMessageSizePopUp selectItemWithTitle: [[NSUserDefaults standardUserDefaults]
					       stringForKey: @"PLAIN_TEXT_MESSAGE_FONT_SIZE"]];
    }
  else
    {
      NSFont *aFont;
      
      aFont = [NSFont systemFontOfSize: aFontSize];
      
      [plainTextMessagePopUp selectItemWithTitle: [aFont familyName]];

      [plainTextMessageSizePopUp selectItemWithTitle: [NSString stringWithFormat: @"%d", (int)[aFont pointSize]]];
    }

   // We synchronize our popup buttons
  [headerNamePopUp synchronizeTitleAndSelectedItem];
  [headerNameSizePopUp synchronizeTitleAndSelectedItem];
  [headerValuePopUp synchronizeTitleAndSelectedItem];
  [headerValueSizePopUp synchronizeTitleAndSelectedItem];
  [messagePopUp synchronizeTitleAndSelectedItem];
  [messageSizePopUp synchronizeTitleAndSelectedItem];
  [plainTextMessagePopUp synchronizeTitleAndSelectedItem];
  [plainTextMessageSizePopUp synchronizeTitleAndSelectedItem];
}


//
//
//
- (void) saveChanges
{
  // We synchronize our popup buttons
  [headerNamePopUp synchronizeTitleAndSelectedItem];
  [headerNameSizePopUp synchronizeTitleAndSelectedItem];
  [headerValuePopUp synchronizeTitleAndSelectedItem];
  [headerValueSizePopUp synchronizeTitleAndSelectedItem];
  [messagePopUp synchronizeTitleAndSelectedItem];
  [messageSizePopUp synchronizeTitleAndSelectedItem];
  [plainTextMessagePopUp synchronizeTitleAndSelectedItem];
  [plainTextMessageSizePopUp synchronizeTitleAndSelectedItem];

  [[NSUserDefaults standardUserDefaults] setObject: [headerNamePopUp titleOfSelectedItem]
                                         forKey: @"HEADER_NAME_FONT_NAME"];
  [[NSUserDefaults standardUserDefaults] setObject: [headerNameSizePopUp titleOfSelectedItem]
                                         forKey: @"HEADER_NAME_FONT_SIZE"];
  
  [[NSUserDefaults standardUserDefaults] setObject: [headerValuePopUp titleOfSelectedItem]
                                         forKey: @"HEADER_VALUE_FONT_NAME"];
  [[NSUserDefaults standardUserDefaults] setObject: [headerValueSizePopUp titleOfSelectedItem]
                                         forKey: @"HEADER_VALUE_FONT_SIZE"];

  [[NSUserDefaults standardUserDefaults] setObject: [messagePopUp titleOfSelectedItem]
                                         forKey: @"MESSAGE_FONT_NAME"];
  [[NSUserDefaults standardUserDefaults] setObject: [messageSizePopUp titleOfSelectedItem]
                                         forKey: @"MESSAGE_FONT_SIZE"];

  [[NSUserDefaults standardUserDefaults] setInteger: [checkbox state]
					 forKey: @"USE_FIXED_FONT_FOR_TEXT_PLAIN_MESSAGES"];

  [[NSUserDefaults standardUserDefaults] setObject: [plainTextMessagePopUp titleOfSelectedItem]
					 forKey: @"PLAIN_TEXT_MESSAGE_FONT_NAME"];
  [[NSUserDefaults standardUserDefaults] setObject: [plainTextMessageSizePopUp titleOfSelectedItem]
                                         forKey: @"PLAIN_TEXT_MESSAGE_FONT_SIZE"];
}

//
// class methods
//
+ (id) singleInstance
{
  if ( !singleInstance )
    {
      singleInstance = [[FontsViewController alloc] initWithNibName: @"FontsView"];
    }

  return singleInstance;
}

@end


//
// Private methods
//
@implementation FontsViewController (Private)

- (void) _initializePopUpButtons
{
  NSMutableArray *availableFontFamilies;

  availableFontFamilies = [[NSMutableArray alloc] initWithArray: [[NSFontManager sharedFontManager] availableFontFamilies]];
  [availableFontFamilies sortUsingSelector: @selector(compare:)];

  [headerNamePopUp removeAllItems];
  [headerValuePopUp removeAllItems];
  [messagePopUp removeAllItems];
  [plainTextMessagePopUp removeAllItems];

  [headerNamePopUp addItemsWithTitles: availableFontFamilies];
  [headerValuePopUp addItemsWithTitles: availableFontFamilies];
  [messagePopUp addItemsWithTitles: availableFontFamilies];
  [plainTextMessagePopUp addItemsWithTitles: availableFontFamilies];
  
  RELEASE(availableFontFamilies);
}

@end
