/*
**  MailboxesView.m
**
**  Copyright (c) 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>   
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#import <AppKit/AppKit.h>
#import "MailboxesView.h"

#import "GNUMailConstants.h"
#import "LabelWidget.h"

@implementation MailboxesView

- (id) initWithParent: (id) theParent
{
  self = [super init];

  parent = theParent;

  return self;
}


//
//
//
- (void) dealloc
{
  NSDebugLog(@"MailboxesView: -dealloc");

  RELEASE(compactMailboxButton);
  RELEASE(reuseMailWindowButton);
  
  RELEASE(defaultInboxPopUpButton);
  RELEASE(defaultOutboxPopUpButton);
  RELEASE(defaultDraftsPopUpButton);
  RELEASE(defaultTrashPopUpButton);

  [super dealloc];
}


//
//
//
- (void) layoutView
{
  LabelWidget *label, *defaultInboxLabel, *defaultOutboxLabel, *defaultDraftsLabel, *defaultTrashLabel;
  
  label =  [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 325, 300, TextFieldHeight)
			label: _(@"Default mailbox used for:")];
  [self addSubview: label];
  
  //
  // Inbox
  //
  defaultInboxLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 290, 50, TextFieldHeight)
				   label: _(@"Inbox:")
				   alignment: NSRightTextAlignment];
  [self addSubview: defaultInboxLabel];
  
  defaultInboxPopUpButton = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(65, 290, 250, ButtonHeight)];
  [defaultInboxPopUpButton addItemWithTitle: @""];
  [self addSubview: defaultInboxPopUpButton];
  

  //
  // Outbox
  //
  defaultOutboxLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 255, 50, TextFieldHeight)
				    label: _(@"Outbox:")
				    alignment: NSRightTextAlignment];
  [self addSubview: defaultOutboxLabel];
  
  defaultOutboxPopUpButton = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(65, 255, 250, ButtonHeight)];
  [defaultOutboxPopUpButton addItemWithTitle: @""];
  [self addSubview: defaultOutboxPopUpButton];


  //
  // Drafts
  //
  defaultDraftsLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 215, 50, TextFieldHeight)
				    label: _(@"Drafts:")
				    alignment: NSRightTextAlignment];
  [self addSubview: defaultDraftsLabel];
  
  defaultDraftsPopUpButton = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(65, 215, 250, ButtonHeight)];
  [defaultDraftsPopUpButton addItemWithTitle: @""];
  [self addSubview: defaultDraftsPopUpButton];


  //
  // Trash
  //
  defaultTrashLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 180, 50, TextFieldHeight)
				   label: _(@"Trash:")
				   alignment: NSRightTextAlignment];
  [self addSubview: defaultTrashLabel];
 
  defaultTrashPopUpButton = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(65, 180, 250, ButtonHeight)];
  [defaultTrashPopUpButton addItemWithTitle: @""];
  [self addSubview: defaultTrashPopUpButton];
  
  
  //
  // label
  //
  label =  [LabelWidget labelWidgetWithFrame: NSMakeRect(5, 150, 430, TextFieldHeight)
			label: _(@"(If you don't see all your IMAP mailboxes, first open the connection to this IMAP server.)")];
  [label setFont: [NSFont systemFontOfSize: 10]];
  [self addSubview: label];
  

  //
  // Compact mailbox button
  //
  compactMailboxButton = [[NSButton alloc] initWithFrame: NSMakeRect(5,55,420,ButtonHeight)];
  [compactMailboxButton setButtonType: NSSwitchButton];
  [compactMailboxButton setBordered: NO];
  [compactMailboxButton setTitle: _(@"Compact mailbox when closing (not recommended)")];
  [self addSubview: compactMailboxButton];
  

  //
  // Reuse MailWindow button
  //
  reuseMailWindowButton = [[NSButton alloc] initWithFrame: NSMakeRect(5,25,420,ButtonHeight)];
  [reuseMailWindowButton setButtonType: NSSwitchButton];
  [reuseMailWindowButton setBordered: NO];
  [reuseMailWindowButton setTitle: _(@"Reuse the mail window when selecting a folder")];
  [self addSubview: reuseMailWindowButton];

}


//
// access / mutation methods
// 
- (NSButton *) compactMailboxButton
{
  return compactMailboxButton;
}

- (NSButton *) reuseMailWindowButton
{
  return reuseMailWindowButton;
}

- (NSPopUpButton *) defaultInboxPopUpButton
{
  return defaultInboxPopUpButton;
}

- (NSPopUpButton *) defaultOutboxPopUpButton
{
  return defaultOutboxPopUpButton;
}

- (NSPopUpButton *) defaultDraftsPopUpButton
{
  return defaultDraftsPopUpButton;
}

- (NSPopUpButton *) defaultTrashPopUpButton
{
  return defaultTrashPopUpButton;
}

@end
