/*
**  PGPController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "PGPController.h"
#import "PGPViewController.h"

// GNUMail.app headers
#import "GNUMailConstants.h"
#import "PasswordPanelController.h"

// Pantomime headers
#import <Pantomime/Constants.h>
#import <Pantomime/InternetAddress.h>
#import <Pantomime/NSDataExtensions.h>
#import <Pantomime/Message.h>
#import <Pantomime/MimeBodyPart.h>
#import <Pantomime/MimeMultipart.h>
#import <Pantomime/MimeUtility.h>
#import <Pantomime/Parser.h>
#import <Pantomime/Part.h>

static PGPController *singleInstance = nil;

@implementation PGPController

//
//
//
- (id) initWithOwner: (id) theOwner
{
  NSBundle *aBundle;
  
  self = [super init];

  owner = theOwner;
 
  aBundle = [NSBundle bundleForClass: [self class]];
  
  resourcePath = [aBundle resourcePath];
  RETAIN(resourcePath);

  // We..
  passphraseCache = [[NSMutableDictionary alloc] init];

  
  // FIXME: must adapt to work with > 1 viewing view
  imageCell = [[NSImageCell alloc] init];
  [imageCell setImage: nil];
  [imageCell setImageScaling: NSScaleToFit];

  return self;
}


//
//
//
- (void) dealloc
{
  RELEASE(resourcePath);
  
  RELEASE(encrypt);
  RELEASE(sign);
  
  RELEASE(passphraseCache);

  RELEASE(imageCell); // FIXME

  [super dealloc];
}


//
//
//
+ (id) singleInstance
{
  NSDebugLog(@"PGPController: -singleInstance");

  if ( !singleInstance )
    {
      singleInstance = [[PGPController alloc] initWithOwner: nil];
    }

  return singleInstance;
}


//
// access / mutation methods
//
- (NSString *) name
{
  return @"PGP";
}


//
//
//
- (NSString *) description
{
  return @"This is a simple PGP bundle.";
}


//
//
//
- (NSString *) version
{
  return @"v0.2.0";
}


//
//
//
- (void) setOwner: (id) theOwner
{
  owner = theOwner;
}


//
// UI elements
//
- (BOOL) hasPreferencesPanel
{
  return YES;
}


//
//
//
- (PreferencesModule *) preferencesModule
{
  return [PGPViewController singleInstance];
}


//
//
//
- (BOOL) hasComposeViewAccessory
{
  return YES;
}


//
//
//
- (id) composeViewAccessory
{
  NSImage *icon;
  NSView *aView;
  
#ifdef MACOSX
  aView = [[NSView alloc] initWithFrame: NSMakeRect(0,0,68,32)];
#else
  aView = [[NSView alloc] initWithFrame: NSMakeRect(0,0,136,64)];
#endif

  //
  // Encrypt / clear button
  //
#ifdef MACOSX
  encrypt = [[NSButton alloc] initWithFrame: NSMakeRect(0,0,32,32)];
  [encrypt setImagePosition: NSImageOnly];
  [encrypt setBordered: NO];
  icon = [[NSImage alloc] initWithContentsOfFile: [NSString stringWithFormat: @"%@/clear_20.tiff", resourcePath]];
  [encrypt setImage: icon];
  RELEASE(icon);
#else
  encrypt = [[NSButton alloc] initWithFrame: NSMakeRect(0,0,64,64)];
  [encrypt setTitle: _(@"Encrypt")];
  [encrypt setFont: [NSFont systemFontOfSize: 8]];
  [encrypt setImagePosition: NSImageAbove];
  icon = [[NSImage alloc] initWithContentsOfFile: [NSString stringWithFormat: @"%@/clear_48.tiff", resourcePath]];
  [encrypt setImage: icon];
  RELEASE(icon);
#endif
  [encrypt setTarget: self];
  [encrypt setAction: @selector(encryptClicked:)];
  [encrypt setTag: NOT_ENCRYPTED];

  [aView addSubview: encrypt];
  
  //
  // Signed / Unsigned button
  //
#ifdef MACOSX
  sign = [[NSButton alloc] initWithFrame: NSMakeRect(36,0,32,32)];
  [sign setImagePosition: NSImageOnly];
  [sign setBordered: NO];
  icon = [[NSImage alloc] initWithContentsOfFile: [NSString stringWithFormat: @"%@/unsigned_20.tiff", resourcePath]];
  [sign setImage: icon];
  RELEASE(icon);
#else
  sign = [[NSButton alloc] initWithFrame: NSMakeRect(72,0,64,64)];
  [sign setTitle: _(@"Sign")];
  [sign setFont: [NSFont systemFontOfSize: 8]];
  [sign setImagePosition: NSImageAbove];
  icon = [[NSImage alloc] initWithContentsOfFile: [NSString stringWithFormat: @"%@/unsigned_48.tiff", resourcePath]];
  [sign setImage: icon];
  RELEASE(icon);
#endif
  [sign setTarget: self];
  [sign setAction: @selector(signClicked:)];
  [sign setTag: NOT_SIGNED];

  [aView addSubview: sign];

  return aView;
}


//
//
//
- (BOOL) hasViewingViewAccessory
{
  return YES;
}


//
//
//
- (id) viewingViewAccessory
{  
  return imageCell;
}


//
//
//
- (enum ViewingViewType) viewingViewAccessoryType
{
  return ViewingViewTypeStatusBar;
}

//
//
//
- (void) viewingViewAccessoryWillBeRemovedFromSuperview: (NSView *) theView
{

}


//
//
//
- (void) setCurrentSuperview: (NSView *) theView
{
  superview = theView;
}


//
//
//
- (NSArray *) submenuForMenu: (NSMenu *) theMenu
{
  return nil;
}


//
//
//
- (NSArray *) menuItemsForMenu: (NSMenu *) theMenu
{
  return nil;
}



//
// action methods
//
- (IBAction) encryptClicked: (id) sender
{
  NSImage *icon;

  if ( [encrypt tag] == NOT_ENCRYPTED )
    {
      [encrypt setTag: ENCRYPTED];

#ifdef MACOSX      
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/encrypted_20.tiff", resourcePath]];
#else
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/encrypted_48.tiff", resourcePath]];
#endif
      [encrypt setImage: icon];
      RELEASE(icon);
    }
  else
    {
      [encrypt setTag: NOT_ENCRYPTED];

#ifdef MACOSX
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/clear_20.tiff", resourcePath]];
#else
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/clear_48.tiff", resourcePath]];
#endif
      [encrypt setImage: icon];
      RELEASE(icon);
    }
}


//
//
//
- (IBAction) signClicked: (id) sender
{
  NSImage *icon;
  
  if ( [sign tag] == NOT_SIGNED )
    {
      [sign setTag: SIGNED];

#ifdef MACOSX
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/signed_20.tiff", resourcePath]];
#else
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/signed_48.tiff", resourcePath]];
#endif
      [sign setImage: icon];
      RELEASE(icon);
    }
  else
    {
      [sign setTag: NOT_SIGNED];

#ifdef MACOSX
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/unsigned_20.tiff", resourcePath]];
#else
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/unsigned_48.tiff", resourcePath]];
#endif

      [sign setImage: icon];
      RELEASE(icon);
    }
}




//
// Pantomime related methods
//
- (Message *) messageWasEncoded: (Message *) theMessage
{  
  Message *aMessage;

  // We first verify if we must at least sign OR encrypt
  if ( [sign tag] == NOT_SIGNED &&
       [encrypt tag] == NOT_ENCRYPTED )
    {
      return theMessage;
    }
  
  // If our content isn't a multipart
  if ( [theMessage isMimeType: @"text" : @"*"] )
    {
      aMessage = [self _encryptMessage: theMessage
		       multipart: NO];
    }
  else
    {
      // We got a multipart content!
      aMessage = [self _encryptMessage: theMessage
		       multipart: YES];
    }

  return aMessage;
}


//
//
//
- (void) messageWasDisplayed: (Message *) theMessage
                      inView: (NSTextView *) theTextView
{
  NSImage *icon;
  
  // Set the icon in MailWindow to inform that this message was
  // PGP encrypted.
  if ( [theMessage extraPropertyForKey: @"CONTENT-STATUS"] &&
       [[theMessage extraPropertyForKey: @"CONTENT-STATUS"] intValue] == ENCRYPTED )
    {
      NSDebugLog(@"setting encrypted logo!");
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/encrypted_20.tiff", resourcePath]];
      
    }
  else if ( [theMessage extraPropertyForKey: @"CONTENT-STATUS"] &&
	    [[theMessage extraPropertyForKey: @"CONTENT-STATUS"] intValue] == SIGNED )
    {
      NSDebugLog(@"setting signed logo!");
      icon = [[NSImage alloc] initWithContentsOfFile: 
				[NSString stringWithFormat: @"%@/signed_20.tiff", resourcePath]];
    }
  else
    {
      icon = nil;
    }
  
  [imageCell setImage: icon];
  
  if ( icon ) RELEASE(icon);
}


//
// FIXME: Consider the Content-Type's protocol!
//
- (void) messageWillBeDisplayed: (Message *) theMessage
                         inView: (NSTextView *) theTextView
{
  // VERIFY IF:
  //             multipart/encrypted
  //
  //             contains exactly TWO parts: application/pgp-encrypted
  //                                         application/octet-stream 
  //
  //             multipart/signed
  //             
  if ( [theMessage isMimeType: @"text" : @"plain"] )
    {
      if ( [(NSString *)[theMessage content] hasPrefix: @"-----BEGIN PGP MESSAGE-----"] || 
	   [(NSString *)[theMessage content] hasPrefix: @"-----BEGIN PGP SIGNED MESSAGE-----"] )
	{
	  [self _decryptPart: theMessage
		multipart: NO
		message: theMessage];
	}
      else
	{
	  [imageCell setImage: nil];
	}
    }
  else if ( [theMessage isMimeType: @"multipart" : @"encrypted"] )
    {
      MimeMultipart *aMimeMultipart;
      int i;
     
      // We search for our octect-stream part.
      aMimeMultipart = (MimeMultipart *)[theMessage content];

      for (i = ([aMimeMultipart count] - 1); i >= 0; i--)
	{
	  MimeBodyPart *aMimeBodyPart;
	  
	  aMimeBodyPart = [aMimeMultipart bodyPartAtIndex: i];
       
	  if ( [aMimeBodyPart isMimeType: @"application" : @"octet-stream"] )
	    {
	      [self _decryptPart: aMimeBodyPart
		    multipart: YES
		    message: theMessage];
	    }
	  else if ( [aMimeBodyPart isMimeType: @"application" : @"pgp-encrypted"] )
	    {
	      [aMimeMultipart removeBodyPart: aMimeBodyPart];
	    }
	}
    }
}

@end



//
// Private methods
//
@implementation PGPController (Private)

//
// When this method is called, all results to --status-fd=2 (stderr)
// have been produced by the GPG task. So, we simply read it,
// and toss the irrelevant information.
//
- (BOOL) _analyseTaskOutput: (NSMutableData *) theMutableData
		     userID: (NSString *) theUserID 
		    message: (NSMutableString *) theMessage
{
  NSArray *allLines;
  BOOL aBOOL;
  int i;

  allLines = [theMutableData componentsSeparatedByCString: "\n"];
  aBOOL = YES;

  for (i = 0; i < [allLines count]; i++)
    {
      if ( [[allLines objectAtIndex: i] hasCPrefix: "[GNUPG:] "] )
	{
	  NSString *aString;

	  aString = [[NSString alloc] initWithData: [[allLines objectAtIndex: i] subdataFromIndex: 9]
				      encoding: NSASCIIStringEncoding];

	  
	  
	  NSDebugLog(@"READ = |%@|", aString);

	  // We analyse...
	  if ( [aString hasPrefix: @"BAD_PASSPHRASE"] )
	    {
	      [theMessage appendString: _(@"\nThe supplied passphrase was wrong or not given")];
	      aBOOL = NO;
	      break;
	    }
	  else if ( [aString hasPrefix: @"DECRYPTION_FAILED"] )
	    {
	      [theMessage appendString: _(@"\nWrong passphrase (or something else)")];
	      aBOOL = NO;
	      break;
	    }
	  else if ( [aString hasPrefix: @"SIGEXPIRED"] ||  [aString hasPrefix: @"KEYEXPIRED"] )
	    {
	      [theMessage appendString: _(@"\nYour key is expired. You must generate a new one\nbefore trying to send again any signed or encrypted messages.")];
	      aBOOL = NO;
	      break;
	    }
	  
	  RELEASE(aString);
	} // if ( [[allLines objectAtIndex: i] hasCPrefix: "[GNUPG:] "] )
      else
	{
	  [theMessage appendFormat: @"\n%@", [[allLines objectAtIndex: i] asciiString]];
	}
    }
  
  return aBOOL;
}


//
// GPG commands:
//
//  --batch  Use batch mode.  Never ask, do not allow  inter
//           active commands.
//
//  --status-fd n
//           Write   special   status  strings  to  the  file
//           descriptor n.  See the file DETAILS in the docu
//           mentation for a listing of them.
//
//  --passphrase-fd n
//           Read the passphrase from file descriptor  n.  If
//           you  use  0  for  n, the passphrase will be read
//           from stdin.     This can only be  used  if  only
//           one  passphrase  is  supplied.   Don't  use this
//           option if you can avoid it.
//
- (void) _decryptPart: (Part *) thePart
	    multipart: (BOOL) aBOOL
	      message: (Message *) theMessage

{	  
  NSPipe *standardInput, *standardError;
  NSTask *aTask;
  
  NSString *aLaunchPath, *aString, *inFilename, *outFilename, *aPassphrase, *aUserID;
  char *s1, *s2;

  NSMutableString *aWarningMessage;
  NSMutableData *aMutableData;
  NSMutableArray *arguments;

  BOOL isOnlySigned;
  
  // We generate tempory filenames
  s1 = NULL;
  inFilename = nil;
  
  s2 = tempnam([NSTemporaryDirectory() cString], NULL);
  outFilename = [NSString stringWithFormat: @"%s.out", s2];
  
  //
  // We get our User ID (E-Mail address only). We use it to obtain the GPG passphrase:
  //
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"PGPBUNDLE_USE_FROM_FOR_SIGNING"] ||
       [[NSUserDefaults standardUserDefaults] integerForKey: @"PGPBUNDLE_USE_FROM_FOR_SIGNING"] == NSOnState )
    {
      aUserID = [[theMessage from] address];
    }
  else
    {
      aUserID = [[NSUserDefaults standardUserDefaults] stringForKey: @"PGPBUNDLE_USER_EMAIL_ADDRESS"];
    }
  

  // We set the right property, depending on the content's prefix
  if ( [(NSString *)[thePart content] hasPrefix: @"-----BEGIN PGP MESSAGE-----"] )
    {
      [theMessage setExtraProperty: [NSNumber numberWithInt: ENCRYPTED]
      		  forKey: @"CONTENT-STATUS"];
      isOnlySigned = NO;
    }
  else
    {
      [theMessage setExtraProperty: [NSNumber numberWithInt: SIGNED]
      		  forKey: @"CONTENT-STATUS"];
      isOnlySigned = YES;
    }
  

  // We first verify our launch path
  // FIXME: Verify if file exist & is executable
  if ( [[NSUserDefaults standardUserDefaults] stringForKey: @"PGPBUNDLE_GPG_PATH"] )
    {
      aLaunchPath = [[NSUserDefaults standardUserDefaults] stringForKey: @"PGPBUNDLE_GPG_PATH"];
    }
  else
    {
#ifdef MACOSX
      aLaunchPath = @"/usr/local/bin/gpg";
#else
      aLaunchPath = @"/usr/bin/gpg";
#endif
    }

  // We create our task object
  aTask = [[NSTask alloc] init];
  [aTask setLaunchPath: aLaunchPath];  
  
  //
  // We initialize our basic arguments
  //
  arguments = [[NSMutableArray alloc] initWithObjects:  @"--batch", 
				      @"--no-tty", 
				      @"--status-fd",
				      @"2",
				      @"-o",
				      outFilename,
				      nil];
  
  if ( !isOnlySigned )
    {
      // We generate our input's filename
      s1 = tempnam([NSTemporaryDirectory() cString], NULL);
      inFilename = [NSString stringWithCString: s1];

      // We write our mail content to a temporary file
      [(NSString *)[thePart content] writeToFile: inFilename  atomically: YES];
      
      [arguments addObject: @"--passphrase-fd"];
      [arguments addObject: @"0"];
      [arguments addObject: @"--decrypt"];
      [arguments addObject: inFilename];

      // We create our standard input handle pipe
      standardInput = [NSPipe pipe];
      
      // We write our passphrase on the pipe
      aPassphrase = [self _passphraseForID: aUserID];  // ex: ludovic@Sophos.ca
      [[standardInput fileHandleForWriting] writeData:
					      [aPassphrase dataUsingEncoding:
							     NSASCIIStringEncoding]];
      [[standardInput fileHandleForWriting] closeFile];
      
      // We set the stdin / stdout
      [aTask setStandardInput: standardInput];
    }
  else
    {
      // We create our standard input handle pipe
      standardInput = [NSPipe pipe];

      [[standardInput fileHandleForWriting] writeData:
					      [(NSString *)[thePart content] dataUsingEncoding:
							     NSASCIIStringEncoding]];
      [[standardInput fileHandleForWriting] closeFile];
      
      // We set the stdin / stdout
      [aTask setStandardInput: standardInput];
    }
  
  // We set our task's standard error
  standardError = [NSPipe pipe];
  [aTask setStandardError: standardError];
  
  // We set our task's arguments
  [aTask setArguments: arguments];
  RELEASE(arguments);

  // We create our mutable data and string object
  aWarningMessage = [[NSMutableString alloc]
		      initWithString: _(@"Decryption failed due the following reason(s):\n")];
  aMutableData = [[NSMutableData alloc] init];

  // We lauch our task
  [aTask launch];
  
  // While the task is dunning, we accumulate the infoz read on stderr
  // into aMutableData.
  while ( [aTask isRunning] )
    {
      NSData *aData;
      
      aData = [[standardError fileHandleForReading] availableData];
      [aMutableData appendData: aData];
    }
  
  
  // We analyse the output of our GPG task to stderr. If it the decryption
  // failed, we do some cleanups.
  if ( ![self _analyseTaskOutput: aMutableData 
	      userID: aUserID
	      message: aWarningMessage] )
    {
      // We show the reason why decryption failed
      NSRunAlertPanel(_(@"Error!"),
		      aWarningMessage,
                      _(@"OK"),   // default
                      NULL,       // alternate
                      NULL);
      

      // FIXME: move to the _analyseTaskOutput:: method
      // We remove the passphrase for this ID
      [passphraseCache removeObjectForKey: aUserID];
      
      // We remove our temporary files and we free some vars
      [[NSFileManager defaultManager] removeFileAtPath: inFilename
				      handler: nil];      
      [[NSFileManager defaultManager] removeFileAtPath: outFilename
				      handler: nil];     
      free(s1);
      free(s2);
      
      RELEASE(aMutableData);
      RELEASE(aWarningMessage);
      RELEASE(aTask);
      
      return;
    }

  // 
  RELEASE(aMutableData);
  RELEASE(aWarningMessage);

  //
  // We set the new content of the message
  //
  if ( !aBOOL )
    {
      // FIXME - Not 8-bit clean
      aString = [NSString stringWithContentsOfFile: outFilename];
      
      [thePart setContent: aString];
    }
  else
    {
      NSMutableData *aMutableData;
      NSData *aData;
      NSRange aRange;

      // We replace all occurences of \r\n by \n
      aMutableData = [[NSMutableData alloc] initWithData: [NSData dataWithContentsOfFile: outFilename]];
      [self _replaceCRLFInMutableData: aMutableData];
      
      // We unfold all lines
      aData = [MimeUtility unfoldLinesFromData: aMutableData];
      
      // We grab only the headers and we parse them
      aRange = [aData rangeOfCString: "\n"];

      // We set the Content-Type & boundary
      [Parser parseContentType: [aData subdataToIndex: aRange.location]
	      inPart: thePart];
      
      // We set the RAW content
      [MimeUtility setContentFromRawSource: [aData subdataFromIndex: (aRange.location + 1)]
		   inPart: thePart];
    }
  
  // Cleanups
  if ( !isOnlySigned )
    {
      free(s1);

      [[NSFileManager defaultManager] removeFileAtPath: inFilename
				      handler: nil];      
    }

  [[NSFileManager defaultManager] removeFileAtPath: outFilename
  				  handler: nil];
 
  free(s2);
  
  RELEASE(aTask);
}




//
//
// GPG commands:
//
// SIGN ONLY to ludovic@Sophos.ca: OR
// SIGN ONLY to julie.jolin@sympatico.ca
//
// /usr/bin/gpg --batch --no-tty --status-fd 2
// --comment 'Using GnuPG with Mozilla - http://enigmail.mozdev.org'
// --always-trust --encrypt-to ludovic@Sophos.ca --clearsign -u ludovic@Sophos.ca --passphrase-fd 0
//
//
// ENCRYPT ONLY to ludovic@Sophos.ca
// 
// To encrypt, we must have the public key of -r
//
//
// /usr/bin/gpg --batch --no-tty --status-fd 2
// --comment 'Using GnuPG with Mozilla - http://enigmail.mozdev.org' 
// --always-trust --encrypt-to ludovic@Sophos.ca -a -e -u ludovic@Sophos.ca -r ludovic@Sophos.ca
//
//
// ENCRYPT ONLY to fabien.vallon@fr.alcove.com
//
// /usr/bin/gpg --batch --no-tty --status-fd 2 
// --comment 'Using GnuPG with Mozilla - http://enigmail.mozdev.org' 
// --always-trust --encrypt-to ludovic@Sophos.ca -a -e -u ludovic@Sophos.ca -r fabien.vallon@fr.alcove.com
//
//
//  -a, --armor
//               Create ASCII armored output.
//
//  -e, --encrypt
//               Encrypt data. This option may be  combined  with
//               --sign.
//
//
// ENCRYPT AND SIGN to ludovic@Sophos.ca
//
// /usr/bin/gpg --batch --no-tty --status-fd 2 
// --comment 'Using GnuPG with Mozilla - http://enigmail.mozdev.org'
// --always-trust --encrypt-to ludovic@Sophos.ca -a -e -s -u ludovic@Sophos.ca -r ludovic@Sophos.ca 
// --passphrase-fd 0
//
// FIXME: Should return the new message instead of modifying directly :)
//
- (Message *) _encryptMessage: (Message *) theMessage
		    multipart: (BOOL) aBOOL
{
  NSPipe *standardInput, *standardError;
  NSTask *aTask;
   
  NSString *aLaunchPath, *aUserID, *aRecipientUserID, *outFilename, *aPassphrase;

  NSMutableString *aWarningMessage;
  NSMutableData *aMutableData;
 

  NSMutableArray *arguments;
  NSArray *allRecipients;
  char *s1;
  int i;

  // We generate our filename  
  s1 = tempnam([NSTemporaryDirectory() cString], NULL);
  outFilename = [NSString stringWithCString: s1];

  // We get our User ID (E-Mail address only). We use it for the following GPG parameters
  // (and also to obtain the GPG passphrase):
  // 
  // --encrypt-to
  // -u
  //
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"PGPBUNDLE_USE_FROM_FOR_SIGNING"] ||
       [[NSUserDefaults standardUserDefaults] integerForKey: @"PGPBUNDLE_USE_FROM_FOR_SIGNING"] == NSOnState )
    {
      aUserID = [[theMessage from] address];
    }
  else
    {
      aUserID = [[NSUserDefaults standardUserDefaults] stringForKey: @"PGPBUNDLE_USER_EMAIL_ADDRESS"];
    }
  
  // We get our recipient User ID
  // FIXME: Support more than 1 recipient
  allRecipients = [theMessage recipients];
  aRecipientUserID = nil;

  for (i = 0; i < [allRecipients count]; i++)
    {
      InternetAddress *aInternetAddress;

      aInternetAddress = [allRecipients objectAtIndex: i];

      if ( [aInternetAddress type] == TO )
	{
	  aRecipientUserID = [aInternetAddress address];
	  break;
	}
    }
 
  
  // We first verify our launch path
  // FIXME: Verify if file exist & is executable
  if ( [[NSUserDefaults standardUserDefaults] stringForKey: @"PGPBUNDLE_GPG_PATH"] )
    {
      aLaunchPath = [[NSUserDefaults standardUserDefaults] stringForKey: @"PGPBUNDLE_GPG_PATH"];
    }
  else
    {
#ifdef MACOSX
      aLaunchPath = @"/usr/local/bin/gpg";
#else
      aLaunchPath = @"/usr/bin/gpg";
#endif
    }
  
  // We create our task object
  aTask = [[NSTask alloc] init];
  [aTask setLaunchPath: aLaunchPath];

  // Let's create our array of arguments
  arguments = [[NSMutableArray alloc] initWithObjects: @"--batch", 
				      @"--no-tty", 
				      @"--status-fd",
				      @"2",
				      @"--comment",
				      @"Using the GPG bundle for GNUMail.app by Ludovic Marcotte",
				      @"--always-trust", 
				      @"--encrypt-to",
				      aUserID,         // ex: ludovic@Sophos.ca
				      nil];
  
  //
  // If we sign ONLY
  //
  if ( [sign tag] == SIGNED && 
       [encrypt tag] == NOT_ENCRYPTED )
    {
      [arguments addObject: @"--clearsign"];
    }
  //
  // If we encrypt ONLY
  //
  else if ( [sign tag] == NOT_SIGNED && 
	    [encrypt tag] == ENCRYPTED )
    {
      [arguments addObject: @"-a"];  // ASCII armored output
      [arguments addObject: @"-e"];  // encrypt
      [arguments addObject: @"-r"];  // recipient user-id    
      [arguments addObject: aRecipientUserID];
    }
  //
  // If we BOTH sign AND encrypt
  //
  else if ( [sign tag] == SIGNED && 
	    [encrypt tag] == ENCRYPTED )
    {
      [arguments addObject: @"-a"];  // ASCII armored output
      [arguments addObject: @"-e"];  // encrypt
      [arguments addObject: @"-s"];  // and sign
      [arguments addObject: @"-r"];  // recipient user-id    
      [arguments addObject: aRecipientUserID];
    }
  
  // Last standard arguments
  //
  [arguments addObject: @"-u"];
  [arguments addObject: aUserID]; // ex: ludovic@Sophos.ca
  [arguments addObject: @"--passphrase-fd"];
  [arguments addObject: @"0"];
  [arguments addObject: @"-o"];
  [arguments addObject: outFilename];
  
  [aTask setArguments: arguments];
  RELEASE(arguments);
	  

  // We create our standard input handle pipe and we set the stdin
  standardInput = [NSPipe pipe];
  [aTask setStandardInput: standardInput];
  
  // We set our task's standard error
  standardError = [NSPipe pipe];
  [aTask setStandardError: standardError];

  // We obtain our passphrase 
  aPassphrase = [self _passphraseForID: aUserID];    // ex: ludovic@Sophos.ca

  // We lauch our task
  [aTask launch];

  // We write everything to the pipe
  [[standardInput fileHandleForWriting] writeData:
					  [aPassphrase dataUsingEncoding:
							 NSASCIIStringEncoding]];
  // We 'flush' the passphrase by writing a \n
  [[standardInput fileHandleForWriting] writeData: 
					  [NSData dataWithBytes: "\n"  length: 1]];

  // We write our part content
  if ( !aBOOL )
    {
      [[standardInput fileHandleForWriting] writeData:
					      [(NSString *)[theMessage content] dataUsingEncoding:
							     NSASCIIStringEncoding]]; 
    }
  else
    {
      // We generate our raw data of the "Part"'s part of our Message
      NSMutableData *rawSourceOfPart;
      NSData *aData;
      NSRange aRange;

      rawSourceOfPart = [[NSMutableData alloc] init];

      // We add our Content-Type: abc/def; boundary="--foo" to our message
      [rawSourceOfPart appendCFormat: @"Content-Type: %@;\n", [theMessage contentType]];
      [rawSourceOfPart appendCFormat: @"\tboundary=\""];
      [rawSourceOfPart appendData: [theMessage boundary]];
      [rawSourceOfPart appendCFormat: @"\"\n\n"];

      // We append our message content
      aData = [theMessage dataUsingSendingMode: SEND_TO_FOLDER];
      aRange = [aData rangeOfCString: "\n\n"];
      aData = [aData subdataFromIndex: (aRange.location + 2)];
      
      [rawSourceOfPart appendData: aData];

      // We write the data to our pipe
      [[standardInput fileHandleForWriting] writeData: rawSourceOfPart];
      RELEASE(rawSourceOfPart);
    }

  [[standardInput fileHandleForWriting] closeFile];

  // We create our mutable data and string object
  aWarningMessage = [[NSMutableString alloc]
		      initWithString: _(@"Encryption failed due the following reason(s):\n")];
  aMutableData = [[NSMutableData alloc] init];

  
  // While the task is dunning, we accumulate the infoz read on stderr
  // into aMutableData.
  while ( [aTask isRunning] )
    {
      NSData *aData;
      
      aData = [[standardError fileHandleForReading] availableData];
      [aMutableData appendData: aData];
    }
  
  // We analyse the output of our GPG task to stderr. If it the encryption
  // failed, we do some cleanups.
  if ( ![self _analyseTaskOutput: aMutableData 
	      userID: aUserID
	      message: aWarningMessage] ||
       [aTask terminationStatus] > 0 )
    {
      // We show the reason why decryption failed
      NSRunAlertPanel(_(@"Error!"),
		      aWarningMessage,
                      _(@"OK"),   // default
                      NULL,       // alternate
                      NULL);
      
      
      // FIXME: move to the _analyseTaskOutput:: method
      // We remove the passphrase for this ID
      [passphraseCache removeObjectForKey: aUserID];

      // We remove our temporary file and we free some vars
      [[NSFileManager defaultManager] removeFileAtPath: outFilename
				      handler: nil];
      
      free(s1);
      RELEASE(aWarningMessage);
      RELEASE(aMutableData);
      RELEASE(aTask);

      return nil;
    }

  RELEASE(aWarningMessage);
  RELEASE(aMutableData);
  
  // FIXME - We must consider the string encoding for 8-bit cleanness
  if ( !aBOOL) 
    {
      NSString *aString;

      aString = [NSString stringWithContentsOfFile: outFilename];
      
      [theMessage setContent: aString];
    }
  else
    {
      MimeMultipart *aMimeMultipart;
      MimeBodyPart *aMimeBodyPart;
      
      NSData *aBoundary;
      NSString *aString;

      // We create our new multipart object
      aMimeMultipart = [[MimeMultipart alloc] init];


      //
      // We add our extra part (Version 1)
      //
      aMimeBodyPart = [[MimeBodyPart alloc] init];
      [aMimeBodyPart setContentTransferEncoding: NONE];
      [aMimeBodyPart setContentType: @"application/pgp-encrypted"];
      [aMimeBodyPart setContent: @"Version 1"];
      [aMimeBodyPart setSize: 9];

      [aMimeMultipart addBodyPart: aMimeBodyPart];
      RELEASE(aMimeBodyPart);

      //
      // We add our encrypted part
      //
      // This is safe since our output is ASCII armored when encrypted
      aString = [NSString stringWithContentsOfFile: outFilename];

      aMimeBodyPart = [[MimeBodyPart alloc] init];
      [aMimeBodyPart setContentTransferEncoding: NONE];
      [aMimeBodyPart setContentType: @"application/octet-stream"];
      [aMimeBodyPart setContent: aString];
      [aMimeBodyPart setSize: [aString length]];

      [aMimeMultipart addBodyPart: aMimeBodyPart];
      RELEASE(aMimeBodyPart);

      // We generate a new boundary
      aBoundary = [MimeUtility generateBoundary];

      // We set the new boundary, the new Content-Type and the Content-Transfer-Encoding to our message
      [theMessage setBoundary: aBoundary];
      [theMessage setContentType: @"multipart/encrypted"];
      [theMessage setProtocol: [[NSString stringWithString: @"application/pgp-encrypted"]
				 dataUsingEncoding: NSASCIIStringEncoding]];
      [theMessage setContentTransferEncoding: NONE];


      // We finally set the new multipart content
      [theMessage setContent: aMimeMultipart];
      RELEASE(aMimeMultipart);
    }

  // Cleanups
  [[NSFileManager defaultManager] removeFileAtPath: outFilename
   				  handler: nil];
  
  free(s1);
  
  RELEASE(aTask);

  return theMessage;
}


//
// The ID is, for example: ludovic@Sophos.ca
//
- (NSString *) _passphraseForID: (NSString *) theID
{
  NSString *aPassphrase;
  
  // We first verify in our cache
  aPassphrase = [passphraseCache objectForKey: theID];
  
  // If we must prompt for the password
  if ( !aPassphrase )
    {
      PasswordPanelController *theController; 
      int result;
      
      theController = [[PasswordPanelController alloc] initWithWindowNibName: @"PasswordPanel"];
      [[theController window] setTitle: [NSString stringWithFormat: _(@"Passphrase for %@"),
						  theID]];
      
      result = [NSApp runModalForWindow: [theController window]];
      
      // If the user has entered a password...
      if (result == NSRunStoppedResponse)
	{
	  aPassphrase = [theController password];
	  
	  // Let's cache this password...
	  [passphraseCache setObject: aPassphrase
			   forKey: theID];
	}
      else
	{
	  aPassphrase = nil;
	}
      
      RELEASE(theController);
    }
  
  return aPassphrase;
}


//
//
//
- (void) _replaceCRLFInMutableData: (NSMutableData *) theMutableData
{
  unsigned char *bytes, *bi, *bo;
  int delta, i,length;
  
  bytes = [theMutableData mutableBytes];
  length = [theMutableData length];
  bi = bo = bytes;
  
  for (i = delta = 0; i < length; i++, bi++)
    {
      if ( i+1 < length && bi[0] == '\r' && bi[1] == '\n')
	{
	  i++;
	  bi++;
	  delta++;
	}
      
      *bo = *bi;
      bo++;
    }
  
  [theMutableData setLength: length-delta];
}

@end
