/*
**  IMAPView.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "IMAPView.h"

#import "GNUMailConstants.h"
#import "LabelWidget.h"
#import "Utilities.h"

const NSString *IMAPViewMailboxTableColumnIdentifier = @"all folders";
const NSString *IMAPViewSubscriptionTableColumnIdentifier = @"subscribed folders";

@implementation IMAPView

//
//
//
- (id) initWithParent: (id) theParent
{
  LabelWidget *retrieveMinutesLabel, *minutesLabel;
  NSButton *list, *subscribe, *unsubscribe;
  NSScrollView *scrollView;
   
  self = [super init];

  [self setFrame: NSMakeRect(0,0,350,225)];
  [self setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];

  retrieveMinutesLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5,185,130,TextFieldHeight)
				      label: _(@"Check for mail every")];
  [retrieveMinutesLabel setAutoresizingMask: NSViewMinYMargin];
  [self addSubview: retrieveMinutesLabel];
  
  retrieveMinutesField = [[NSTextField alloc] initWithFrame: NSMakeRect(140,185,130,TextFieldHeight)];
  [retrieveMinutesField setEditable: YES];
  [retrieveMinutesField setSelectable: YES];
  [retrieveMinutesField setAutoresizingMask: NSViewMinYMargin];
  [self addSubview: retrieveMinutesField];

  minutesLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(280,185,70,TextFieldHeight)
			      label: _(@"minutes")];
  [minutesLabel setAutoresizingMask: NSViewMinYMargin];
  [self addSubview: minutesLabel];
  
  mailboxColumn = [[NSTableColumn alloc] initWithIdentifier: (id) IMAPViewMailboxTableColumnIdentifier];
  [mailboxColumn setEditable: NO];
  [[mailboxColumn headerCell] setStringValue: _(@"Folders")];
  [mailboxColumn setMinWidth: 270];

  subscriptionColumn = [[NSTableColumn alloc] initWithIdentifier: (id) IMAPViewSubscriptionTableColumnIdentifier];
  [subscriptionColumn setEditable: NO];
  [[subscriptionColumn headerCell] setStringValue: _(@"Subscribed")];

  outlineView = [[NSOutlineView alloc] initWithFrame: NSMakeRect(5,40,340,125)];
  [outlineView addTableColumn: mailboxColumn];
  [outlineView addTableColumn: subscriptionColumn];
  [outlineView setOutlineTableColumn: mailboxColumn];
  [outlineView setDrawsGrid: NO];
  [outlineView setIndentationPerLevel: 10];
  [outlineView setAllowsColumnSelection: NO];
  [outlineView setAllowsColumnReordering: NO];
  [outlineView setAllowsEmptySelection: NO];
  [outlineView setAllowsMultipleSelection: NO];
  [outlineView setIndentationMarkerFollowsCell: YES];
  [outlineView setAutoresizesOutlineColumn: YES];
  [outlineView sizeLastColumnToFit];
  [outlineView setDataSource: theParent]; 
  [outlineView setDelegate: theParent];
  [outlineView setTarget: theParent];
  [outlineView setDoubleAction: @selector(doubleClickedOnNode:)];

  scrollView = [[NSScrollView alloc] initWithFrame: NSMakeRect(5,40,340,125)];
  [scrollView setHasHorizontalScroller: YES];
  [scrollView setHasVerticalScroller: YES];
  [scrollView setDocumentView: outlineView];
  [scrollView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [self addSubview: scrollView];
  RELEASE(scrollView);
  
  list = [[NSButton alloc] initWithFrame: NSMakeRect(5,5,100,ButtonHeight)];
  [list setTitle: _(@"List")];
  [list setTarget: theParent];
  [list setAction: @selector(list:)];
  [list setAutoresizingMask: NSViewMaxXMargin|NSViewMaxYMargin];
  [self addSubview: list];
  RELEASE(list);

  subscribe = [[NSButton alloc] initWithFrame: NSMakeRect(110,5,100,ButtonHeight)];
  [subscribe setTitle: _(@"Subscribe")];
  [subscribe setTarget: theParent];
  [subscribe setAction: @selector(subscribe:)];
  [subscribe setAutoresizingMask: NSViewMaxXMargin|NSViewMaxYMargin];
  [self addSubview: subscribe];
  RELEASE(subscribe);

  unsubscribe = [[NSButton alloc] initWithFrame: NSMakeRect(215,5,100,ButtonHeight)];
  [unsubscribe setTitle: _(@"Unsubscribe")];
  [unsubscribe setTarget: theParent];
  [unsubscribe setAction: @selector(unsubscribe:)];
  [unsubscribe setAutoresizingMask: NSViewMaxXMargin|NSViewMaxYMargin];
  [self addSubview: unsubscribe];
  RELEASE(unsubscribe);
     
  return self;
}


//
//
//
- (void) dealloc
{
  //NSDebugLog(@"IMAPView: -dealloc");
  
  RELEASE(mailboxColumn);
  RELEASE(subscriptionColumn);

  RELEASE(outlineView);
  
  [super dealloc];
}


//
//
//
- (NSTextField *) retrieveMinutesField
{
  return retrieveMinutesField;
}


//
//
//
- (NSOutlineView *) outlineView
{
  return outlineView;
}


//
//
//
- (NSTableColumn *) mailboxColumn
{
  return mailboxColumn;
}

- (NSTableColumn *) subscriptionColumn
{
  return subscriptionColumn;
}

@end
