/*
**  POP3ViewController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "POP3ViewController.h"

#import "FolderNode.h"
#import "GNUMailConstants.h"
#import "LabelWidget.h"
#import "POP3View.h"
#import "Utilities.h"

@implementation POP3ViewController

//
//
//
- (id) initWithParent: (id) theParent
{
  self = [super init];

  parent = theParent;

#ifdef MACOSX

  if ( ![NSBundle loadNibNamed: @"POP3View" owner: self] )
    { 
      NSDebugLog(@"Fatal error occured while loading the POP3View nib file");
      AUTORELEASE(self);
      return nil;
    }
  
  RETAIN(pop3View);

#else
  // We link our view
  pop3View = [[POP3View alloc] initWithParent: self];
  
  leaveOnServer = [(POP3View*)pop3View leaveOnServer];
  daysField = [(POP3View*)pop3View daysField];
  useAPOP = [(POP3View*)pop3View useAPOP];
  mailboxPopUpButton = [(POP3View*)pop3View mailboxPopUpButton];
#endif

  // This MUST be the first step
  allNodes = RETAIN([Utilities initializeLocalAndIMAPFolderNodes]);
  
  // We then populate our popup
  [Utilities addItemsToPopUpButton: mailboxPopUpButton
	     usingFolderNodes: allNodes];

  return self;
}

//
//
//
- (void) dealloc
{
  RELEASE(pop3View);

  RELEASE(allNodes);

  [super dealloc];
}

//
// access / mutation methods
//
- (NSButton *) leaveOnServer
{
  return leaveOnServer;
}

- (NSTextField *) daysField
{
  return daysField;
}

- (NSButton *) useAPOP
{
  return useAPOP;
}

- (NSView *) pop3View
{
  return pop3View;
}

- (NSPopUpButton *) mailboxPopUpButton
{
  return mailboxPopUpButton;
}

- (FolderNode *) allNodes
{
  return allNodes;
}

@end
