/*
**  ReceivingWindow.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#import "ReceivingWindow.h"
#import "LabelWidget.h"
#import "GNUMailConstants.h"

@implementation ReceivingWindow

- (void) dealloc
{
  //NSDebugLog(@"ReceivingWindow: -dealloc");
  
  RELEASE(tabView);

  RELEASE(tabViewItem1);
  RELEASE(tabViewItem2);
  
  RELEASE(serverNameField);
  RELEASE(usernameField);
  RELEASE(passwordSecureField);
  RELEASE(rememberPassword);
  RELEASE(verifyOnStartup);
  RELEASE(useSecureConnection);

  RELEASE(popup);

  [super dealloc];
}

- (void) layoutWindow
{
  // For the UI
  LabelWidget *serverNameLabel, *serverPortLabel, *serverTypeLabel, *usernameLabel, *passwordLabel;
  NSButton *okButton, *cancelButton;
  NSBox *box;
  
  tabView = [[NSTabView alloc] initWithFrame: NSMakeRect(0,35,400,265)];
  [tabView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];

  box = [[NSBox alloc] initWithFrame: NSMakeRect(5,5,350,225)];
  [box setAutoresizingMask: NSViewWidthSizable|NSViewMaxXMargin];
  [box setTitlePosition: NSNoTitle];
  [box setBorderType: NSNoBorder];


  // Server type
  serverTypeLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5,200,80,TextFieldHeight)
				 label: _(@"Server type:")];
  [serverTypeLabel setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: serverTypeLabel];

  popup = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(100,200,105,ButtonHeight)];
  [popup addItemWithTitle: _(@"POP3")];
  [popup addItemWithTitle: _(@"IMAP")];
  [popup addItemWithTitle: _(@"UNIX")];
  [popup setTarget: [self windowController] ];
  [popup setAction: @selector(setType:) ];
  [popup setAutoresizingMask: NSViewWidthSizable|NSViewMaxXMargin|NSViewMinYMargin|NSViewMaxYMargin];
  [[box contentView] addSubview: popup];

  // Server name
  serverNameLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5,165,80,TextFieldHeight)
				 label: _(@"Server name:")];
  [serverNameLabel setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: serverNameLabel];

  serverNameField = [[NSTextField alloc] initWithFrame:NSMakeRect(100,165,150,TextFieldHeight)];
  [serverNameField setSelectable: YES];
  [serverNameField setAutoresizingMask: NSViewWidthSizable|NSViewMaxXMargin|NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: serverNameField];


  // Server port
  serverPortLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(260,165,40,TextFieldHeight)
				 label: _(@"Port:")];
  [serverPortLabel setAutoresizingMask: NSViewMinXMargin|NSViewMaxXMargin|NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: serverPortLabel];

  serverPortField = [[NSTextField alloc] initWithFrame:NSMakeRect(300,165,35,TextFieldHeight)];
  [serverPortField setSelectable: YES];
  [serverPortField setAutoresizingMask: NSViewMinXMargin|NSViewWidthSizable|NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: serverPortField];
    

  // Username
  usernameLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5,130,65,TextFieldHeight) 
			       label: _(@"Username:")];
  [usernameLabel setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: usernameLabel];
    
  usernameField = [[NSTextField alloc] initWithFrame: NSMakeRect(80,130,125,TextFieldHeight)];
  [usernameField setSelectable: YES];
  [usernameField setAutoresizingMask: NSViewWidthSizable|NSViewMaxXMargin|NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: usernameField];
  
  
  // Password
  passwordLabel = [LabelWidget labelWidgetWithFrame: NSMakeRect(5,100,65,TextFieldHeight) 
			       label: _(@"Password:")];
  [passwordLabel setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: passwordLabel];  
  
  passwordSecureField = [[NSSecureTextField alloc] initWithFrame: NSMakeRect(80,100,125,TextFieldHeight)];
  [passwordSecureField setSelectable: YES];
  [passwordSecureField setAutoresizingMask: NSViewWidthSizable|NSViewMaxXMargin|NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: passwordSecureField];
  

  // Remember password
  rememberPassword = [[NSButton alloc] initWithFrame: NSMakeRect(5,65,200,ButtonHeight)];
  [rememberPassword setButtonType: NSSwitchButton];
  [rememberPassword setBordered: NO];
  [rememberPassword setTitle: _(@"Remember password")];
  [rememberPassword setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: rememberPassword];

  
  // Verify on startup
  verifyOnStartup = [[NSButton alloc] initWithFrame: NSMakeRect(5,35,250,ButtonHeight)];
  [verifyOnStartup setButtonType: NSSwitchButton];
  [verifyOnStartup setBordered: NO];
  [verifyOnStartup setTitle: _(@"Verify for new mail on startup")];
  [verifyOnStartup setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: verifyOnStartup];


  // Use SSL
  useSecureConnection = [[NSButton alloc] initWithFrame: NSMakeRect(5,5,250,ButtonHeight)];
  [useSecureConnection setButtonType: NSSwitchButton];
  [useSecureConnection setBordered: NO];
  [useSecureConnection setTitle: _(@"Use secure connection (SSL)")];
  [useSecureConnection setTarget: [self windowController] ];
  [useSecureConnection setAction: @selector(sslClicked:)];
  [useSecureConnection setAutoresizingMask: NSViewMinYMargin|NSViewMaxYMargin];
  [box addSubview: useSecureConnection];
  
  // We init our first tabview item
  tabViewItem1 = [[NSTabViewItem alloc] initWithIdentifier: MainTabViewItemIdentifier];
  [tabViewItem1 setLabel: _(@"General")];
  [tabViewItem1 setView: box];
  [tabView addTabViewItem: tabViewItem1];
  RELEASE(box);
 
  // We init our second tabview item but we leave it empty for now
  tabViewItem2 = [[NSTabViewItem alloc] initWithIdentifier: ServerTypeTabViewItemIdentifier];
  [tabViewItem2 setLabel: @""];
  [tabView addTabViewItem: tabViewItem2];

  
  // Our Cancel / Ok buttons
  cancelButton = [[NSButton alloc] initWithFrame: NSMakeRect(220,5,80,ButtonHeight)];
  [cancelButton setButtonType: NSMomentaryPushButton];
  [cancelButton setTitle: _(@"Cancel")];
  [cancelButton setTarget: [self windowController] ];
  [cancelButton setAction: @selector(cancelClicked:)];
  [cancelButton setAutoresizingMask: NSViewMinXMargin|NSViewMaxYMargin];
  [[self contentView] addSubview: cancelButton];
  RELEASE(cancelButton);
 
  okButton = [[NSButton alloc] initWithFrame: NSMakeRect(310,5,80,ButtonHeight)];
  [okButton setButtonType: NSMomentaryPushButton];
  [okButton setKeyEquivalent: @"\r"];
  [okButton setImagePosition: NSImageRight];
  [okButton setImage: [NSImage imageNamed: @"common_ret"]];
  [okButton setAlternateImage: [NSImage imageNamed: @"common_retH"]];
  [okButton setTitle: _(@"OK")];
  [okButton setTarget: [self windowController] ];
  [okButton setAction: @selector(okClicked:)];
  [okButton setAutoresizingMask: NSViewMinXMargin|NSViewMaxYMargin];
  [[self contentView] addSubview: okButton];
  RELEASE(okButton);
  
  // We finally  add our tabView
  [[self contentView] addSubview: tabView];

  // We set the initial responder and the next key views
  [self setInitialFirstResponder: serverNameField];
  [serverNameField setNextKeyView: popup];
  [popup setNextKeyView: usernameField];
  [usernameField setNextKeyView: passwordSecureField];
  [passwordSecureField setNextKeyView: rememberPassword];
  [rememberPassword setNextKeyView: verifyOnStartup];
  [verifyOnStartup setNextKeyView: useSecureConnection];
  [useSecureConnection setNextKeyView: cancelButton];
  [cancelButton setNextKeyView: okButton];
  [okButton setNextKeyView: serverNameField];

  [self setMinSize: NSMakeSize(400,340)];
}


//
// access/mutation methods
//

- (NSTabView *) tabView
{
  return tabView;
}

- (NSTabViewItem *) tabViewItem1
{
  return tabViewItem1;
}

- (NSTabViewItem *) tabViewItem2
{
  return tabViewItem2;
}

- (NSTextField *) serverNameField
{
  return serverNameField;
}

- (NSTextField *) serverPortField
{
  return serverPortField;
}

- (NSTextField *) usernameField
{
  return usernameField;
}

- (NSPopUpButton *) popup
{
  return popup;
}

- (NSSecureTextField *) passwordSecureField
{
  return passwordSecureField;
}

- (NSButton *) rememberPassword
{
  return rememberPassword;
}

- (NSButton *) verifyOnStartup
{
  return verifyOnStartup;
}

- (NSButton *) useSecureConnection
{
  return useSecureConnection;
}

@end





