/*
**  UNIXViewController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "UNIXViewController.h"

#import "FolderNode.h"
#import "GNUMailConstants.h"
#import "LabelWidget.h"
#import "UNIXView.h"
#import "Utilities.h"

@implementation UNIXViewController

//
//
//
- (id) initWithParent: (id) theParent
{
  self = [super init];

  parent = theParent;

#ifdef MACOSX

  if ( ![NSBundle loadNibNamed: @"UNIXView" owner: self] )
    { 
      NSDebugLog(@"Fatal error occured while loading the UNIXView nib file");
      AUTORELEASE(self);
      return nil;
    }
  
  RETAIN(unixView);

#else
  // We link our view
  unixView = [[UNIXView alloc] initWithParent: self];
  mailspoolFileField = [(UNIXView *)unixView mailspoolFileField];
  mailboxPopUpButton = [(UNIXView*)unixView mailboxPopUpButton];
#endif

   // This MUST be the first step
  allNodes = RETAIN([Utilities initializeLocalAndIMAPFolderNodes]);
  
  // We then populate our popup
  [Utilities addItemsToPopUpButton: mailboxPopUpButton
	     usingFolderNodes: allNodes];

  return self;
}

//
//
//
- (void) dealloc
{
  RELEASE(unixView);


  [super dealloc];
}


//
// action methods
//
- (IBAction) mailspoolFileButtonClicked: (id) sender
{
  NSArray *fileToOpen;
  NSOpenPanel *oPanel;
  NSString *fileName;
  int count, result;
  
  oPanel = [NSOpenPanel openPanel];
  [oPanel setAllowsMultipleSelection:NO];
  result = [oPanel runModalForDirectory:NSHomeDirectory() file:nil types:nil];
  
  if (result == NSOKButton) {
    
    fileToOpen = [oPanel filenames];
    count = [fileToOpen count];
    
    if (count > 0) {
      fileName = [fileToOpen objectAtIndex:0];
      [mailspoolFileField setStringValue:fileName];
    }
  }
}


//
// access / mutation methods
//
- (NSView *) unixView
{
  return unixView;
}

- (NSTextField *) mailspoolFileField
{
  return mailspoolFileField;
}

- (NSPopUpButton *) mailboxPopUpButton
{
  return mailboxPopUpButton;
}

- (FolderNode *) allNodes
{
  return allNodes;
}


@end
