/*
**  ConsoleWindowController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "ConsoleWindowController.h"

#import "GNUMailConstants.h"

#ifndef MACOSX
#import "ConsoleWindow.h"
#endif

static ConsoleWindowController *singleInstance = nil;

@implementation ConsoleWindowController

- (id) initWithWindowNibName: (NSString *) windowNibName
{
#ifdef MACOSX
  
  self = [super initWithWindowNibName: windowNibName];
  
#else
  ConsoleWindow *consoleWindow;

  consoleWindow = [[ConsoleWindow alloc] initWithContentRect:NSMakeRect(100,100,500,250)
					     styleMask: (NSTitledWindowMask
							 | NSClosableWindowMask
							 | NSResizableWindowMask)
					 backing: NSBackingStoreBuffered
					 defer: NO];
  
  self = [super initWithWindow: consoleWindow];
  
  [consoleWindow layoutWindow];
  [consoleWindow setDelegate: self];

  // We link our outlets
  tableView = [consoleWindow tableView];

  RELEASE(consoleWindow);
#endif

  [[self window] setTitle: @""];
  
  // We finally set our autosave window frame name and restore the one from the user's defaults.
  [[self window] setFrameAutosaveName: @"ConsoleWindow"];
  [[self window] setFrameUsingName: @"ConsoleWindow"];

  return self;
}


//
//
//
- (void) dealloc
{
  //NSDebugLog(@"ConsoleWindowController: -dealloc");
  
  [[NSDistributedNotificationCenter defaultCenter] removeObserver: self
						   name: GNUMailConsoleMessage
						   object: nil];

#ifdef MACOSX
  [tableView setDataSource: nil];
#endif

  RELEASE(allMessages);

  [super dealloc];
}


//
// delegate methods
//
- (void) windowWillClose: (NSNotification *) theNotification
{
  // Do nothing
}


//
//
//
- (void) windowDidLoad
{
  allMessages = [[NSMutableArray alloc] init];

  [[NSDistributedNotificationCenter defaultCenter] addObserver: self
						   selector: @selector(messageWasReceived:)
						   name: GNUMailConsoleMessage
						   object: nil];

#ifdef MACOSX
  [tableView reloadData];
#endif
}


//
// Data Source methods
//
- (int) numberOfRowsInTableView: (NSTableView *)aTableView
{
  return [allMessages count];
}


//
//
//
- (id)           tableView: (NSTableView *) aTableView
 objectValueForTableColumn: (NSTableColumn *) aTableColumn 
		       row: (int) rowIndex
{
  ConsoleMessage *aMessage;

  aMessage = [allMessages objectAtIndex: rowIndex];

  if ( [[[aTableColumn headerCell] stringValue] isEqual: _(@"Date")] )
    {
      return [aMessage->date descriptionWithCalendarFormat: _(@"%H:%M:%S.%F")
		      timeZone: [aMessage->date timeZone]
		      locale: nil];
    }
  else
    {
      return aMessage->message;
    }
  
  // never reached
  return nil;
}


//
// other methods
//
- (void) messageWasReceived: (id) theNotification
{
  ConsoleMessage *aMessage;
  NSDictionary *aDictionary;

  aDictionary = [theNotification userInfo];

  aMessage = [[ConsoleMessage alloc] initWithMessage: [aDictionary objectForKey: @"Message"]];
  
  [allMessages insertObject: aMessage
	       atIndex: 0];
  RELEASE(aMessage);

  // We never keep more than 25 object in our array
  if ( [allMessages count] > 25 )
    {
      [allMessages removeLastObject];
    }

  [tableView reloadData];
}


//
// class methods
//
+ (id) singleInstance
{
  if ( singleInstance == nil )
    {
      singleInstance = [[ConsoleWindowController alloc] initWithWindowNibName: @"ConsoleWindow"];
    }

  return singleInstance;
}

@end



//
// 
//
@implementation ConsoleMessage

- (id) initWithMessage: (NSString *) theMessage
{
  self = [super init];

  message = RETAIN(theMessage);
  date = RETAIN([NSCalendarDate calendarDate]);

  return self;
}

- (void) dealloc
{
  RELEASE(message);
  RELEASE(date);

  [super dealloc];
}

@end
