/*
**  EditWindowToolbar.m
**
**  Copyright (c) 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "EditWindowToolbar.h"

@implementation EditWindowController (EditWindowToolbar)

//
// NSToolbar delegate methods
//
- (NSToolbarItem *) toolbar: (NSToolbar *) toolbar
      itemForItemIdentifier: (NSString *) itemIdentifier
  willBeInsertedIntoToolbar: (BOOL) flag
{
  NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
  
  if ([itemIdentifier isEqualToString: @"send"])
    {
      [item setLabel: @"Send"];
      [item setPaletteLabel: @"Send Message"];
      [item setImage: [NSImage imageNamed: @"send_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(sendMessage:)];
      send = (id)item;
    }
  else if ([itemIdentifier isEqualToString: @"insert"])
    {
      [item setLabel: @"Attach"];
      [item setPaletteLabel: @"Insert Attachment"];
      [item setImage: [NSImage imageNamed: @"attach_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(insertFile:)];
      insert = (id)item;
    }
  else if ([itemIdentifier isEqualToString: @"add_cc"])
    {
      [item setLabel: @"Add Cc"];
      [item setPaletteLabel: @"Add Cc Field"];
      [item setImage: [NSImage imageNamed: @"add_cc_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(showCc:)];
      addCc = (id)item;
    }
  else if ([itemIdentifier isEqualToString: @"add_bcc"])
    {
      [item setLabel: @"Add Bcc"];
      [item setPaletteLabel: @"Add Bcc Field"];
      [item setImage: [NSImage imageNamed: @"add_bcc_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(showBcc:)];
      addBcc = (id)item;
    }
  else if ([itemIdentifier isEqualToString: @"addresses"])
    {
      [item setLabel: @"Addresses"];
      [item setPaletteLabel: @"Show Address Book"];
      [item setImage: [NSImage imageNamed: @"addresses_32.tiff"]];
      [item setTarget: [NSApp delegate]];
      [item setAction: @selector(showAddressBook:)];
      addresses = (id)item;
    }
  else
    {
      NSDebugLog(@"Returning toolbar item from bundle.");
      return [additionalToolbarItems objectForKey: itemIdentifier];
    }

  return [item autorelease];
}


//
//
//
- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar*) toolbar
{
  return allowedToolbarItemIdentifiers;
}


//
//
//
- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar*) toolbar
{
  return [NSArray arrayWithObjects: @"send",
		  @"insert",
                  @"add_cc",
                  @"add_bcc",
		  @"addresses",
		  nil];
}


//
// NSToolbarItemValidation protocol
//
- (BOOL) validateToolbarItem:(NSToolbarItem *)theItem
{
  return YES;
}

@end
