/*
**  ExtendedOutlineView.m
**
**  Copyright (c) 2002
**
**  Author: Francis Lachapelle <francis@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "ExtendedOutlineView.h"

/*!
 * @class ExtendedOutlineView
 * @abstract This class overwrites some methods of NSOutlineView.
 * @discussion This class is used by the MailboxManager class so
 *             all items look like folders un Mac OS X and items
 *             can be dragged on items and not inbetween nodes.
 *             
 */
@implementation ExtendedOutlineView: NSOutlineView


#ifdef MACOSX
/*!
 * @method isExpandable:
 * @abstract Specifies if a node can be expanded.
 * @discussion
 * @param item The node of the outline view.
 * @result Always returns YES, so that every node appears
 *         as expandable.
 */
- (BOOL) isExpandable: (id) item
{
  return YES;
}
#endif


/*!
 * @method setDropItem: dropChildIndex:
 * @abstract Retargets drop on child index.
 * @discussion This method sets the behaviour of the outline
 *             view so that items dragged over a node can be
 *             dropped on the node and inbetween nodes.
 * @param item The item involved.
 * param index The index of the item.
 */
- (void) setDropItem: (id) item
      dropChildIndex: (int) index
{
  [super setDropItem: item
	 dropChildIndex: NSOutlineViewDropOnItemIndex];
}


/*!
 * @method shouldCollapseAutoExpandedItemsForDeposited:
 * @abstract Indicate if auto expanded items should return to
 *           their original collapsed state. 
 * @discussion
 * @param deposited Tells whether or not the drop terminated due
 *                  to a successful drop.
 * @result Always returns NO, so that items never return to
 *         their original collapsed state.
 */
- (BOOL)shouldCollapseAutoExpandedItemsForDeposited: (BOOL) deposited
{
  return NO;
}

@end
