/*
**  GNUMail.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "GNUMail.h"

#import "Address.h"
#import "AddressBook.h"
#import "AddressBookController.h"
#import "AddressBookWindow.h"
#import "BounceWindowController.h"
#import "ConsoleWindowController.h"
#import "EditWindowController.h"
#import "ExtendedWindow.h"
#import "Filter.h"
#import "FilterManager.h"
#import "FindWindowController.h"
#import "FolderNode.h"
#import "Group.h"
#import "GNUMail/GNUMailBundle.h"
#import "GNUMailConstants.h"
#import "MailWindowController.h"
#import "MailboxManagerController.h"
#import "MessageViewWindowController.h"
#import "PreferencesWindowController.h"
#import "Utilities.h"

#import <Pantomime/Constants.h>
#import <Pantomime/InternetAddress.h>
#import <Pantomime/IMAPCacheManager.h>
#import <Pantomime/IMAPFolder.h>
#import <Pantomime/IMAPStore.h>
#import <Pantomime/LocalFolder.h>
#import <Pantomime/LocalStore.h>
#import <Pantomime/Message.h>
#import <Pantomime/Store.h>
#import <Pantomime/URLName.h>

@implementation GNUMail

static NSMutableArray *allMailWindows = nil;
static NSMutableArray *allBundles;

static NSString *currentWorkingPath = nil;

static id lastAddressTakerWindowOnTop = nil;
static id lastMailWindowOnTop = nil;

//
//
//
- (id) init
{
  self = [super init];

  messageUsingTextEncoding = nil;

  return self;
}


//
//
//
- (void) dealloc
{
  TEST_RELEASE(messageUsingTextEncoding);

  [super dealloc];
}


//
// action methods
//
- (IBAction) applyManualFilter: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      FilterManager *aFilterManager;
      
      NSArray *selectedMessages;
      int i, aTag, nbOfTransferredMessages;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      selectedMessages = [aMailWindowController selectedMessages];

      if ( !selectedMessages || [selectedMessages count] == 0 )
	{
	  NSBeep();
	  return;
	}

      aTag = [sender tag];
      nbOfTransferredMessages = 0;

      aFilterManager = [FilterManager filtersFromDisk];
      RETAIN(aFilterManager);
      
      for (i = 0; i < [selectedMessages count]; i++)
	{
	  Message *aMessage;
	  Filter *aFilter;

	  aMessage = [selectedMessages objectAtIndex: i];
	  
	  // If we have selected ALL our filters...
	  if ( aTag < 0 )
	    {
	      // FIXME: If we are in the Outbox, we should consider
	      //        ONLY outgoing filters. Otherwise, we always
	      //        consider ONLY incoming filters.
	      aFilter = [aFilterManager matchedFilterForMessage: aMessage
					type: TYPE_INCOMING];
	    }
	  else
	    {
	      aFilter = [aFilterManager filterAtIndex: aTag];
	    }

	  // We verify if our filter matches our message.
	  if ( aFilter && [aFilterManager matchExistsForFilter: aFilter
					  message: aMessage] )
	    {
	      NSString *aFolderName;
	      Store *aStore;

	      aFolderName = nil;
	      aStore = nil;
	      
	      // FIXME - this part of the code is WRONG
	      // We get the right folder name
	      if ( [aFilter action] == TRANSFER_TO_FOLDER )
		{
		  URLName *theURLName;

     		  theURLName = [[URLName alloc] initWithString: [aFilter actionFolderName]
						path: [[NSUserDefaults standardUserDefaults] 
							objectForKey: @"LOCALMAILDIR"]];
		  AUTORELEASE(theURLName);
		  
		  
		  aFolderName = [theURLName foldername];
		  aStore = [Utilities storeForURLName: theURLName];
		  
		  if ( !aStore )
		    {
		      NSRunAlertPanel(_(@"Error!"),
				      _(@"An error occured while attempting to obtain the message store for target filter."),
				      _(@"OK"),
				      NULL,
				      NULL);
		      aStore = nil;
		      aFolderName = nil;
		    }
		  
		  // We verify that our target folder isn't the one that is currently open (and last on top)
		  // If so, we do nothing.
		  if ( aFolderName && aStore )
		    {
		      if ( [aFolderName isEqualToString: [[aMailWindowController folder] name]] &&
			   aStore == [[aMailWindowController folder] store] )
			{
			  aStore = nil;
			  aFolderName = nil;
			}
		    }

		  // We verify that our folder hasn't been deleted!
		  // If it was deleted, we do nothing. 
		  if ( aFolderName && aStore )
		    {
		      if ( ![[MailboxManagerController singleInstance] folderNameExists: aFolderName
								       forStore: aStore] )
			{
			  aStore = nil;
			  aFolderName = nil;
			}
		    }
		}
	      // We verify if the operation is DELETE. If so, we transfer the message to the trash folder.
	      // FIXME, must support IMAP folder
	      else if ( [aFilter action] == DELETE )
		{      
		  URLName *theURLName;
		  
		  theURLName = [[URLName alloc] initWithString: [[NSUserDefaults standardUserDefaults] 
								  objectForKey: @"TRASHFOLDERNAME"]
						path: [[NSUserDefaults standardUserDefaults] 
							objectForKey: @"LOCALMAILDIR"]];
		  AUTORELEASE(theURLName);
		  
		  aFolderName = [theURLName foldername];
		  aStore = [Utilities storeForURLName: theURLName];
		  
		  if ( !aStore )
		    {
		      NSRunAlertPanel(_(@"Error!"),
				      _(@"An error occured while attempting to obtain the message store for target filter."),
				      _(@"OK"),
				      NULL,
				      NULL);
		      aStore = nil;
		      aFolderName = nil;
		    }
		}
	      else
		{
		  aStore = nil;
		  aFolderName = nil;
		}

	      // We got a valid folder name
	      if ( aFolderName && aStore )
		{
		  //NSDebugLog(@"foldername |%@|", aFolderName);

		  
		  if ( [[MailboxManagerController singleInstance] transferMessageFromRawSource: [aMessage rawSource]
								  flags: AUTORELEASE([[aMessage flags] copy])
								  toFolderWithName: aFolderName
								  orToFolder: nil
								  store: aStore] )
		    {
		      [[aMessage flags] add: TRANSFERRED];
		      nbOfTransferredMessages++;
		    }
		  else
		    {
		      NSDebugLog(@"Transfer failed for message to folder %@", aFolderName);
		    }
		}
	      else
		{
		  NSDebugLog(@"Folder name is nil.");
		}
	    } // if ( aFilter && [aFilterManager ...] )
	} // for (...)
      
      if ( nbOfTransferredMessages > 0 )
	{
	  // We force the reload of our cache in Folder
	  [[aMailWindowController folder] updateCache];
	  [aMailWindowController tableViewShouldReloadData];
	}

      RELEASE(aFilterManager);
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) bounceMessage: (id) sender
{
  // We first verify if we have at least one transport agent defined
  if (! [[NSUserDefaults standardUserDefaults] objectForKey: @"SENDING"] ||
      ! [(NSArray *)[[NSUserDefaults standardUserDefaults] objectForKey: @"SENDING"] count] )
    {
      NSRunAlertPanel(_(@"Error!"),
		      _(@"You must have at least one transport agent defined.\nSee Preferences -> Sending."),
		      _(@"OK"), // default
		      NULL,     // alternate
		      NULL);
      
      return;
    }

  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      id aWindowController;
      Message *theMessage;
      
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      theMessage = [aWindowController selectedMessage];
      
      if ( theMessage )
	{
	  BounceWindowController *bounceWindowController;
	  
	  bounceWindowController = [[BounceWindowController alloc] initWithWindowNibName: @"BounceWindow"];
	  [bounceWindowController setMessage: theMessage];
	  [bounceWindowController showWindow: self];
	}
      else
	{
	  NSBeep();
	}
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) changeTextEncoding: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      id aWindowController;
      Message *theMessage;
      
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      
      if ( !aWindowController )
	{
	  NSBeep();
	  return;
	}

      if ( [aWindowController isKindOfClass: [MailWindowController class]] )
	{
	  theMessage = [aWindowController selectedMessage];
	}
      else
	{
	  theMessage = [aWindowController message];
	}
      
      if ( theMessage )
	{
	  // We get the selected charset
	  if ( [sender tag] >= 0 )
	    {
	      NSAutoreleasePool *pool;
	      NSData *theRawSource;
	      NSString *theCharset;
	      
	      pool = [[NSAutoreleasePool alloc] init];
	      theRawSource = [theMessage rawSource];    
	      theCharset = [[[MimeUtility allCharsets] allKeysForObject: [sender title]] objectAtIndex: 0];
	      
	      // We release our previous initialized message
	      TEST_RELEASE(messageUsingTextEncoding);
	      
	      // We re-init using an other charset
	      messageUsingTextEncoding = [[Message alloc] initWithData: theRawSource
							  charset: theCharset];
	      
	      // We release our local pool
	      RELEASE(pool);
	    }
	  else
	    {
	      messageUsingTextEncoding = theMessage;
	      RETAIN(messageUsingTextEncoding);
	    }
	  
	  // We show the new message
	  [Utilities showMessage: messageUsingTextEncoding
		     target: [aWindowController textView]
		     showAllHeaders: [aWindowController showAllHeaders]];
	}
      else
	{
	  NSBeep();
	}
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) checkForUpdates: (id) sender
{
  NSString *currVersionNumber;
  NSDictionary *productVersionDict;
  NSString *latestVersionNumber;
  

  currVersionNumber = [[[NSBundle bundleForClass: [self class]] infoDictionary] 
			objectForKey: @"CFBundleVersion"];

#ifdef MACOSX
  productVersionDict = [NSDictionary dictionaryWithContentsOfURL:
				       [NSURL URLWithString:@"http://www.collaboration-world.com/versionlist.xml"]];
#else
  productVersionDict = [[NSString stringWithContentsOfURL:
                         [NSURL URLWithString: @"http://www.collaboration-world.com/versionlist.xml"]]
                          propertyList];
#endif
  
  latestVersionNumber = [productVersionDict valueForKey:@"GNUMail"];

  if ([latestVersionNumber isEqualToString: currVersionNumber])
    {
      NSRunAlertPanel(_(@"Your Software is up-to-date"),
		      _(@"You have the most recent version of GNUMail."),
		      _(@"OK"),
		      NULL, 
		      NULL);
    }
  else
    {
      int choice;

      choice = NSRunAlertPanel(_(@"A New Version is Available"),
			       _(@"A new version of GNUMail is available (version %@). Would you like to download the new version now?"),
			       _(@"OK"),
			       _(@"Cancel"),
			       NULL, 
			       latestVersionNumber);
      if ( choice == NSOKButton )
	{
#ifdef MACOSX
	  [[NSWorkspace sharedWorkspace] openURL:
					   [NSURL URLWithString:@"http://www.collaboration-world.com/cgi-bin/collaboration-world/release.cgi?pid=2"]];
#else
	  NSRunAlertPanel(_(@"Feature not implemented!"),
			  _(@"Sorry, this feature is only available under Mac OS X."),
			  _(@"OK"),
			  NULL,
			  NULL);
#endif
	}
    }
}


//
//
//
- (IBAction) close: (id) sender
{
  if ( [NSApp keyWindow] )
    {
      [[NSApp keyWindow] performClose: sender];
    }
}


//
//
//
- (IBAction) compactMailbox: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      int choice;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      choice = NSRunAlertPanel(_(@"Compact..."),
			       _(@"Compacting a mailbox will permanently remove deleted messages.\nDo you want to continue?"),
			       _(@"No"),  // default
			       _(@"Yes"), // alternate
			       NULL);

      if (choice == NSAlertAlternateReturn)
	{
	  [[NSDistributedNotificationCenter defaultCenter]
	    postNotificationName: GNUMailConsoleMessage
	    object: nil
	    userInfo: [NSDictionary dictionaryWithObject: [NSString stringWithFormat: _(@"Compacting mailbox, please wait..."),
								    [[aMailWindowController folder] name]]
				    forKey: @"Message"]];
	  
	  [[aMailWindowController folder] expunge: NO];
	  [aMailWindowController tableViewShouldReloadData];
	  [aMailWindowController updateStatusLabel];

	  [[NSDistributedNotificationCenter defaultCenter]
	    postNotificationName: GNUMailConsoleMessage
	    object: nil
	    userInfo: [NSDictionary dictionaryWithObject: [NSString stringWithFormat: _(@"Done compacting mailbox %@."),
								    [[aMailWindowController folder] name]]
				    forKey: @"Message"]];
	}
    }
  else
    {
      NSBeep();
    }
}


//
// This method is used to compose a new message, with an empty content.
//
- (IBAction) composeMessage: (id) sender
{
  EditWindowController *editWindowController;
  Message *aMessage;
  
  // We create an empty message
  aMessage = [[Message alloc] init];
  
  // We create our controller and we show the window
  editWindowController = [[EditWindowController alloc] initWithWindowNibName: @"EditWindow"];

  if ( editWindowController )
    {
      [[editWindowController window] setTitle: _(@"New message...")];
      [editWindowController setMessage: aMessage];
      [editWindowController setShowCc: NO];
            
      [editWindowController showWindow: self];
    }

  RELEASE(aMessage);
}



//
//
//
- (IBAction) copy: (id) sender
{
  NSPasteboard *pboard;
  int i, count;

  
  pboard = [NSPasteboard generalPasteboard];
  
  if ( [[[NSApp keyWindow] delegate] isKindOfClass: [MailWindowController class]] && [GNUMail lastMailWindowOnTop] )
    {
      //
      // MessagePboardType
      // 

      MailWindowController *aMailWindowController;
      NSArray *anArray;
      Message *aMessage;
      NSMutableArray *propertyList;
      NSString *firstSubject;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      anArray = [aMailWindowController selectedMessages];
      count = [anArray count];
      firstSubject = nil;

      NSDebugLog(@"copy: %i message(s)", count);

      if ( count )
	{
	  propertyList = [[NSMutableArray alloc] initWithCapacity: count];
      
	  // By default, we assume that our copy operation was sucessful
	  lastCopyOpWasSuccessful = YES;

	  for (i = (count - 1); i >= 0; i--)
	    {
	      // For each implicated message, insert its raw representation
	      // in a dictionary with two keys: 'flags' and 'message'
	      
	      NSMutableDictionary *aDictionary = [[NSMutableDictionary alloc] initWithCapacity: 2];
	      NSData *flags, *rawSource;
	      
	      aMessage = (Message *)[anArray objectAtIndex: i];
	      
	      if (firstSubject == nil)
		{
		  firstSubject = [aMessage subject];
		}

	      flags = [NSArchiver archivedDataWithRootObject: [aMessage flags]];
	      [aDictionary setObject: flags forKey: @"flags"];
	      
	      rawSource = [NSData dataWithData: [aMessage rawSource]];
	      
	      if ( rawSource )
		{
		  [aDictionary setObject: rawSource forKey: @"message"];
		  [propertyList addObject: aDictionary];
		}
	      else
		{
		  lastCopyOpWasSuccessful = NO;
		}
	      
	      RELEASE(aDictionary);
	    }
	  
	  [pboard declareTypes: [NSArray arrayWithObjects: NSStringPboardType, MessagePboardType, nil] 
		  owner: self];
	  [pboard setPropertyList: propertyList 
		  forType: MessagePboardType];
	  if (firstSubject != nil)
	    {
	      [pboard setString: firstSubject
		      forType: NSStringPboardType];
	  }
	  RELEASE(propertyList);
	}
      else
	{
	  NSBeep();
	}
    }
  else if ( [[[NSApp keyWindow] delegate] isKindOfClass: [AddressBookController class]] )
    {
      //
      // AddressPboardType
      //
      
      AddressBookController *addressBookController;
      AddressBook *addressBook;
      NSBrowser *browser;
      Group *aGroup;
      Address *anAddress;

      addressBookController = [AddressBookController singleInstance];
      addressBook = [addressBookController addressBook];
      browser = [addressBookController browser];
      
      if ( [browser selectedCellInColumn: 1] )
	{
	  aGroup = [addressBook groupAtIndex: [browser selectedRowInColumn: 0]];
	  anAddress = [aGroup addressAtIndex: [browser selectedRowInColumn: 1]];
	  NSDebugLog(@"copy: address \"%@\"", [anAddress name]);

	  [pboard declareTypes: [NSArray arrayWithObjects: NSStringPboardType, AddressPboardType, nil]
		  owner: self];
	  [pboard setData: [NSArchiver archivedDataWithRootObject: anAddress] 
		  forType: AddressPboardType];
	  [pboard setString: [anAddress formattedAddress]
		  forType: NSStringPboardType];
	}
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) cut: (id) sender
{
  if ( [[[NSApp keyWindow] delegate] isKindOfClass: [MailWindowController class]] && [GNUMail lastMailWindowOnTop] )
    {
      //
      // MessagePboardType
      //
      MailWindowController *aMailWindowController;
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];

      // We copy the selected messages to the pasteboard
      [self copy: nil];
      
      if ( !lastCopyOpWasSuccessful )
	{
	  NSDebugLog(@"The previous copy operation failed for some (or all) messages.");
	  return;
	}

      // We mark the selected messages as transferred
      [aMailWindowController setSelectedMessagesAsTransferred];
    }
  else if ( [[[NSApp keyWindow] delegate] isKindOfClass: [AddressBookController class]] )
    {
      //
      // AddressPboardType
      //
      AddressBookController *addressBookController;
      AddressBook *addressBook;
      NSBrowser *browser;
      Group *group;
      Address *address;

      addressBookController = [AddressBookController singleInstance];
      addressBook = [addressBookController addressBook];
      browser = [addressBookController browser];
      
      if ( [browser selectedCellInColumn: 1] )
	{
	  group = [addressBook groupAtIndex: [browser selectedRowInColumn: 0]];
	  address = [group addressAtIndex: [browser selectedRowInColumn: 1]];
	  
	  // We copy the selected address to the pasteboard
	  [self copy: nil];

	  // We remove the selected address from its group
	  [group removeAddress: address];
	  
	  [addressBook synchronize];
		      
	  [browser reloadColumn: 1];
	}
      else
	{
	  NSBeep();
	}
    }
  else
    {
      NSBeep();
    }
}


//
// This method is used to togle the flags of the selected message
// from the last mail window on top to 'undelete' (or delete) if it was
// set to 'deleted' before the call.
//
- (IBAction) deleteOrUndeleteMessage: (id) sender
{  
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      // We delete (or undelete) our message
      [aMailWindowController deleteMessage: nil];
      
      // We 'swap' our title and our tag
      if ( [deleteOrUndelete tag] == DELETE_MESSAGE )
	{
	  [deleteOrUndelete setTitle: _(@"Undelete")];
	  [deleteOrUndelete setTag: UNDELETE_MESSAGE];
	}
      else
	{
	  [deleteOrUndelete setTitle: _(@"Delete")];
	  [deleteOrUndelete setTag: DELETE_MESSAGE];
	}
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) enterSelectionInFindPanel: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      NSTextView *aTextView;

      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      aTextView = [aMailWindowController textView];
      
      [[[FindWindowController singleInstance] 
	 findField] setStringValue: [[aTextView string] substringWithRange: [aTextView selectedRange]]];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) findNext: (id) sender
{
  [[FindWindowController singleInstance] nextMessage: nil];
}


//
//
//
- (IBAction) findPrevious: (id) sender
{
  [[FindWindowController singleInstance] previousMessage: nil];
}


// 
// This method is used to forward the selected messsage from the last
// mail window that was on top.
//
- (IBAction) forwardMessage: (id) sender
{  
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;
      
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      [aWindowController forwardMessage: nil];
    }
}


//
// This method is used to get the new messages if the LocalInboxWindow
// is currently opened and visible.
//
- (IBAction) getNewMessages: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] &&
       [[[GNUMail lastMailWindowOnTop] delegate] isKindOfClass: [MailWindowController class]] )
    {
      id aWindowController;
      
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];

      [aWindowController getNewMessages: sender];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) markMessageAsReadOrUnread: (id) sender
{
  MailWindowController *aMailWindowController;
  
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      NSEnumerator *anEnumerator;
      Message *aMessage;
      NSNumber *aRow;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      // We mark our message as read (or not)
      anEnumerator = [[aMailWindowController tableView] selectedRowEnumerator];

      while ( (aRow = [anEnumerator nextObject]) )
	{
	  aMessage = [[aMailWindowController allMessages] objectAtIndex: [aRow intValue]];
 
	  // If we must mark all messages as read
	  if ( [markAsReadOrUnread tag] == MARK_AS_READ )
	    {
	      if ( [[aMessage flags] contain: RECENT] )
		{
		  [[aMessage flags] remove: RECENT];
		  [[aMessage flags] add: SEEN];
		} 
	    }
	  // Else, we must mark them all as unread
	  else
	    {
	      if ( [[aMessage flags] contain: SEEN] )
		{
		  [[aMessage flags] remove: SEEN];
		  [[aMessage flags] add: RECENT];
		} 
	    }
	} // while (...)
      
      // We always refresh our tableView and our status label
      [[aMailWindowController tableView] setNeedsDisplay: YES];
      [aMailWindowController updateStatusLabel];
      
      // We 'swap' our title and our tag
      if ( [markAsReadOrUnread tag] == MARK_AS_READ )
	{
	  [markAsReadOrUnread setTitle: _(@"Mark as Unread")];
	  [markAsReadOrUnread setTag: MARK_AS_UNREAD];
	}
      else
	{
	  [markAsReadOrUnread setTitle: _(@"Mark as Read")];
	  [markAsReadOrUnread setTag: MARK_AS_READ];
	}
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) paste: (id) sender
{
  NSPasteboard *pboard;
  NSArray *types;
  NSString *aType;

  NSArray *propertyList;
  NSData *data;
  int i, j, count;
  

  // We verify every supported pasteboard type
  
  pboard = [NSPasteboard generalPasteboard];
  types = [pboard types];
  for (i = 0; i < [types count]; i++)
    {
      aType = (NSString*)[types objectAtIndex: i];
      NSDebugLog(@"paste: type %i = %@", i, aType);
      
      if ( [MessagePboardType isEqualToString: aType] )
	{
	  //
	  // MessagePboardType
	  //
	  propertyList = [pboard propertyListForType: MessagePboardType];

	  if ( propertyList )
	    {
	      count = [propertyList count];
	      
	      NSDebugLog(@"paste: %i message(s)", count);
	      
	      if ( [NSApp keyWindow] == [[MailboxManagerController singleInstance] window] )
		{
		  //
		  // The front window is the Mailbox Manager
		  //

		  MailboxManagerController *mailboxManagerController;
		  id item;
		  int row, level;
		  NSString *aFolderName;
		  Folder *aFolder;
		  Store *aStore;
		  
		  mailboxManagerController = [MailboxManagerController singleInstance];
		  row = [[mailboxManagerController outlineView] selectedRow];
		  
		  if ( row < 0 )
		    {
		      NSBeep();
		      return;
		    }
		  
		  item = [[mailboxManagerController outlineView] itemAtRow: row];
		  level = [[mailboxManagerController outlineView] levelForItem: item];
		  
		  //
		  // We must verify that:
		  //
		  // a) we have at least one selected row
		  // b) we haven't selected our root, or a store (local or IMAP)
		  //
		  if ( [[mailboxManagerController outlineView] numberOfSelectedRows] != 1 ||
		       level < 2 )
		    {
		      NSRunInformationalAlertPanel(_(@"Mailbox error!"),
						   _(@"You must select a valid mailbox to open!"),
						   _(@"OK"),
						   NULL, 
						   NULL,
						   NULL);
		      continue;
		    }
		  
		  aFolderName = [Utilities pathOfFolderFromFolderNode: (FolderNode *)item
					   separator: @"/"];
		  
		  // We get our store
		  if ( [[Utilities completePathForFolderNode: (FolderNode *)item separator: @"/"]
			 hasPrefix: [NSString stringWithFormat: @"/%@", _(@"Local")]] )
		    {
		      aStore = [mailboxManagerController storeForName: @"GNUMAIL_LOCAL_STORE"
							 username: NSUserName()];
		    }
		  else
		    {
		      NSString *aServerName, *aUsername;
		      
		      [Utilities storeKeyForFolderNode: (FolderNode *)item
				 serverName: &aServerName
				 username: &aUsername];
		      
		      aStore = [mailboxManagerController storeForName: aServerName
							 username: aUsername];
		    }
		  
		  // We get a reference to our destination folder, w/o parsing it if it's not already open.
		  // or w/o selecting it if it's an IMAP store.
		  if ( [(id<NSObject>)aStore isKindOfClass: [IMAPStore class]] )
		    {
		      aFolder = (Folder *)[(IMAPStore *)aStore folderForName: aFolderName
							select: NO];
		    }
		  else
		    {
		      aFolder = (Folder *)[(LocalStore *)aStore folderForName: aFolderName];
		    }
		  
		  for (j = 0; j < count; j++)
		    {
		      // We retrieve the message from the property list
		      NSDictionary *aDictionary;
		      Flags *flags;
		      NSData *rawSource;
		      
		      aDictionary = (NSDictionary*)[propertyList objectAtIndex: j];
		      
		      rawSource = [aDictionary objectForKey: @"message"];
		      flags = (Flags*)[NSUnarchiver unarchiveObjectWithData: (NSData*)[aDictionary objectForKey: 
												     @"flags"]];
		      
		      if (rawSource && flags)
			{
			  // We add the message to the folder
			  if ( ![[MailboxManagerController singleInstance] transferMessageFromRawSource: rawSource
									   flags: flags
									   toFolderWithName: aFolderName
									   orToFolder: aFolder
									   store: aStore] )
			    {
			      // The transfer failed; we exit the loop
			      break;
			    }
			}
		    }
		  
		  // If the folder wasn't open, we just close it.
		  if ( aFolder )
		    {
		      [aFolder close];
		    } 
		}
	      else if ( [[[NSApp keyWindow] delegate] isKindOfClass: [MailWindowController class]] &&
			[GNUMail lastMailWindowOnTop] )
		{
		  //
		  // The front window is a Mail Window
		  //

		  MailWindowController *aMailWindowController;
		  
		  aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
		  
		  for (j = count - 1; j >= 0; j--)
		    {
		      // We retrieve the message from the property list
		      NSDictionary *aDictionary;
		      Flags *flags;
		      NSData *rawSource;
		      
		      aDictionary = (NSDictionary*)[propertyList objectAtIndex: j];
		      
		      rawSource = [aDictionary objectForKey: @"message"];
		      
		      flags = (Flags*)[NSUnarchiver unarchiveObjectWithData: (NSData*)[aDictionary objectForKey:
												     @"flags"]];
		      if (rawSource && flags)
			{
			  // We add the message to the folder
			  if ( ![[MailboxManagerController singleInstance] transferMessageFromRawSource: rawSource
									   flags: flags
									   toFolderWithName: [[aMailWindowController folder] name]
									   orToFolder: [aMailWindowController folder]
									   store: [[aMailWindowController folder] store]] )
			    {
			      // The transfer failed; we exit the loop
			      break;
			    }
			}
		    }
		  
		  if (count > 0)
		    {
		      // We refresh the table view
		      [aMailWindowController tableViewShouldReloadData];
		      
		      // We update the status label
		      [aMailWindowController updateStatusLabel];
		    }
		}
	      else
		{
		  NSBeep();
		  NSDebugLog(@"-paste Message to unknown destination");
		}
	    }
	}

      else if ( [AddressPboardType isEqualToString: aType] )
	{
	  //
	  // AddressPboardType
	  //

	  data = [pboard dataForType: AddressPboardType];
	  if ( [data length] )
	    {
	      Address *anAddress;
      
	      anAddress = (Address*)[NSUnarchiver unarchiveObjectWithData: data];
	      NSDebugLog(@"paste: %@", [anAddress name]);
	      
	      if ( [NSApp keyWindow] == [[AddressBookController singleInstance] window] )
		{
		  // The front window is the Address Book Manager
		  
		  AddressBookController *addressBookController;
		  AddressBook *addressBook;
		  NSBrowser *browser;
		  Group *aGroup;
	  
		  addressBookController = [AddressBookController singleInstance];
		  addressBook = [addressBookController addressBook];
		  browser = [addressBookController browser];
		  
		  if ( [browser selectedCellInColumn: 0] )
		    {
		      aGroup = [addressBook groupAtIndex: [browser selectedRowInColumn: 0]];
		      
		      [aGroup addAddress: anAddress];
		      
		      [addressBook synchronize];
		      
		      [browser reloadColumn: 1];
		    }
		}
	      else if ( [NSApp keyWindow] )
		{
		  [[NSApp keyWindow] insertText: [anAddress name]];
		}
	      else
		{
		  NSBeep();
		  NSDebugLog(@"-paste Address to unknown destination");
		}
	    }
	}
    }
}


//
//
//
- (IBAction) print: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;

      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      
      [[NSPrintOperation printOperationWithView: [aWindowController textView]] runOperation];
    }
  else
    {
      NSBeep();
    }
}


//
// This method is used to reply to the selected message
// to all recipients.
//
- (IBAction) replyAllMessage: (id) sender
{  
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;

      aWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      [aWindowController replyAllMessage: nil];
    }
  else
    {
      NSBeep();
    }
}


//
// This method is used to reply to the selected message.
//
- (IBAction) replyMessage: (id) sender
{
  id aWindowController;
  
  if ( [GNUMail lastMailWindowOnTop] )
    {
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      [aWindowController replyMessage: nil];
    }
}


//
//
//
- (IBAction) restoreDraft: (id) sender
{
  URLName *theURLName;
  Message *aMessage;
  
  aMessage = nil;
  

  theURLName = [[URLName alloc] initWithString: [[NSUserDefaults standardUserDefaults] objectForKey: @"DRAFTSFOLDERNAME"]
				path: [[NSUserDefaults standardUserDefaults] objectForKey: @"LOCALMAILDIR"]];

  if ( [[theURLName protocol] caseInsensitiveCompare: @"LOCAL"] == NSOrderedSame )
    {
      aMessage = [self _messageFromLocalDraftsFolderUsingURLName: theURLName];
    }
  else
    {
      aMessage = [self _messageFromIMAPDraftsFolderUsingURLName: theURLName];
    }

  RELEASE(theURLName);
    
  
  // We create our controller and we show the window if we have a valid message
  if ( aMessage )
    {
      EditWindowController *editWindowController;
      
      editWindowController = [[EditWindowController alloc] initWithWindowNibName: @"EditWindow"];
      
      if ( editWindowController )
	{
	  [[editWindowController window] setTitle: _(@"New message...")];
	  [editWindowController setMessageFromDraftsFolder: aMessage];
	  [editWindowController setShowCc: NO];
	  
	  [editWindowController showWindow: self];
	}
      
      RELEASE(aMessage);
    }
  else
    {
      NSBeep();
    }
}


//
// This method is used to select all the messages from the last
// mail window that was on top.
//
- (IBAction) selectAllMessages: (id) sender
{  
  if ( [GNUMail lastMailWindowOnTop] )
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      [[aMailWindowController tableView] selectAll: sender];
      [[aMailWindowController tableView] setNeedsDisplay: YES];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) saveInDrafts: (id) sender
{
  if ( [GNUMail lastAddressTakerWindowOnTop] )
    {
      EditWindowController *aController;

      NSString *aFolderName;
      NSData *aData;
      
      aController = [GNUMail lastAddressTakerWindowOnTop];
            
      [aController updateMessageContentFromTextView: NO];
      
      aFolderName = [[NSUserDefaults standardUserDefaults] objectForKey: @"DRAFTSFOLDERNAME"];
      aData = [[aController message] dataUsingSendingMode: SEND_TO_FOLDER];
      
      [[MailboxManagerController singleInstance] transferMessageFromRawSource: aData
						 flags: nil
						 toFolderWithName: aFolderName
						 orToFolder: nil
						 store: [[MailboxManagerController singleInstance]
							  storeForName: @"GNUMAIL_LOCAL_STORE"
							  username: NSUserName()]];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) saveAttachment: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;
      NSTextAttachment *attachment;

      attachment = [sender textAttachment];
      
      // We get a reference to our mail window controller
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];

      [aWindowController textView: [aWindowController textView]
			    clickedOnCell: [attachment attachmentCell]
			    inRect: NSZeroRect
			    atIndex: 0];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) saveAllAttachments: (id) sender
{
  NSTextAttachment *aTextAttachment;
  NSFileWrapper *aFilewrapper;
  NSSavePanel *aSavePanel;
  
  BOOL useSameDir, ask;
  int i;

  aSavePanel = [NSSavePanel savePanel];
  [aSavePanel setAccessoryView: nil];
  [aSavePanel setRequiredFileType: @""];

  useSameDir = NO;
  ask = YES;

  for ( i = 1; i < [[(GNUMail *)[NSApp delegate] saveAttachmentMenu] numberOfItems]; i++ )
    {
      aTextAttachment = [(ExtendedMenuItem *)[[(GNUMail *)[NSApp delegate] saveAttachmentMenu] itemAtIndex: i] 
					     textAttachment];
      aFilewrapper = [aTextAttachment fileWrapper];
      
      if ( !useSameDir )
        {
          int aChoice;

          aChoice = [aSavePanel runModalForDirectory: [GNUMail currentWorkingPath]
                                file: [aFilewrapper preferredFilename]];
	  
          // if successful, save file under designated name
          if ( aChoice == NSOKButton )
            {
              if ( ![aFilewrapper writeToFile: [aSavePanel filename]
				  atomically: YES
				  updateFilenames: YES] )
                {
                  NSBeep();
                }
	      
              [GNUMail setCurrentWorkingPath: [[aSavePanel filename] stringByDeletingLastPathComponent]];
            }
        }
      else
        {
          // We save the file in the same directory
          if ( ![aFilewrapper writeToFile: [[GNUMail currentWorkingPath] stringByAppendingPathComponent:
									   [aFilewrapper preferredFilename]]
			      atomically: YES
			      updateFilenames: YES] )
            {
              NSBeep();
            }
        }

      if ( ask )
        {
          int use = NSRunAlertPanel(_(@"Information"),
                                    _(@"Use the same directory (%@) to save all other attachments? (override the files with the same name)."),
                                    _(@"Yes"),
                                    _(@"No"),
                                    NULL,
                                    [GNUMail currentWorkingPath]);
          
	  if ( use == NSAlertDefaultReturn )
            {
              useSameDir = YES;
            }
          else if ( use == NSAlertAlternateReturn )
            {
              useSameDir = NO;
            }
	  
          ask = NO;
        }
    }
}



//
// This method is used to save only the textual content of a message
// to the local file system. It skips attachments.
//
- (IBAction) saveTextFromMessage: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;
      NSSavePanel *aSavePanel;
      NSData *aData;
      int aChoice;

      // We get a reference to our mail window controller
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      
      if ( [aWindowController selectedMessage] )
	{
	  // We get our content of a message (just the text displayed in mail window)
	  aData = [[[aWindowController textView] string] 
		    dataUsingEncoding: NSASCIIStringEncoding
		    allowLossyConversion: YES];
	  
	  aSavePanel = [NSSavePanel savePanel];
	  [aSavePanel setAccessoryView: nil];
	  [aSavePanel setRequiredFileType: @""];
	  
	  aChoice = [aSavePanel runModalForDirectory: [GNUMail currentWorkingPath] file: @"Unknown"];
      
	  /* if successful, save file under designated name */
	  if (aChoice == NSOKButton)
	    {
	      if (![aData writeToFile: [aSavePanel filename]
			  atomically: YES] )
		{
		  NSBeep();
		}

              [GNUMail setCurrentWorkingPath: [[aSavePanel filename] stringByDeletingLastPathComponent]];
	    }
	}
      else
	{
	  NSBeep();
	}
    }
  else
    {
      NSBeep();
    }
}

//
//
//
#ifdef MACOSX
- (IBAction) showAboutPanel: (id) sender
{
    NSDictionary *aDictionary = [NSDictionary dictionaryWithObjectsAndKeys:
        @"GNUMail.app", 			                     @"ApplicationName",
        [NSImage imageNamed: @"GNUMail.tiff"],                       @"ApplicationIcon",
        GNUMAIL_VERSION,					     @"ApplicationVersion",
        @"1999, 2000, 2001, 2002 Ludovic Marcotte <ludovic@Sophos.ca> and contributors", @"Copyright",
        GNUMAIL_VERSION,					     @"Version",
        nil];

    [NSApp orderFrontStandardAboutPanelWithOptions: aDictionary];
}
#endif

//
//
//
- (IBAction) sortByNumber: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      [aMailWindowController
	tableView: [aMailWindowController tableView]
	didClickTableColumn: [[aMailWindowController tableView]
			       tableColumnWithIdentifier: @"Id"]];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) sortByDate: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      [aMailWindowController
	tableView: [aMailWindowController tableView]
	didClickTableColumn: [[aMailWindowController tableView]
			       tableColumnWithIdentifier: @"Date"]];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) sortByName: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      [aMailWindowController
	tableView: [aMailWindowController tableView]
	didClickTableColumn: [[aMailWindowController tableView]
			       tableColumnWithIdentifier: @"From"]];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) sortBySubject: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      [aMailWindowController
	tableView: [aMailWindowController tableView]
	didClickTableColumn: [[aMailWindowController tableView]
			       tableColumnWithIdentifier: @"Subject"]];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) sortBySize: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] ) 
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];
      
      [aMailWindowController
	tableView: [aMailWindowController tableView]
	didClickTableColumn: [[aMailWindowController tableView]
			       tableColumnWithIdentifier: @"Size"]];
    }
  else
    {
      NSBeep();
    }
}



//
// This method is used to show the address book on the screen.
// We use a singleton.
//
- (IBAction) showAddressBook: (id) sender
{
  [[AddressBookController singleInstance] showWindow: self];
}


//
// This method is used to view or hide all headers in the 
// last mail window on top.
//
- (IBAction) showAllHeaders: (id) sender
{ 
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;
      Message *aMessage;
      
      BOOL aBOOL;
      int row;
      
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      row = -1;

      if ( [aWindowController isKindOfClass: [MailWindowController class]] &&
	   (row = [[aWindowController tableView] selectedRow]) &&
	   (row == -1 || [[aWindowController tableView] numberOfSelectedRows] > 1) )
	{
	  NSBeep();
	  return;
	}
      
      if ( [sender tag] == SHOW_ALL_HEADERS )
	{
	  aBOOL = YES;
	  [aWindowController setShowAllHeaders: aBOOL];
          
          if ( [sender isKindOfClass: [NSButton class]] ||
               [sender isKindOfClass: [NSMenuItem class]] )
            {
	      [sender setTitle: _(@"Show Filtered Headers")];
            }
          else
            {
              [sender setLabel: _(@"Show Filtered Headers")];
            }
        
	  [sender setTag: HIDE_ALL_HEADERS];
	}
      else
	{
	  aBOOL = NO;
	  [aWindowController setShowAllHeaders: aBOOL];	
          
          if ( [sender isKindOfClass: [NSButton class]] ||
               [sender isKindOfClass: [NSMenuItem class]] )
            {
	      [sender setTitle: _(@"Show All Headers")];
            }
          else
            {
              [sender setLabel: _(@"Show All Headers")];
            }

	  [sender setTag: SHOW_ALL_HEADERS];
	}
      
      [menu sizeToFit];

      if ( [aWindowController isKindOfClass: [MailWindowController class]] )
	{
	  aMessage = [[aWindowController allMessages] objectAtIndex: row];
	}
      else
	{
	  aMessage = [aWindowController message];
	}
      
      [Utilities showMessage: aMessage
		 target: [aWindowController textView]
		 showAllHeaders: aBOOL];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) showConsoleWindow: (id) sender
{
  [[ConsoleWindowController singleInstance] showWindow: self];
}

//
// This method is used to show the find window on the screen.
// We use a singleton.
//
- (IBAction) showFindWindow: (id) sender
{
  [[FindWindowController singleInstance] showWindow: self];
}


//
// This method is used to show the mailbox manager window on the screen.
// We use a singleton.
//
- (IBAction) showMailboxManager: (id) sender
{
  [[MailboxManagerController singleInstance] showWindow: self];
}


//
//
//
- (IBAction) showOrHideDeletedMessages: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];

      if ( [showOrHideDeleted tag] == SHOW_DELETED_MESSAGES )
	{
	  [self updateShowOrHideDeletedMenuItem: YES];
	  [[aMailWindowController folder] setShowDeleted: YES];
	}
      else
	{
	  [self updateShowOrHideDeletedMenuItem: NO];
	  [[aMailWindowController folder] setShowDeleted: NO];
	}
      
      [aMailWindowController tableViewShouldReloadData];
      [aMailWindowController updateStatusLabel];
    }
  else
    {
      NSBeep();
    }
}


//
//
//
- (IBAction) showOrHideReadMessages: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];

      if ( [showOrHideRead tag] == SHOW_READ_MESSAGES )
	{
	  [self updateShowOrHideReadMenuItem: YES];
	  [[aMailWindowController folder] setShowRead: YES];
	}
      else
	{
	  [self updateShowOrHideReadMenuItem: NO];
	  [[aMailWindowController folder] setShowRead: NO];
	}
      
      [aMailWindowController tableViewShouldReloadData];
      [aMailWindowController updateStatusLabel];
    }
  else
    {
      NSBeep();
    }
}

//
//
//
- (IBAction) showOrHideStatusBarForMailWindow: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      MailWindowController *aMailWindowController;
      
      aMailWindowController = (MailWindowController *)[[GNUMail lastMailWindowOnTop] delegate];

      if ( [showOrHideStatusBar tag] == SHOW_STATUS_BAR )
	{
	  [showOrHideStatusBar setTitle: _(@"Hide Status Bar")];
	  [showOrHideStatusBar setTag: HIDE_STATUS_BAR];
	  [aMailWindowController setShowStatusBar: YES];
	}
      else
	{
	  [showOrHideStatusBar setTitle: _(@"Show Status Bar")];
	  [showOrHideStatusBar setTag: SHOW_STATUS_BAR];
	  [aMailWindowController setShowStatusBar: NO];
	}
      
      [aMailWindowController tableViewShouldReloadData];
    }
  else
    {
      NSBeep();
    }
}


//
// This method is used to show the preferences window on the screen.
// The window is modal.
//
- (IBAction) showPreferencesWindow: (id) sender
{
  [[PreferencesWindowController singleInstance] showWindow: self];
}


//
// This method is used to show the raw source of the selected message
// from the last mail window on top.
//
- (IBAction) showRAWSource: (id) sender
{
  if ( [GNUMail lastMailWindowOnTop] )
    {
      id aWindowController;
      Message *aMessage;
      
      aWindowController = [[GNUMail lastMailWindowOnTop] delegate];
      
      if ( !aWindowController )
	{
	  NSBeep();
	  return;
	}
      
      if ( [aWindowController isKindOfClass: [MailWindowController class]] )
	{
	  aMessage = [aWindowController selectedMessage];
	}
      else
	{
	  aMessage = [aWindowController message];
	}
      
      if ( aMessage )
	{
	  if ( ![aWindowController showRawSource] )
	    {
	      [Utilities showMessageRawSource: aMessage target: [aWindowController textView]];
	      [aWindowController setShowRawSource: YES];
	      [showRawSource setTitle: _(@"Show Normal Display")];
	    }
	  else
	    {
	      [Utilities showMessage: aMessage
			 target: [aWindowController textView]
			 showAllHeaders: [aWindowController showAllHeaders]];
	      [aWindowController setShowRawSource: NO];
	      [showRawSource setTitle: _(@"Show RAW Source")];
	    }
	}
      else
	{
	  NSBeep();
	}
      
    }
  else
    {
      NSBeep();
    }
}


//
// delegate methods
//
- (BOOL) applicationShouldTerminate: (NSApplication *) theSender
{
  // Closes all opened MailWindow and terminates the application
  [[NSNotificationCenter defaultCenter]
    postNotificationName: ShouldCloseMailbox
    object: nil
    userInfo: nil];
  
  // We remove our observer on message selection changes
  [[NSNotificationCenter defaultCenter]
    removeObserver: self
    name: SelectionOfMessageHasChanged
    object: nil];
  
  // We remove our observer on the filter changes
  [[NSNotificationCenter defaultCenter]
    removeObserver: self
    name: FiltersHaveChanged
    object: nil];
    

  // We wait until all our windows are closed
  while ( [allMailWindows count] > 0 )
    {
      [[NSRunLoop currentRunLoop] runMode: NSDefaultRunLoopMode
				  beforeDate: [NSDate distantFuture]];
    }

  // We release our array containing all our mail windows
  RELEASE(allMailWindows);
  allMailWindows = nil;

  // We close all Stores
  RELEASE([MailboxManagerController singleInstance]);

  // We close our Console
  RELEASE([ConsoleWindowController singleInstance]);
  
  // We release our current working path
  TEST_RELEASE(currentWorkingPath);

  // We release our array containing all our bundles
  RELEASE(allBundles);

  // We release our array containing our password cache
  RELEASE([Utilities passwordCache]);

  return YES;
}


//
//
//
- (void) applicationWillFinishLaunching: (NSNotification *) theNotification
{
#ifndef MACOSX
  // Local variable
  SEL action = NULL;
  
  // We begin by setting our NSApp's logo
  [NSApp setApplicationIconImage: [NSImage imageNamed: @"GNUMail.tiff"]];

  // We continue by creating our NSMenu
  menu = [[NSMenu alloc] init];
  
  [menu addItemWithTitle:_(@"Info") 
	action: action 
	keyEquivalent: @""];
  [menu addItemWithTitle:_(@"Mailbox") 
	action: action 
	keyEquivalent: @""];
  [menu addItemWithTitle:_(@"Message") 
	action: action 
	keyEquivalent: @""];
  [menu addItemWithTitle:_(@"Compose")
	action: action 
	keyEquivalent: @""];
  [menu addItemWithTitle:_(@"Edit")
	action: action
	keyEquivalent: @"h"];
  [menu addItemWithTitle:_(@"Windows")
	action: action
	keyEquivalent: @"p"];
  [menu addItemWithTitle:_(@"Print")
	action: action
	keyEquivalent: @"p"];
  [menu addItemWithTitle:_(@"Services")
	action: action
	keyEquivalent: @""];
  [menu addItemWithTitle:_(@"Hide")
	action: @selector (hide:)
	keyEquivalent: @"h"];
  [menu addItemWithTitle:_(@"Quit")
	action: @selector(terminate:)
	keyEquivalent: @"q"];
  
  // Our Info menu / submenus
  info = [[NSMenu alloc] init];
  [menu setSubmenu:info forItem:[menu itemWithTitle:_(@"Info")]];
  [info addItemWithTitle:_(@"Info Panel...")
	action: @selector(orderFrontStandardInfoPanel:)   
	keyEquivalent:@""];
  [info addItemWithTitle:_(@"Check For Updates")
	action: @selector(checkForUpdates:)   
	keyEquivalent:@""];
  [info addItemWithTitle:_(@"Preferences...")
	action:@selector(showPreferencesWindow:)
	keyEquivalent:@""];
  [info addItemWithTitle:_(@"Help...")
	action: action
	keyEquivalent:@"?"];
  RELEASE(info);

  // Our Mailbox menu / submenus
  mailbox = [[NSMenu alloc] init];
  [menu setSubmenu:mailbox forItem:[menu itemWithTitle:_(@"Mailbox")]];
  [mailbox addItemWithTitle: _(@"Mailboxes...")
	   action:@selector(showMailboxManager:) 
	   keyEquivalent: @"M"];

  incomingMailServers = [[NSMenu alloc] init];
  [mailbox addItemWithTitle: _(@"Get New Mail")
	   action: action
	   keyEquivalent: @""];
  [[mailbox itemAtIndex: 1] setSubmenu: incomingMailServers];
  RELEASE(incomingMailServers);

  [mailbox addItemWithTitle: _(@"Sorting")
	   action: action 
	   keyEquivalent: @""];

  // We create our sorting menu
  sorting = [[NSMenu alloc] init];
  [[mailbox itemAtIndex: 2] setSubmenu: sorting];
  
  [sorting addItemWithTitle: _(@"Sort by Date")
	   action: @selector(sortByDate:) 
	   keyEquivalent: @""];
  [sorting addItemWithTitle: _(@"Sort by Name")
	   action: @selector(sortByName:)
	   keyEquivalent: @"S"];
  [sorting addItemWithTitle: _(@"Sort by Number")
	   action: @selector(sortByNumber:)
	   keyEquivalent: @""];
  [sorting addItemWithTitle: _(@"Sort by Size")
	   action: @selector(sortBySize:)
	   keyEquivalent: @""];
  [sorting addItemWithTitle: _(@"Sort by Subject")
	   action: @selector(sortBySubject:)
	   keyEquivalent: @""];
  RELEASE(sorting);
  
  // We add the rest of our mailbox menu items
  showOrHideDeleted = [[NSMenuItem alloc] init];
  [showOrHideDeleted setTitle: _(@"Hide Deleted")]; // and Show Deleted
  [showOrHideDeleted setAction: @selector(showOrHideDeletedMessages:)];
  [showOrHideDeleted setKeyEquivalent: @""];
  [showOrHideDeleted setTag: HIDE_DELETED_MESSAGES];
  [mailbox addItem: showOrHideDeleted];
  RELEASE(showOrHideDeleted);
	  
  showOrHideRead = [[NSMenuItem alloc] init];
  [showOrHideRead setTitle: _(@"Hide Read")]; // and Show Read
  [showOrHideRead setAction: @selector(showOrHideReadMessages:)];
  [showOrHideRead setKeyEquivalent: @""];
  [showOrHideRead setTag: HIDE_READ_MESSAGES];
  [mailbox addItem: showOrHideRead];
  RELEASE(showOrHideRead);

  showOrHideStatusBar = [[NSMenuItem alloc] init];
  [showOrHideStatusBar setTitle: _(@"Hide Status Bar")]; // and Show Sizes
  [showOrHideStatusBar setAction: @selector(showOrHideStatusBarForMailWindow:)];
  [showOrHideStatusBar setKeyEquivalent: @""];
  [showOrHideStatusBar setTag: HIDE_STATUS_BAR];
  [mailbox addItem: showOrHideStatusBar];
  RELEASE(showOrHideStatusBar);

  [mailbox addItemWithTitle: _(@"Compact")
	   action: @selector(compactMailbox:)
	   keyEquivalent: @"C"];
  RELEASE(mailbox);
  
  // Our Message menu / submenus
  message = [[NSMenu alloc] init];
  [menu setSubmenu:message forItem:[menu itemWithTitle:_(@"Message")]];

  [message addItemWithTitle:_(@"Select All Messages")
	   action: @selector(selectAllMessages:)
	   keyEquivalent: @""];
  
  showAllHeaders = [[NSMenuItem alloc] init];
  [showAllHeaders setTitle: _(@"Show All Headers")];
  [showAllHeaders setAction: @selector(showAllHeaders:)];
  [showAllHeaders setKeyEquivalent: @""];
  [showAllHeaders setTag: SHOW_ALL_HEADERS];
  [message addItem: showAllHeaders];
  RELEASE(showAllHeaders);

  showRawSource = [[NSMenuItem alloc] init];
  [showRawSource setTitle: _(@"Show RAW Source")];
  [showRawSource setAction: @selector(showRAWSource:)];
  [showRawSource setKeyEquivalent: @""];
  [message addItem: showRawSource];
  RELEASE(showRawSource);

  //[message addItemWithTitle:_(@"Show RAW Source")
  //	   action:@selector(showRAWSource:)  
  //	   keyEquivalent: @""];

  [message addItemWithTitle:_(@"Send to Address Book")
	   action: action
	   keyEquivalent: @""];
  
  sendToAddressBook = [[NSMenu alloc] init];
  [message setSubmenu: sendToAddressBook
	   forItem: [message itemWithTitle: _(@"Send to Address Book")]];
  RELEASE(sendToAddressBook);

  [message addItemWithTitle: _(@"Save Attachment")
	   action: action
	   keyEquivalent: @""];
  
  saveAttachment = [[NSMenu alloc] init];
  [message setSubmenu: saveAttachment
	   forItem: [message itemWithTitle: _(@"Save Attachment")]];
  RELEASE(saveAttachment);

  [message addItemWithTitle: _(@"Save Text from Message")
	   action: @selector(saveTextFromMessage:)  
	   keyEquivalent: @""];
  
  
  [message addItemWithTitle: _(@"Text Encodings")
	   action: action
	   keyEquivalent: @""];
  
  textEncodings = [[NSMenu alloc] init];
  [message setSubmenu: textEncodings
	   forItem: [message itemWithTitle: _(@"Text Encodings")]];
  RELEASE(textEncodings);
  
  markAsReadOrUnread = [[NSMenuItem alloc] init];
  [markAsReadOrUnread setTitle: _(@"Mark as Read")];
  [markAsReadOrUnread setAction: @selector(markMessageAsReadOrUnread:)];
  [markAsReadOrUnread setKeyEquivalent: @""];
  [message addItem: markAsReadOrUnread];
  RELEASE(markAsReadOrUnread);
  
  RELEASE(message);

  deleteOrUndelete = [[NSMenuItem alloc] init];
  [deleteOrUndelete setTitle: _(@"Undelete")];
  [deleteOrUndelete setAction: @selector(deleteOrUndeleteMessage:)];
  [deleteOrUndelete setKeyEquivalent: @"u"];
  [message addItem: deleteOrUndelete];
  RELEASE(deleteOrUndelete);

  [message addItemWithTitle: _(@"Bounce...")
	   action:@selector(bounceMessage:)  
	   keyEquivalent: @""];

  // We create our 'filters' menu item / menu
  filters = [[NSMenu alloc] init];
  [message addItemWithTitle: _(@"Apply Manual Filters")
	   action: action
	   keyEquivalent: @""];
  [[message itemAtIndex: 10] setSubmenu: filters];
  RELEASE(filters);
  
  [[message itemAtIndex: 0] setTag: SHOW_ALL_HEADERS];
  
  // Our Compose menu / submenus
  compose = [[NSMenu alloc] init];
  [menu setSubmenu:compose forItem:[menu itemWithTitle:_(@"Compose")]];
  [compose addItemWithTitle: _(@"New")
	   action:@selector(composeMessage:) 
	   keyEquivalent:@"n"];
  [compose addItemWithTitle: _(@"Reply")
	   action:@selector(replyMessage:)  
	   keyEquivalent:@"R"];
  [compose addItemWithTitle: _(@"Reply All")
	   action:@selector(replyAllMessage:)   
	   keyEquivalent:@"E"];
  [compose addItemWithTitle: _(@"Forward")
	   action:@selector(forwardMessage:)  
	   keyEquivalent:@"W"];
  [compose addItemWithTitle: _(@"Addresses...")
	   action: @selector(showAddressBook:)
	   keyEquivalent:@"A"];
  [compose addItemWithTitle: _(@"Drafts")
	   action: action
	   keyEquivalent:@""];
  
  drafts = [[NSMenu alloc] init];
  
  saveInDrafts = [[NSMenuItem alloc] initWithTitle: _(@"Save in Drafts")
				     action: NULL
				     keyEquivalent: @"s"];
  [drafts addItem: saveInDrafts];
  RELEASE(saveInDrafts);
  
  [drafts addItemWithTitle: _(@"Restore Draft")
	  action: @selector(restoreDraft:)
	  keyEquivalent: @""];
  [[compose itemAtIndex: 5] setSubmenu: drafts];
  RELEASE(drafts);
  RELEASE(compose);
  
  // Our Edit menu / submenus
  edit = [[NSMenu alloc] init];
  [menu setSubmenu: edit forItem: [menu itemWithTitle: _(@"Edit")]];

  [edit addItemWithTitle: _(@"Cut")
	action: @selector(cut:)
	keyEquivalent: @"x"];
  [edit addItemWithTitle: _(@"Copy")
	action: @selector(copy:)
	keyEquivalent: @"c"];
  [edit addItemWithTitle: _(@"Paste")
	action: @selector(paste:)
	keyEquivalent: @"v"];
  [edit addItemWithTitle: _(@"Delete")
	action: @selector(delete:)
	keyEquivalent: @""];
  [edit addItemWithTitle: _(@"Undo")
	action: action
	keyEquivalent: @"z"];

  // Our find submenu and its submenus
  [edit addItemWithTitle: _(@"Find")
  	action: action
  	keyEquivalent: @""];

  [edit addItemWithTitle: _(@"Spelling...")
	action: @selector(checkSpelling:)
	keyEquivalent: @":"];

  [edit addItemWithTitle: _(@"Check Spelling")
	action: @selector(showGuessPanel:)
	keyEquivalent: @";"];

  [edit addItemWithTitle: _(@"Select All")
	action: @selector(selectAll:)
	keyEquivalent: @"a"];
  RELEASE(edit);
  
  find = [[NSMenu alloc] init];
  [[edit itemAtIndex: 5] setSubmenu:find];

  [find addItemWithTitle: _(@"Find Panel...")
	action: @selector(showFindWindow:)
	keyEquivalent: @"f"];
  [find addItemWithTitle: _(@"Find Next")
	action: @selector(findNext:)
	keyEquivalent: @"g"];
  [find addItemWithTitle: _(@"Find Previous")
	action: @selector(findPrevious:)
	keyEquivalent: @"d"];

  enterSelection = [[NSMenuItem alloc] initWithTitle: _(@"Enter Selection")
				       action: action
				       keyEquivalent: @"e"];
  [find addItem: enterSelection];
  RELEASE(enterSelection);

  RELEASE(find);

  // Our Windows menu
  windows = [[NSMenu alloc] init];
  [menu setSubmenu:windows forItem: [menu itemWithTitle:_(@"Windows")]];
  [windows addItemWithTitle: _(@"Console")
	   action: @selector(showConsoleWindow:)
	   keyEquivalent: @""];
  [windows addItemWithTitle: _(@"Arrange")
	   action: @selector(arrangeInFront:)
	   keyEquivalent: @""];
  [windows addItemWithTitle: _(@"Miniaturize")
	   action: @selector(performMiniaturize:)
	       keyEquivalent: @"m"];
  [windows addItemWithTitle: _(@"Close")
	   action: @selector(close:)
	   keyEquivalent: @"w"];
  RELEASE(windows);
  
  // Our Print menu / submenus
  print = [[NSMenu alloc] init];
  [menu setSubmenu: print forItem: [menu itemWithTitle: _(@"Print")]];
  [print addItemWithTitle: _(@"Print")
	 action: @selector(print:)
	 keyEquivalent: @"p"];
  [print addItemWithTitle: _(@"Setup Printer")
	 action:action
	 keyEquivalent: @"P"];
  RELEASE(print);

  // Our Services menu
  services = [[NSMenu alloc] init];
  [menu setSubmenu: services forItem: [menu itemWithTitle: _(@"Services")]];
  [NSApp setServicesMenu: services];
  RELEASE(services);

  [NSApp setMainMenu: menu];
  [NSApp setWindowsMenu: windows];

  [menu setTitle: @"GNUMail.app"];
  RELEASE(menu);
#endif
}


//
//
//
- (void) applicationDidFinishLaunching: (NSNotification *) theNotification
{
  NSString *pathToLocalMailDir;

  NSFileManager *fileManager;
  BOOL isDir;

  LocalStore *aLocalStore;

  //[NSApp hide: self];

  fileManager = [NSFileManager defaultManager];
  
  // We now verify if the User's Library directory does exist (to store 
  // the AddressBook, the MimeTypes, etc) and if not, we create it.
  if ( [fileManager fileExistsAtPath: (NSString *)GNUMailUserLibraryPath()
		    isDirectory: &isDir] )
    {
      if ( !isDir )
	{
	  NSDebugLog(@"%@ exists but it is a file not a directory.",
		GNUMailUserLibraryPath());
	  exit(1);
	}
    }
  else    {
    if ( ![fileManager createDirectoryAtPath: (NSString *)GNUMailUserLibraryPath()
			  attributes: nil] )
      {
	// directory creation failed.  quit.
	NSDebugLog(@"Could not create directory: %@", GNUMailUserLibraryPath());
	exit(1);
      }
    else
      {
	NSDebugLog(@"Created directory: %@", GNUMailUserLibraryPath());
      }
  }
  
  // We register for our notifications
  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(selectionOfMessageHasChanged:)
    name: SelectionOfMessageHasChanged
    object: nil];

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(updateGetNewMailMenuItems:)
    name: ReceivingValuesHaveChanged
    object: nil];

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(updateFilterMenuItems:)
    name: FiltersHaveChanged
    object: nil];

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(selectionInTextViewHasChanged:)
    name: NSTextViewDidChangeSelectionNotification
    object: nil];

  // We initialize our mutable array containing all opened mail windows
  allMailWindows = [[NSMutableArray alloc] init];
  
  // We initialize our mutable array containing all our bundles
  allBundles = [[NSMutableArray alloc] init];

  [GNUMail setCurrentWorkingPath: NSHomeDirectory()];
  

  // Setup our mailbox locations, if LOCALMAILDIR isn't defined, we define
  // it to ~/gnumail.
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"LOCALMAILDIR"] )
    {
      pathToLocalMailDir = [NSHomeDirectory() stringByAppendingPathComponent: @"gnumail"];
      
      [[NSUserDefaults standardUserDefaults]
	setObject: pathToLocalMailDir
	forKey: @"LOCALMAILDIR"];
    }
  else
    {
      pathToLocalMailDir = [[NSUserDefaults standardUserDefaults] objectForKey: @"LOCALMAILDIR"];
    }
  
  //
  // Inbox
  //
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"INBOXFOLDERNAME"] )
    {
      [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/Inbox", pathToLocalMailDir]
					     forKey: @"INBOXFOLDERNAME"];
    }
  else
    {
      // Let's see if we need to add local:// to it
      NSString *aFolderName;
      URLName *theURLName;

      aFolderName = [[NSUserDefaults standardUserDefaults] objectForKey: @"INBOXFOLDERNAME"];
      theURLName = [[URLName alloc] initWithString: aFolderName];
      
      if ( ![theURLName protocol] )
	{
	  [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/%@", pathToLocalMailDir, aFolderName]
						 forKey: @"INBOXFOLDERNAME"];
	}

      RELEASE(theURLName);
    }

  //
  // Outbox
  //
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"OUTBOXFOLDERNAME"] )
    {
      [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/Outbox", pathToLocalMailDir]
					     forKey: @"OUTBOXFOLDERNAME"];
    }
  else
    {
      // Let's see if we need to add local:// to it
      NSString *aFolderName;
      URLName *theURLName;

      aFolderName = [[NSUserDefaults standardUserDefaults] objectForKey: @"OUTBOXFOLDERNAME"];
      theURLName = [[URLName alloc] initWithString: aFolderName];
      
      if ( ![theURLName protocol] )
	{
	  [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/%@", pathToLocalMailDir, aFolderName]
						 forKey: @"OUTBOXFOLDERNAME"];
	}

      RELEASE(theURLName);
    }
  
  //
  // Trash
  //
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"TRASHFOLDERNAME"] )
    {
      [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/Trash", pathToLocalMailDir]
					     forKey: @"TRASHFOLDERNAME"];
    }
  else
    {
      // Let's see if we need to add local:// to it
      NSString *aFolderName;
      URLName *theURLName;

      aFolderName = [[NSUserDefaults standardUserDefaults] objectForKey: @"TRASHFOLDERNAME"];
      theURLName = [[URLName alloc] initWithString: aFolderName];
      
      if ( ![theURLName protocol] )
	{
	  [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/%@", pathToLocalMailDir, aFolderName]
						 forKey: @"TRASHFOLDERNAME"];
	}

      RELEASE(theURLName);
    }
  
  //
  // Drafts
  //
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"DRAFTSFOLDERNAME"] )
    {
      [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/Drafts", pathToLocalMailDir]
					     forKey: @"DRAFTSFOLDERNAME"];
    }
  else
    {
      // Let's see if we need to add local:// to it
      NSString *aFolderName;
      URLName *theURLName;

      aFolderName = [[NSUserDefaults standardUserDefaults] objectForKey: @"DRAFTSFOLDERNAME"];
      theURLName = [[URLName alloc] initWithString: aFolderName];

      if ( ![theURLName protocol] )
	{
	  [[NSUserDefaults standardUserDefaults] setObject: [NSString stringWithFormat: @"local://%@/%@", pathToLocalMailDir, aFolderName]
						 forKey: @"DRAFTSFOLDERNAME"];
	}

      RELEASE(theURLName);
    }

  //
  // From 1.0.2 -> 1.0.3, we convert the MAILSPOOLFILE into a RECEIVING preference
  //
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"TRANSFERFROMMAILSPOOLFILE"] &&
       [[NSUserDefaults standardUserDefaults] integerForKey: @"TRANSFERFROMMAILSPOOLFILE"] == NSOnState )
    {
      NSMutableDictionary *allServers, *aMutableDictionary;

      aMutableDictionary = [[NSMutableDictionary alloc] init];

      allServers = [[NSMutableDictionary alloc] init];
      [allServers addEntriesFromDictionary: [[NSUserDefaults standardUserDefaults] objectForKey: @"RECEIVING"] ];
      

      [aMutableDictionary setObject: [[NSUserDefaults standardUserDefaults] objectForKey: @"MAILSPOOLFILE"]
			  forKey: @"MAILSPOOLFILE"];
      [aMutableDictionary setObject: [[NSUserDefaults standardUserDefaults] objectForKey: @"INBOXFOLDERNAME"]
			  forKey: @"MAILBOXNAME"];
      [aMutableDictionary setObject: [NSNumber numberWithInt: UNIX]
			  forKey: @"SERVERTYPE"];
      
      [allServers setObject: aMutableDictionary
		  forKey: [NSString stringWithFormat: @"local @ %@", 
				    [[NSUserDefaults standardUserDefaults] objectForKey: @"MAILSPOOLFILE"]]];
      
      [[NSUserDefaults standardUserDefaults] setObject: allServers
					     forKey: @"RECEIVING"];

      RELEASE(aMutableDictionary);
      RELEASE(allServers);
    }

  //
  // We remove deprecated preferences values
  //
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"SHOWICONS"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"SHOWWAITPANELS"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"RECT_EDITWINDOW"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"RECT_MAILWINDOW"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"SIGNATURE"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"SIGNATURE_SOURCE"];
  
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"TAG_FONT_NAME"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"TAG_FONT_SIZE"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"HEADER_FONT_NAME"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"HEADER_FONT_SIZE"];

  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"MAILSPOOLFILE"];
  [[NSUserDefaults standardUserDefaults] removeObjectForKey: @"TRANSFERFROMMAILSPOOLFILE"];

  // We verify if GNUMail.app has been configured, if not, we suggest the user to do so
  if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"PERSONAL"] )
    {
      NSMutableDictionary *aMutableDictionary;
      URLName *theURLName;
      int choice;
      BOOL isDir;
      
      choice = NSRunAlertPanel(_(@"Important!"),
			       _(@"No configuration found for GNUMail.app"),
			       _(@"Configure"), // default
			       _(@"Cancel"),    // alternate
			       NULL);
      
      // If the user doesn't want to configure GNUMail.app, we simply quit 
      if (choice == NSAlertAlternateReturn) 
	{
	  [NSApp terminate: self];
	}
      
      // We create the mailboxes directory for GNUMail.app, if we need to!
      if ( ![[NSFileManager defaultManager]
	      fileExistsAtPath: pathToLocalMailDir
	      isDirectory: &isDir] )
	{
	  [[NSFileManager defaultManager] createDirectoryAtPath:
					    [NSHomeDirectory() stringByAppendingPathComponent: @"gnumail"]
					  attributes: nil];
	}
      else
	{
	  // If it's a file
	  if ( !isDir )
	    {
	      NSDebugLog(@"%@ is a file and not a directory. Please remove that file before restarting GNUMail.app",
		    pathToLocalMailDir);
	      [NSApp terminate: self];
	    }
	}
      
      //
      // We create the following Mailboxes automatically (only if we need to!):
      //
      // - Inbox  : for the newly received messages
      // - Outbox : for the messages that have been sent
      // - Trash  : for the messages we want to delete and transfer locally in IMAP
      // - Drafts : for un-sent messages
      //

      // Inbox
      theURLName = [[URLName alloc] initWithString: [[NSUserDefaults standardUserDefaults] objectForKey: @"INBOXFOLDERNAME"]
				    path: pathToLocalMailDir];

      if ( ![[NSFileManager defaultManager] fileExistsAtPath:
					      [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]] )
	{
	  // We first verify if we can find our "Welcome" mailbox. If so, the Inbox will be created
	  // with the content of that file.
	  NSString *pathToFile;
	  
	  pathToFile = [NSString stringWithFormat: @"%@/Welcome", [[NSBundle mainBundle] resourcePath]];
	  
	  if ( [[NSFileManager defaultManager] fileExistsAtPath: pathToFile] )
	    {
	      [[NSFileManager defaultManager] copyPath: pathToFile
					      toPath: [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]
					      handler: nil];
	    }
	  else
	    {
	      [[NSFileManager defaultManager] createFileAtPath: [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]
					      contents: nil
					      attributes: nil];
	    }
	}
      RELEASE(theURLName);
      
      
      // Outbox
      theURLName = [[URLName alloc] initWithString: [[NSUserDefaults standardUserDefaults] objectForKey: @"OUTBOXFOLDERNAME"]
				    path: pathToLocalMailDir];
      if ( ![[NSFileManager defaultManager] fileExistsAtPath:
					      [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]]  )
	{
	  [[NSFileManager defaultManager] createFileAtPath: [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]
					  contents: nil
					  attributes: nil];
	}
      RELEASE(theURLName);
      
      
      // Trash
      theURLName = [[URLName alloc] initWithString: [[NSUserDefaults standardUserDefaults] objectForKey: @"TRASHFOLDERNAME"]
				    path: pathToLocalMailDir];
      if ( ![[NSFileManager defaultManager] fileExistsAtPath:
					      [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]] )
	{
	  [[NSFileManager defaultManager] createFileAtPath: [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]
					  contents: nil
					  attributes: nil];
	}
      RELEASE(theURLName);


      // Drafts
      theURLName = [[URLName alloc] initWithString: [[NSUserDefaults standardUserDefaults] objectForKey: @"DRAFTSFOLDERNAME"]
				    path: pathToLocalMailDir];
      if ( ![[NSFileManager defaultManager] fileExistsAtPath:
					      [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]] )
	{
	  [[NSFileManager defaultManager] createFileAtPath: [[theURLName path] stringByAppendingPathComponent: [theURLName foldername]]
					  contents: nil
					  attributes: nil];
	}
      RELEASE(theURLName);
      
      //
      // We create our 'default' personal setting with the new key (PERSONAL)
      //
      aMutableDictionary = [[NSMutableDictionary alloc] init];
      [aMutableDictionary setObject: @"Your name"            forKey: @"NAME"];
      [aMutableDictionary setObject: @"Your E-Mail address"  forKey: @"EMAILADDR"];
      [aMutableDictionary setObject: @""                     forKey: @"ORGANIZATION"];
      [aMutableDictionary setObject: @""                     forKey: @"REPLYTOADDR"];
      
      [[NSUserDefaults standardUserDefaults] setObject: [NSDictionary dictionaryWithObject: aMutableDictionary
								      forKey: @"General"]
					     forKey: @"PERSONAL"];
      
      RELEASE(aMutableDictionary);
      
      // We create the basic set of hidden headers
      [[NSUserDefaults standardUserDefaults] 
	setObject: [NSArray arrayWithObjects: @"Date", @"From", @"To", @"Cc", @"Subject", nil]
	forKey:@"SHOWNHEADERS"];
      
      [self showPreferencesWindow: nil];
      
    } // if ( ![[NSUserDefaults standardUserDefaults] objectForKey: @"PERSONAL"] )
  
  //
  // We initialize our store
  //
  aLocalStore = [[LocalStore alloc] initWithPathToDirectory: pathToLocalMailDir];
  
  if ( !aLocalStore )
    {
      printf("\n\nCould not open LocalStore.\n");
      printf("That means that GNUMail.app is not properly configured.\n");
      printf("To fix this problem, do the following commands:\n\n");
      printf("%% mkdir ~/gnumail\n");
      printf("%% touch ~/gnumail/Inbox\n");
      printf("%% touch ~/gnumail/Outbox\n");
      printf("%% touch ~/gnumail/Trash\n");
      printf("%% touch ~/gnumail/Drafts\n\n");
      printf("and restart GNUMail.app after that!\n\n");
      
      [NSApp terminate: self];
    }

  // We got a valid Local store, let's add it to our list of opened store in the MailboxManagerController
  [[MailboxManagerController singleInstance] setStore: aLocalStore
					     name: @"GNUMAIL_LOCAL_STORE"
					     username: NSUserName()];
  
  [[MailboxManagerController singleInstance] reloadAllFolders];
  [[[MailboxManagerController singleInstance] outlineView] reloadData];
  
  // Sync with the defaults file
  [[NSUserDefaults standardUserDefaults] synchronize];

  // We create our console
  [ConsoleWindowController singleInstance];

  // We load all our bundles
  [self loadBundles];
  
  // We finally show our Inbox (or any other folders)
  [self showFoldersToOpen];

  // If we have at least one mail window open, we verify for mails
  // on startup.
  if ( [GNUMail lastMailWindowOnTop] )
    {
      [[[GNUMail lastMailWindowOnTop] windowController] verifyForNewMailOnStartup];
    }

  // We register our service
  [NSApp setServicesProvider: self];

  // We set up our initial list of incoming mail servers
  [self updateGetNewMailMenuItems: nil];

  // We set up our initial list of filters for the menu
  [self updateFilterMenuItems: nil];

  [self updateTextEncodingsMenu: self];

  // We show of MailboxManager window, if we need too
  if ( [[NSUserDefaults standardUserDefaults] boolForKey: @"OPEN_MAILBOXMANAGER_ON_STARTUP"] )
    {
      [self showMailboxManager: nil];
    }
}


//
// methods invoked by notifications
//
- (void) selectionInTextViewHasChanged: (id) sender
{
  NSRange aRange;

  // Cocoa bug - NSOldSelectedCharacterRange isn't declared under OSX
#ifndef MACOSX
  aRange = [[[sender userInfo] objectForKey: NSOldSelectedCharacterRange] rangeValue];
#else
  aRange = [[[sender userInfo] objectForKey: @"NSOldSelectedCharacterRange"] rangeValue];
#endif

  if ( aRange.length )
    {
      [enterSelection setAction: @selector(enterSelectionInFindPanel:)];
    }
  else
    {
      [enterSelection setAction: NULL];
    }
}


//
//
//
- (void) selectionOfMessageHasChanged: (id) sender
{
  MessageViewWindowController *messageViewWindowController;
  MailWindowController *aMailWindowController;
  id aWindowDelegate;

  messageViewWindowController = nil;
  aMailWindowController = nil;
  aWindowDelegate = nil;

  if ( [GNUMail lastMailWindowOnTop] )
    { 
      aWindowDelegate = [[GNUMail lastMailWindowOnTop] delegate];
      
      if ( aWindowDelegate )
    	{
	  if ( [aWindowDelegate isKindOfClass: [MailWindowController class]] )
	    {
	      aMailWindowController = (MailWindowController *)aWindowDelegate;
	    }
	  else
	    {
	      messageViewWindowController = (MessageViewWindowController *)aWindowDelegate;
	    }
	  
	  // No matter how the selection has changed, we always remove all items from the
	  // "Send to Address Book" menu.
	  [self removeAllItemsFromMenu: sendToAddressBook];
	  
	  // We we are in a situation with multiple rows selected, we must 
	  // take the first message in our enumerator for flags comparaison.
	  if ( aMailWindowController && [[aMailWindowController tableView] numberOfSelectedRows] > 1 )
	    {
	      NSEnumerator *anEnumerator;
	      Message *aMessage;
	      NSNumber *aRow;
	      
	      anEnumerator = [[aMailWindowController tableView] selectedRowEnumerator];
	      aRow = [anEnumerator nextObject];
	      aMessage = [[[aMailWindowController folder] allMessages] objectAtIndex: [aRow intValue]];
	      
	      [self updateMenuItemsForMessage: aMessage];
	    }
	  else
	    {
	      Message *aMessage;
	      int i;
	      
	      aMessage = [aWindowDelegate selectedMessage];
	      
	      [self updateMenuItemsForMessage: aMessage];

	      // We finally build our 'Send to Address Book' menu
	      if ( [[aMessage from] unicodeStringValue] )
		{
		  [sendToAddressBook addItemWithTitle: [[aMessage from] unicodeStringValue]
				     action: @selector(addToAddressBook:)
				     keyEquivalent: @""];
		}
	      
	      for (i = 0; i < [aMessage recipientsCount]; i++)
		{
		  if ( [[[aMessage recipients] objectAtIndex: i] unicodeStringValue] )
		    {
		      [sendToAddressBook addItemWithTitle: [[[aMessage recipients] objectAtIndex: i] unicodeStringValue]
					 action: @selector(addToAddressBook:)
					 keyEquivalent: @""];
		    }
		}
	    }
	} // if ( aMailWindowController ) ...
    }
  
  // We now have to decide what the "Show All Headers" menu item should read.
  // If the top window is a MailWindowController, no matter what, we always 
  // reset our menu item for showing all headers when this action method is called 
  // since we are switching to another message anyway.
  // If the top window is a MessageViewWindowController, we have to find out what the state 
  // the message is in.
  if ( aWindowDelegate &&
       [aWindowDelegate isKindOfClass: [MailWindowController class]] && 
       ![aMailWindowController showAllHeaders] )
    {      
      [showAllHeaders setTitle: _(@"Show All Headers")];
      [showAllHeaders setTag: SHOW_ALL_HEADERS];
      [showRawSource setTitle: _(@"Show RAW Source")];
      [aWindowDelegate setShowRawSource: NO];
    }
  else if ( aWindowDelegate &&
	    [aWindowDelegate isKindOfClass: [MessageViewWindowController class]] )
    {
      if ( [aWindowDelegate showAllHeaders] )
        {
	  [showAllHeaders setTitle: _(@"Show Filtered Headers")];
	  [showAllHeaders setTag: HIDE_ALL_HEADERS];
        }
      else
        {
	  [showAllHeaders setTitle: _(@"Show All Headers")];
	  [showAllHeaders setTag: SHOW_ALL_HEADERS];
        }
      // set the menu text for showinf raw source
      if ( [aWindowDelegate showRawSource] )
	{
	  [showRawSource setTitle: _(@"Show Normal Display")];
	}
      else
	{
	  [showRawSource setTitle: _(@"Show RAW Source")];
	}
    }
}


//
// access / mutation methods
//
+ (NSArray *) allBundles
{
  return allBundles;
}


+ (NSArray *) allMailWindows
{
  return allMailWindows;
}


+ (NSString *) currentWorkingPath
{
  return currentWorkingPath;
}


+ (void) setCurrentWorkingPath: (NSString *) thePath
{
  RETAIN(thePath);
  RELEASE(currentWorkingPath);
  currentWorkingPath = thePath;
}


+ (id) lastAddressTakerWindowOnTop
{
  return lastAddressTakerWindowOnTop;
}


+ (void) setLastAddressTakerWindowOnTop: (id) aWindow
{
  lastAddressTakerWindowOnTop = aWindow;
}


+ (id) lastMailWindowOnTop
{ 
  return lastMailWindowOnTop;
}


+ (void) setLastMailWindowOnTop: (id) aWindow
{
  lastMailWindowOnTop = aWindow;

  if ( [NSApp delegate] && lastMailWindowOnTop )
    {
      [[NSApp delegate] updateShowOrHideDeletedMenuItem:
			  [[[lastMailWindowOnTop windowController] folder] showDeleted]];
      [[NSApp delegate] updateShowOrHideReadMenuItem:
			  [[[lastMailWindowOnTop windowController] folder] showRead]];
      
    }
}


//
//
//
- (NSMenu *) saveAttachmentMenu
{
  return saveAttachment;
}


//
//
//
- (void) setEnableSaveInDraftsMenuItem: (BOOL) aBOOL
{
  if ( aBOOL )
    {
      [saveInDrafts setAction: @selector(saveInDrafts:)];
    }
  else
    {
      [saveInDrafts setAction: NULL];
    }
}


//
//
//
- (void) setShowRawSourceMenuItem: (BOOL) aBool
{
  if (aBool)
    {
      [showRawSource setTitle: _(@"Show RAW Source")];
    }
  else
    {
      [showRawSource setTitle: _(@"Show Normal Display")];
    }
}


//
//
//
- (NSMenuItem *) showAllHeadersMenuItem
{
  return showAllHeaders;
}


//
// other methods
//
- (void) addItemToMenuFromTextAttachment: (NSTextAttachment *) theTextAttachment
{
  NSFileWrapper *aFileWrapper;
  ExtendedMenuItem *menuItem;

  aFileWrapper = [theTextAttachment fileWrapper];
 
  menuItem = [[ExtendedMenuItem alloc] initWithTitle: [aFileWrapper preferredFilename]
				       action: @selector(saveAttachment:)
				       keyEquivalent: @""];
  [menuItem setTextAttachment: theTextAttachment];

  [[self saveAttachmentMenu] addItem: menuItem];

  RELEASE(menuItem);
}


//
//
//
+ (void) addMailWindow: (id) theMailWindow
{
  if (allMailWindows && theMailWindow )
    {
      [allMailWindows addObject: theMailWindow];
    }
}


//
//
//
- (void) addToAddressBook: (id) sender
{
  InternetAddress *anInternetAddress;
  Address *anAddress;

  if (! [sender isKindOfClass: [NSMenuItem class]] )
    {
      return;
    }

  anInternetAddress = [[InternetAddress alloc] initWithString: [sender title]];
  
  if (! anInternetAddress )
    {
      NSDebugLog(@"Could not add %@ to the Address Book.", [sender title]);
      return;
    }
  
  anAddress = [[Address alloc] init];  
  [anAddress setName: [anInternetAddress personal]];
  [anAddress setEmailAddress: [anInternetAddress address]];
  [[AddressBookController singleInstance] addToAddressBook: anAddress];

  RELEASE(anInternetAddress);
  RELEASE(anAddress);
}


//
// Method used to load all bundles in $GNUSTEP_USER_ROOT/Library/GNUMail
//
- (void) loadBundles
{
  NSFileManager *aFileManager;
  NSArray *allFiles;
  int i;


  aFileManager = [NSFileManager defaultManager];
  allFiles = [aFileManager directoryContentsAtPath: GNUMailUserLibraryPath()];

  for (i = 0; i < [allFiles count]; i++)
    {
      NSString *aString;
      
      aString = [allFiles objectAtIndex: i];
      
      // If we found a bundle, let's load it!
      if ( [[aString pathExtension] isEqualToString: @"bundle"] )
	{
	  id<GNUMailBundle> aModule;
	  NSBundle *aBundle;
	  NSString *path;

	  path = [NSString stringWithFormat: @"%@/%@",
			   GNUMailUserLibraryPath(),
			   aString];
	  
	  aBundle = [NSBundle bundleWithPath: path];

	  if ( aBundle )
	    {
	      aModule = [[aBundle principalClass] singleInstance];

	      if ( aModule )
		{
		  [aModule setOwner: self];
		  [allBundles addObject: aModule];
		  [[NSDistributedNotificationCenter defaultCenter]
		    postNotificationName: GNUMailConsoleMessage
		    object: nil
		    userInfo: [NSDictionary dictionaryWithObject: [NSString stringWithFormat: _(@"Loaded bundle at path %@"),
									    path]
					    forKey: @"Message"]];
		}
	      else
		{
		  [[NSDistributedNotificationCenter defaultCenter]
		    postNotificationName: GNUMailConsoleMessage
		    object: nil
		    userInfo: [NSDictionary dictionaryWithObject: [NSString stringWithFormat: 
									      _(@"Failed to initialize bundle at path %@"), 
									    path]
					    forKey: @"Message"]];
		}
	    }
	  else
	    {
	      [[NSDistributedNotificationCenter defaultCenter]
		postNotificationName: GNUMailConsoleMessage
		object: nil
		userInfo: [NSDictionary dictionaryWithObject: [NSString stringWithFormat: 
									  _(@"Error loading bundle at path %@"), path]
					forKey: @"Message"]];
	    }
	}
    }
}


//
//
//
- (void) removeAllItemsFromMenu: (NSMenu *) theMenu
{
  int i;
  
  for (i = ([theMenu numberOfItems] - 1); i >= 0; i--)
    {
      [theMenu removeItemAtIndex: i];
    }
}


//
//
//
+ (void) removeMailWindow: (id) theMailWindow
{
  if (allMailWindows && theMailWindow )
    {
      [allMailWindows removeObject: theMailWindow];
    }
}


//
// This method is used to show the local inbox window after
// the startup of GNUMail.app or any other folder (for example,
// an IMAP folder).
//
- (void) showFoldersToOpen
{ 
  NSMutableArray *foldersToOpen;
  int i;
  
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"FOLDERS_TO_OPEN"] )
    {
      foldersToOpen = [[NSUserDefaults standardUserDefaults] objectForKey: @"FOLDERS_TO_OPEN"];
    }
  else
    {
      foldersToOpen = [NSMutableArray array];
      [foldersToOpen addObject: [[NSUserDefaults standardUserDefaults] objectForKey: @"INBOXFOLDERNAME"]];

      [[NSUserDefaults standardUserDefaults] setObject: foldersToOpen
					     forKey: @"FOLDERS_TO_OPEN"];
      [[NSUserDefaults standardUserDefaults] synchronize];
    }
  
  
  for (i = 0; i < [foldersToOpen count]; i++)
    {
      URLName *urlName;

      urlName = [[URLName alloc] initWithString: [foldersToOpen objectAtIndex: i]
				 path: [[NSUserDefaults standardUserDefaults] objectForKey: @"LOCALMAILDIR"]];

      if ( [[urlName protocol] caseInsensitiveCompare: @"LOCAL"] == NSOrderedSame )
	{
	  [[MailboxManagerController singleInstance] openLocalFolderWithName: [urlName foldername]];
	}
      else if ( [[urlName protocol] caseInsensitiveCompare: @"IMAP"] == NSOrderedSame )
	{
	  MailboxManagerController *aController;

	  aController = [MailboxManagerController singleInstance];

	  if ( [aController initializeIMAPStoreWithKey: [NSString stringWithFormat: @"%@ @ %@",
								  [urlName username],
								  [urlName host]]] )
	    {
	      [aController openIMAPFolderWithName: [urlName foldername]
  			   store: (IMAPStore *)[aController storeForName: [urlName host]
  							    username: [urlName username]]];
	    }
	}
      
      RELEASE(urlName);
    }
}


//
//
//
- (void) updateGetNewMailMenuItems: (id) sender
{
  NSMenuItem *aMenuItem;
  NSArray *allKeys;
  NSDictionary *allValues;
  NSString *aKey;
  int i;

  // We first remove all our items in the current menu
  [self removeAllItemsFromMenu: incomingMailServers];
  
  // Our "All" menu item
  aMenuItem = [[NSMenuItem alloc] initWithTitle: _(@"All")
				  action: @selector(getNewMessages:)
				  keyEquivalent: @"N"];
  [aMenuItem setTarget: self];
  [aMenuItem setTag: -1];
  [incomingMailServers addItem: aMenuItem];
  RELEASE(aMenuItem);

  // We sort the array to be sure to keep the order.
  allKeys = [[[[NSUserDefaults standardUserDefaults] objectForKey: @"RECEIVING"] allKeys]
	      sortedArrayUsingSelector: @selector(compare:)];

  for (i = 0; i < [allKeys count]; i++)
    {
      aKey = [allKeys objectAtIndex: i];
      allValues = [[[NSUserDefaults standardUserDefaults] objectForKey: @"RECEIVING"] objectForKey: aKey];
      
      // FIXME: We must also consider UNIX receiving type accounts.
      //        MailWindowController: -getNewMessages: MUST also be fixed.
      if ( ![allValues objectForKey: @"SERVERTYPE"] ||
	   [[allValues objectForKey: @"SERVERTYPE"] intValue] == POP3 )
	{
	  aMenuItem = [[NSMenuItem alloc] initWithTitle: aKey
					  action: @selector(getNewMessages:)
					  keyEquivalent: @""];
	  [aMenuItem setTarget: self];
	  [aMenuItem setTag: i];
	  [incomingMailServers addItem: aMenuItem];
	  RELEASE(aMenuItem);
	}
    }
}


//
//
//
- (void) updateFilterMenuItems: (id) sender
{
  BOOL isDir;
  
  if ( [[NSFileManager defaultManager] fileExistsAtPath: PathToFilters()
				       isDirectory: &isDir] &&
       ! isDir )
    {
      FilterManager *aFilterManager;
      NSMenuItem *aMenuItem;
      int i;
  
      // We first remove all our items in the current menu
      [self removeAllItemsFromMenu: filters];
    
      aFilterManager = [FilterManager filtersFromDisk];
      RETAIN(aFilterManager);
      
      // Our "All" menu item
      aMenuItem = [[NSMenuItem alloc] initWithTitle: _(@"All")
				      action: @selector(applyManualFilter:)
				      keyEquivalent: @""];
      [aMenuItem setTag: -1];
      [filters addItem: aMenuItem];
      RELEASE(aMenuItem);

      for (i = 0; i < [[aFilterManager filters] count]; i++)
	{
	  Filter *aFilter;
	  
	  aFilter = [[aFilterManager filters] objectAtIndex: i];
	  
	  aMenuItem = [[NSMenuItem alloc] initWithTitle: [aFilter description]
					  action: @selector(applyManualFilter:)
					  keyEquivalent: @""];
	  [aMenuItem setTag: i];
	  [filters addItem: aMenuItem];
	  RELEASE(aMenuItem);
	}

      RELEASE(aFilterManager);
    }
}


//
//
//
- (void) updateMenuItemsForMessage: (Message *) theMessage
{
  // Initial verification, we never know!
  if (! theMessage )
    {
      return;
    }
  
  // We verify if we need to delete or undelete our message and 
  // we set the tag to the right value
  if ( [[theMessage flags] contain: DELETED] )
    {
      [deleteOrUndelete setTitle: _(@"Undelete")];
      [deleteOrUndelete setTag: UNDELETE_MESSAGE];
      [deleteOrUndelete setKeyEquivalent: @"u"];
    }
  else
    {
      [deleteOrUndelete setTitle: _(@"Delete")];
      [deleteOrUndelete setTag: DELETE_MESSAGE];
      [deleteOrUndelete setKeyEquivalent: @"d"];
    }
  
  // We verify if we need to mark the message as read or unread
  if ( [[theMessage flags] contain: SEEN] )
    {
      [markAsReadOrUnread setTitle: _(@"Mark as Unread")];
		  [markAsReadOrUnread setTag: MARK_AS_UNREAD];
    }
  else
    {
      [markAsReadOrUnread setTitle: _(@"Mark as Read")];
      [markAsReadOrUnread setTag: MARK_AS_READ];
    }      
}


//
//
//
- (void) updateShowOrHideDeletedMenuItem: (BOOL) aBOOL
{
  if ( aBOOL )
    {
      [showOrHideDeleted setTitle: _(@"Hide Deleted")];
      [showOrHideDeleted setTag: HIDE_DELETED_MESSAGES];
    }
  else
    {
      [showOrHideDeleted setTitle: _(@"Show Deleted")];
      [showOrHideDeleted setTag: SHOW_DELETED_MESSAGES];
    }
}


//
//
//
- (void) updateShowOrHideReadMenuItem: (BOOL) aBOOL
{
  if ( aBOOL )
    {
      [showOrHideRead setTitle: _(@"Hide Read")];
      [showOrHideRead setTag: HIDE_READ_MESSAGES];
    }
  else
    {
      [showOrHideRead setTitle: _(@"Show Read")];
      [showOrHideRead setTag: SHOW_READ_MESSAGES];
    }
}


//
// services methods
//
- (void) newMessageWithContent: (NSPasteboard *) pboard
	              userData: (NSString *) userData
                         error: (NSString **) error
{
  EditWindowController *editWindowController;
  NSString *aString;
  NSArray *allTypes;
  Message *aMessage;

  allTypes = [pboard types];

  if ( ![allTypes containsObject: NSStringPboardType] )
    {
      *error = @"No string type supplied on pasteboard";
      return;
    }
  
  aString = [pboard stringForType: NSStringPboardType];
  
  if (aString == nil)
    {
      *error = @"No string value supplied on pasteboard";
      return;
    }
  
  // We create a new message with our pasteboard content
  aMessage = [[Message alloc] init];
  [aMessage setContent: aString];

  // We create our controller and we show the window
  editWindowController = [[EditWindowController alloc] initWithWindowNibName: @"EditWindow"];

  if ( editWindowController )
    {
      [[editWindowController window] setTitle: _(@"New message...")];
      [editWindowController setMessage: aMessage];
      [editWindowController setShowCc: NO];
    
      [[editWindowController window] orderFrontRegardless];
    }
  
  RELEASE(aMessage);
}


//
//
//
- (void) updateTextEncodingsMenu: (id) sender
{
  NSMutableArray *aMutableArray;
  NSMenuItem *item;
  int i;

  [self removeAllItemsFromMenu: textEncodings];
  
  item = [[NSMenuItem alloc] initWithTitle: _(@"Default")
			     action: @selector(changeTextEncoding:)
			     keyEquivalent: @""];
  [item setTag: -1];
  [textEncodings addItem: item];
  RELEASE(item);

  aMutableArray = [[NSMutableArray alloc] init];
  [aMutableArray addObjectsFromArray: [[MimeUtility allCharsets] allValues]];
  [aMutableArray sortUsingSelector: @selector(compare:)];

  for (i = 0; i < [aMutableArray count]; i++)
    {
      item = [[NSMenuItem alloc] initWithTitle: [aMutableArray objectAtIndex: i]
				 action: @selector(changeTextEncoding:)
				 keyEquivalent: @""];
      [item setTag: i];
      [textEncodings addItem: item];
      RELEASE(item);
    }

  RELEASE(aMutableArray);
}


//
//
//
- (void) newMessageWithRecipient: (NSPasteboard *) pboard
	                userData: (NSString *) userData
                           error: (NSString **) error
{
  EditWindowController *editWindowController;
  InternetAddress *anInternetAddress;
  NSString *aString;
  NSArray *allTypes;
  Message *aMessage;

  allTypes = [pboard types];

  if ( ![allTypes containsObject: NSStringPboardType] )
    {
      *error = @"No string type supplied on pasteboard";
      return;
    }
  
  aString = [pboard stringForType: NSStringPboardType];
  
  if (aString == nil)
    {
      *error = @"No string value supplied on pasteboard";
      return;
    }
  
  // We create a new message and we set the recipient
  aMessage = [[Message alloc] init];
  anInternetAddress = [[InternetAddress alloc] initWithString: aString];
  [anInternetAddress setType: TO];
  [aMessage addToRecipients: anInternetAddress];
  RELEASE(anInternetAddress);

  // We create our controller and we show the window
  editWindowController = [[EditWindowController alloc] initWithWindowNibName: @"EditWindow"];

  if ( editWindowController )
    {
      [[editWindowController window] setTitle: _(@"New message...")];
      [editWindowController setMessage: aMessage];
      [editWindowController setShowCc: NO];
    
      [[editWindowController window] orderFrontRegardless];
    }
  
  RELEASE(aMessage);
}

@end


//
// Private methods
//
@implementation GNUMail (Private)

//
// This method will ALWAYS return a message with a retain count == 1.
//
- (Message *) _messageFromLocalDraftsFolderUsingURLName: (URLName *) theURLName
{
  MailboxManagerController *aMailboxManagerController;
  
  LocalFolder *draftsFolder;
  LocalStore *aStore;

  Message *aMessage;
  id aWindow;

  aMailboxManagerController = [MailboxManagerController singleInstance];
  
  aMessage = nil;
  aWindow = nil;
  

  aStore = [aMailboxManagerController storeForName: @"GNUMAIL_LOCAL_STORE"
				      username: NSUserName()];
  
  draftsFolder = [aStore folderForName: [theURLName foldername]];
  
  // The folder is opened, we get the selected message
  if ( !draftsFolder )
    {
      MailWindowController *aMailWindowController;
      
      aWindow = [Utilities windowForFolderName: [theURLName foldername]
			   store: (Store *)aStore];
      
      aMailWindowController = [aWindow windowController];
      draftsFolder = (LocalFolder *)[aMailWindowController folder];
      
      if ( [draftsFolder count] > 0 &&
	   [aMailWindowController selectedMessage])
	{
	  aMessage = [aMailWindowController selectedMessage];
	  RETAIN(aMessage);
	}
      else
	{
	  NSBeep();
	  return nil;
	}
    }
  else
    {
      // The folder is NOT opened, let's get the last message
      if ( [draftsFolder count] > 0 )
	{
	  aMessage = [draftsFolder messageAtIndex: ([draftsFolder count] - 1)];
	  
	  // We MUST initialize our message
	  [aMessage setInitialized: YES];
	  RETAIN(aMessage);
	}
      else
	{
	  NSBeep();
	  [draftsFolder close];
	  return nil;
	}
    }


  if ( aWindow != nil )
    {
      [draftsFolder close];
    }

  return aMessage;
}


//
// This method will ALWAYS return a message with a retain count == 1.
//
- (Message *) _messageFromIMAPDraftsFolderUsingURLName: (URLName *) theURLName
{
  MailboxManagerController *aMailboxManagerController;
  
  IMAPFolder *draftsFolder;
  IMAPStore *aStore;
  
  Message *aMessage;
  id aWindow;

  aMailboxManagerController = [MailboxManagerController singleInstance];

  aMessage = nil;
  aWindow = nil;
 
  aStore = [aMailboxManagerController storeForName: [theURLName host]
				      username: [theURLName username]];
  
  
  // We verify if the folder opened on this server is the Drafts folder.
  if ( aStore && [aStore folderForNameIsOpen: [theURLName foldername]] )
    {
      // We do nothing here. We will simply restore the selected message
      // by using this store.
      MailWindowController *aMailWindowController;
      
      aWindow = [Utilities windowForFolderName: [theURLName foldername]
			   store: (Store *)aStore];
      
      aMailWindowController = [aWindow windowController];
      draftsFolder = (IMAPFolder *)[aMailWindowController folder];
      
      if ( [draftsFolder count] > 0 &&
	   [aMailWindowController selectedMessage])
	{
	  aMessage = [aMailWindowController selectedMessage];
	  RETAIN(aMessage);
	}
      else
	{
	  NSBeep();
	  return nil;
	}
    }
  // Store is nil or an other folder is opened. We simply establish a new IMAP connection
  else
    {
      NSDictionary *allValues;
      NSNumber *portValue;
      NSString *aKey;
      
      aKey = [NSString stringWithFormat: @"%@ @ %@", [theURLName username], [theURLName host]];
      
      allValues = [[[NSUserDefaults standardUserDefaults] objectForKey: @"RECEIVING"] 
		    objectForKey: aKey];
      
      portValue = [allValues objectForKey: @"PORT"];
      
      // We use the default IMAP port if it's not defined.
      if ( !portValue )
	{
	  portValue = [NSNumber numberWithInt: 143];
	}
      
      // We verify if we must establish an IMAP/SSL connection.
      if ( [allValues objectForKey: @"USESECURECONNECTION"] &&
	   [[allValues objectForKey: @"USESECURECONNECTION"] intValue] == NSOnState )
	{
	  aStore = [[IMAPStore alloc] initSSLWithName: [theURLName host]
				      port: [portValue intValue]];
	}
      else
	{
	  aStore = [[IMAPStore alloc] initWithName: [theURLName host]
				      port: [portValue intValue]];
	}
      
      if ( aStore )
	{	      
	  if ( [aStore authenticateWithUsername: [theURLName username]
		       password: [aMailboxManagerController passwordForKey: aKey]] )
	    {
	      IMAPCacheManager *theIMAPCacheManager;
	      
	      theIMAPCacheManager = [IMAPCacheManager imapCacheObjectsFromDiskWithPath:
							[NSString stringWithFormat: @"%@/IMAPCache_%@_%@",
								  GNUMailUserLibraryPath(),
								  [Utilities flattenPathFromString: aKey
									     separator: @"/"],
								  [Utilities flattenPathFromString: [theURLName foldername]
									     separator: @"/"]]];
	      
	      draftsFolder = [aStore folderForName: [theURLName foldername]
				     withIMAPCacheManager: theIMAPCacheManager];
	      
	      if ( [draftsFolder count] > 0 )
		{
		  aMessage = [draftsFolder messageAtIndex: ([draftsFolder count] - 1)];
		  
		  // We MUST initialize our message
		  [aMessage setInitialized: YES];
		  RETAIN(aMessage);
		}
	      
	      [draftsFolder close];
	    }
	  
	  [aStore close];
	  RELEASE(aStore);
	}
    }
  
  return aMessage;
}

@end


//
// 
//
@implementation ExtendedMenuItem
{
  NSTextAttachment *textAttachment;
}

- (NSTextAttachment *) textAttachment
{
  return textAttachment;
}

- (void) setTextAttachment: (NSTextAttachment *) theTextAttachment
{
  textAttachment = theTextAttachment;
}

@end


//
// Starting point of GNUMail.app
//
int main(int argc, const char *argv[], char *env[])
{
  NSAutoreleasePool *pool;
  GNUMail *gnumail;

  pool = [[NSAutoreleasePool alloc] init];
  gnumail = [[GNUMail alloc] init];

  [NSApplication sharedApplication];
  [NSApp setDelegate: gnumail];

  NSApplicationMain(argc, argv);

  RELEASE(gnumail);
  RELEASE(pool);

  return 0;
}
