/*
**  MailWindowToolbar.m
**
**  Copyright (c) 2002
**
**  Author: Francis Lachapelle <francis@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "MailWindowToolbar.h"


@implementation MailWindowController (MailWindowToolbar)


//
// NSToolbar delegate methods
//

- (NSToolbarItem *) toolbar: (NSToolbar *) toolbar
      itemForItemIdentifier: (NSString *) itemIdentifier
  willBeInsertedIntoToolbar: (BOOL) flag
{
  NSToolbarItem *item;
  
  if ([itemIdentifier isEqualToString: @"delete"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Delete"];
      [item setPaletteLabel: @"Delete Message"];
      [item setImage: [NSImage imageNamed: @"delete_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(deleteMessage:)];
    }
  else if ([itemIdentifier isEqualToString: @"retrieve"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Get"];
      [item setPaletteLabel: @"Get New Messages"];
      [item setImage: [NSImage imageNamed: @"retrieve_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(getNewMessages:)];
      get = (id)item;
    }
  else if ([itemIdentifier isEqualToString: @"mailbox"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Mailbox"];
      [item setPaletteLabel: @"Show Mailboxes"];
      [item setImage: [NSImage imageNamed: @"mailboxes_32.tiff"]];
      [item setTarget: [NSApp delegate]];
      [item setAction: @selector(showMailboxManager:)];
    }
  else if ([itemIdentifier isEqualToString: @"compose"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Compose"];
      [item setPaletteLabel: @"Compose New Message"];
      [item setImage: [NSImage imageNamed: @"create_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(composeMessage:)];
    }
  else if ([itemIdentifier isEqualToString: @"reply"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Reply"];
      [item setPaletteLabel: @"Reply To Message"];
      [item setImage: [NSImage imageNamed: @"reply_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(replyMessage:)];
    }
  else if ([itemIdentifier isEqualToString: @"forward"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Forward"];
      [item setPaletteLabel: @"Forward Message"];
      [item setImage: [NSImage imageNamed: @"forward_32.tiff"]];
      [item setTarget: self];
      [item setAction: @selector(forwardMessage:)];
    }
  else if ([itemIdentifier isEqualToString: @"addresses"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Addresses"];
      [item setPaletteLabel: @"Addresses"];
      [item setImage: [NSImage imageNamed: @"addresses_32.tiff"]];
      [item setTarget: [NSApp delegate]];
      [item setAction: @selector(showAddressBook:)];
    }
  else if ([itemIdentifier isEqualToString: @"find"])
    {
      item = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @"Find"];
      [item setPaletteLabel: @"Find Messages"];
      [item setImage: [NSImage imageNamed: @"find_32.tiff"]];
      [item setTarget: [NSApp delegate]];
      [item setAction: @selector(showFindWindow:)];
    }
  else if ([itemIdentifier isEqualToString: @"navigation"])
    {
      NSRect aRect = [navigationView frame];
      item = [[MailNavigationToolbarItem alloc] initWithItemIdentifier: itemIdentifier];
      [item setLabel: @""];
      [item setPaletteLabel: @"Navigation"];
      [item setView: navigationView];
      [item setMinSize: aRect.size];
      [item setMaxSize: aRect.size];
      [(MailNavigationToolbarItem*) item setDelegate: self];
    }

  return [item autorelease];
}


- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar*) toolbar
{
  return [NSArray arrayWithObjects: NSToolbarSeparatorItemIdentifier,
		                    NSToolbarSpaceItemIdentifier,
		                    NSToolbarFlexibleSpaceItemIdentifier,
		                    NSToolbarCustomizeToolbarItemIdentifier, 
		  @"delete",
		  @"retrieve",
		  @"mailbox",
		  @"compose",
                  @"reply",
		  @"forward",
		  @"addresses",
		  @"find",
		  @"navigation",
		  nil];
}


- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar*) toolbar
{
  return [NSArray arrayWithObjects: @"delete",
		  @"retrieve",
		  @"mailbox",
		  @"compose",
		  @"reply",
		  @"forward",
		  @"addresses",
		  @"find",
		  NSToolbarFlexibleSpaceItemIdentifier,
		  @"navigation",
		  nil];
}


//
// NSToolbarItemValidation protocol
//

- (BOOL) validateToolbarItem: (NSToolbarItem *) theItem
{
  
  return YES;
}


@end


@implementation MailNavigationToolbarItem: NSToolbarItem

- (void) setDelegate: (id) theDelegate
{
  delegate = theDelegate;
}


- (void) validate
{
  MailWindowController* aMailWindowController;
  int index;

  aMailWindowController = (MailWindowController*) delegate;
  index = [[aMailWindowController tableView] selectedRow];

  [aMailWindowController->previous setEnabled: ( index > 0 )];
  [aMailWindowController->next setEnabled: 
			  ( index < ([[aMailWindowController tableView] numberOfRows] - 1) )];
}

@end
