/*
**  MailboxManager.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#import "MailboxManager.h"

#import "GNUMail.h"
#import "GNUMailConstants.h"
#import "LabelWidget.h"
#import "ExtendedOutlineView.h"

#import <Pantomime/LocalStore.h>


@implementation MailboxManager

//
//
//
- (void) layoutWindow
{
  NSButton *create, *delete, *close, *rename;

  icon = [[NSImageView alloc] initWithFrame: NSMakeRect(4,362,16,16)];
  [icon setImageAlignment: NSImageAlignCenter];
  [icon setImage: nil];
  [icon setImageFrameStyle: NSImageFrameNone];
  [icon setEditable: NO];
  [icon setAutoresizingMask: NSViewMinYMargin];
  [[self contentView] addSubview: icon];

  delete = [[NSButton alloc] initWithFrame: NSMakeRect(24,388,40,40)];
  [delete setButtonType: NSMomentaryPushButton];
  [delete setTitle: @""];
  [delete setTarget: [self windowController]];
  [delete setAction: @selector(delete:)];
  [delete setImagePosition: NSImageOnly];
  [delete setImage: [NSImage imageNamed: @"delete_32.tiff"]];
  [delete setAutoresizingMask: NSViewMinYMargin];
  [[self contentView] addSubview: delete];
  RELEASE(delete);
   
  create = [[NSButton alloc] initWithFrame: NSMakeRect(69,388,40,40)];
  [create setButtonType: NSMomentaryPushButton];
  [create setTitle: @""];
  [create setTarget: [self windowController]];
  [create setAction: @selector(create:)];
  [create setImagePosition: NSImageOnly];
  [create setImage: [NSImage imageNamed: @"mailbox_add_32.tiff"]];
  [create setAutoresizingMask: NSViewMinYMargin];
  [[self contentView] addSubview: create];
  RELEASE(create);
  
  rename = [[NSButton alloc] initWithFrame: NSMakeRect(114,388,40,40)];
  [rename setButtonType: NSMomentaryPushButton];
  [rename setTitle: @""];
  [rename setTarget: [self windowController]];
  [rename setAction: @selector(rename:)];
  [rename setImagePosition: NSImageOnly];
  [rename setImage: [NSImage imageNamed: @"mailbox_rename_32.tiff"]];
  [rename setAutoresizingMask: NSViewMinYMargin];
  [[self contentView] addSubview: rename];
  RELEASE(rename);

  move = [[NSButton alloc] initWithFrame: NSMakeRect(291,388,40,40)];
  [move setButtonType: NSMomentaryPushButton];
  [move setTitle: @""];
  [move setTag: MOVE_MESSAGES];
  [move setTarget: [self windowController]];
  [move setAction: @selector(transfer:)];
  [move setImagePosition: NSImageOnly];
  [move setImage: [NSImage imageNamed: @"mailbox_move_32.tiff"]];
  [move setAutoresizingMask: NSViewMinXMargin|NSViewMinYMargin];
  [[self contentView] addSubview: move];
  RELEASE(move);

  copy = [[NSButton alloc] initWithFrame: NSMakeRect(336,388,40,40)];
  [copy setButtonType: NSMomentaryPushButton];
  [copy setTitle: @""];
  [copy setTag: COPY_MESSAGES];
  [copy setTarget: [self windowController]];
  [copy setAction: @selector(transfer:)];
  [copy setImagePosition: NSImageOnly];
  [copy setImage: [NSImage imageNamed: @"mailbox_copy_32.tiff"]];
  [copy setAutoresizingMask: NSViewMinXMargin|NSViewMinYMargin];
  [[self contentView] addSubview: copy];
  RELEASE(copy);

  openOnStartup = [[NSButton alloc] initWithFrame: NSMakeRect(24,356,370,ButtonHeight)];
  [openOnStartup setButtonType: NSSwitchButton];
  [openOnStartup setBordered: NO];
  [openOnStartup setEnabled: NO];
  [openOnStartup setTitle: _(@"Open this folder on startup")];
  [openOnStartup setTarget: [self windowController]];
  [openOnStartup setAction: @selector(openOnStartupClicked:)];
  [openOnStartup setAutoresizingMask: NSViewMinYMargin];
  [[self contentView] addSubview: openOnStartup];

  mailboxColumn = [[NSTableColumn alloc] initWithIdentifier: @"Mailbox"];
  [mailboxColumn setEditable: NO];
  [[mailboxColumn headerCell] setStringValue: _(@"Mailbox")];
  [mailboxColumn setMinWidth: 200];
  
  sizeColumn = [[NSTableColumn alloc] initWithIdentifier: @"Size"];
  [sizeColumn setEditable: NO];
  [[sizeColumn headerCell] setStringValue: _(@"Size")];
  [sizeColumn setMinWidth: 60];

  messagesColumn = [[NSTableColumn alloc] initWithIdentifier: @"Messages"];
  [messagesColumn setEditable: NO];
  [[messagesColumn headerCell] setStringValue: _(@"Messages")];

  outlineView = [[ExtendedOutlineView alloc] initWithFrame: NSMakeRect (0, 40, 400, 310)];
  [outlineView addTableColumn: mailboxColumn];
  [outlineView addTableColumn: sizeColumn];
  [outlineView addTableColumn: messagesColumn];
  [outlineView setOutlineTableColumn: mailboxColumn];
  [outlineView setDrawsGrid: NO];
  [outlineView setIndentationPerLevel: 10];
  [outlineView setAutoresizesOutlineColumn: YES];
  [outlineView setIndentationMarkerFollowsCell: YES];
  [outlineView setAllowsColumnSelection: NO];
  [outlineView setAllowsColumnReordering: NO];
  [outlineView setAutoresizesAllColumnsToFit: YES];
  [outlineView sizeLastColumnToFit];
  [outlineView setDataSource: [self windowController]];
  [outlineView setDelegate: [self windowController]];
  [outlineView setDoubleAction: @selector(open:)];

  scrollView = [[NSScrollView alloc] initWithFrame: NSMakeRect (0, 40, 400, 310)];
  [scrollView setDocumentView: outlineView];
  [scrollView setHasHorizontalScroller: NO];
  [scrollView setHasVerticalScroller: YES];
  [scrollView setBorderType: NSBezelBorder];
  [scrollView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [[self contentView] addSubview: scrollView];
  
  close = [[NSButton alloc] initWithFrame: NSMakeRect(335,7,60,ButtonHeight)];
  [close setButtonType: NSMomentaryPushButton];
  [close setTitle: _(@"Close")];
  [close setTarget: self];
  [close setAction: @selector(close)];
  [close setAutoresizingMask: NSViewMinXMargin|NSViewMaxYMargin];
  [[self contentView] addSubview:close];
  RELEASE(close);

  [self setMinSize: NSMakeSize(300,250)];
}


//
//
//
- (void) dealloc
{
  //NSDebugLog(@"MailboxManager: -dealloc");

  RELEASE(mailboxColumn);
  RELEASE(sizeColumn);
  RELEASE(messagesColumn);
  
  RELEASE(outlineView);
  RELEASE(scrollView);

  RELEASE(openOnStartup);

  [super dealloc];
}


//
// access/mutation methods
//
- (NSButton *) move
{
  return move;
}

- (NSButton *) copy
{
  return copy;
}

- (NSImageView *) icon
{
  return icon;
}

- (ExtendedOutlineView *) outlineView
{
  return outlineView;
}

- (NSScrollView *) scrollView
{
  return scrollView;
}

- (NSButton *) openOnStartup
{
  return openOnStartup;
}

@end
