/*
**  MailboxManagerController.h
**
**  Copyright (C) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import <AppKit/AppKit.h>

@class ExtendedOutlineView;
@class Flags;
@class Folder;
@class FolderNode;
@class IMAPStore;
@class MailboxManagerCache;
@class Store;
@class URLName;

@interface MailboxManagerController: NSWindowController
{
  // Outlets
  IBOutlet NSButton *move, *copy;
  IBOutlet ExtendedOutlineView *outlineView;
  IBOutlet NSButton *openOnStartup;
  IBOutlet NSImageView *icon;
  IBOutlet NSScrollView *scrollView;

  // Other ivars
  MailboxManagerCache *mailboxManagerCache;

  NSMutableArray *allFolders;
  FolderNode *localNodes;

  NSMutableDictionary *allStores;
}

- (id) initWithWindowNibName: (NSString *) windowNibName;
- (void) dealloc;

//
// delegate methods
//
- (void) windowWillClose: (NSNotification *) theNotification;


//
// action methods
//

- (IBAction) openOnStartupClicked: (id) sender;

- (IBAction) open: (id) sender;
- (IBAction) transfer: (id) sender;
- (IBAction) create: (id) sender;
- (IBAction) delete: (id) sender;
- (IBAction) rename: (id) sender;

- (void) receivingValuesHaveChanged: (id) sender;

//
// access/mutation methods
//

- (NSOutlineView *) outlineView;

- (id) storeForName: (NSString *) theName
           username: (NSString *) theUsername;

- (void) setStore: (id) theStore
             name: (NSString *) theName
         username: (NSString *) theUsername;


- (MailboxManagerCache *) mailboxManagerCache;
- (void) setMailboxManagerCache: (MailboxManagerCache *) theMailboxManagerCache;

//
// class methods
//
+ (id) singleInstance;


//
// Other methods
//
- (void) openLocalFolderWithName: (NSString *) theFolderName;

- (void) openIMAPFolderWithName: (NSString *) theFolderName
                          store: (IMAPStore *) theStore;

- (void) reloadAllFolders;

- (BOOL) initializeIMAPStoreWithKey: (NSString *) theKey;

- (BOOL) transferMessageFromRawSource: (NSData *) theRawSource
                                flags: (Flags *) theFlags
		     toFolderWithName: (NSString *) theFolderName
                           orToFolder: (Folder *) theFolder
                                store: (Store *) theStore;

- (BOOL) folderNameExists: (NSString *) theFolderName
                 forStore: (Store *) theStore;

- (NSString *) passwordForKey: (NSString *) theKey;

@end


//
// Private methods
//
@interface MailboxManagerController (Private)

- (void) _nbOfMessages: (int *) theNbOfMessages
    nbOfUnreadMessages: (int *) theNbOfUnreadMessages
             totalSize: (int *) theTotalSize
               forItem: (id) theItem;

- (void) _reloadFoldersForIMAPStoreWithName: (NSString *) theName
                                   username: (NSString *) theUsername;

- (FolderNode *) _storeFolderNodeForName: (NSString *) theName;

- (void) _reloadFoldersAndExpandParentsFromNode: (FolderNode *) theNode
                             selectNodeWithPath: (NSString *) thePath;

- (FolderNode *) _rootFolderNodeForNode: (FolderNode *) theNode;

- (void) _updateShowOrHideReadAndDeletedForFolder: (Folder *) theFolder;

@end
