/*
**  MessageViewWindowController.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Ujwal Sathyam on Thu Jun 13 2002.
**          Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "MessageViewWindowController.h"

#import "GNUMail.h"
#import "GNUMail/GNUMailBundle.h"
#import "GNUMailConstants.h"
#import "ExtendedCell.h"
#import "MailHeaderCell.h"
#import "MailWindowController.h"
#import "MimeType.h"
#import "MimeTypeManager.h"
#import "Utilities.h"

#ifndef MACOSX
#import "MessageViewWindow.h"
#endif

#import <Pantomime/Constants.h>
#import <Pantomime/Folder.h>
#import <Pantomime/IMAPFolder.h>
#import <Pantomime/IMAPStore.h>
#import <Pantomime/InternetAddress.h>
#import <Pantomime/LocalFolder.h>
#import <Pantomime/LocalStore.h>
#import <Pantomime/Message.h>
#import <Pantomime/MimeBodyPart.h>
#import <Pantomime/MimeMultipart.h>


@implementation MessageViewWindowController

//
//
//
- (id) initWithWindowNibName: (NSString *) theWindowNibName
{
#ifdef MACOSX
  self = [super initWithWindowNibName: theWindowNibName];
#else
  MessageViewWindow *messageViewWindow;
  
  messageViewWindow = [[MessageViewWindow alloc] initWithContentRect: NSMakeRect(150,100,720,600)
						 styleMask: NSClosableWindowMask|NSTitledWindowMask|
						 NSMiniaturizableWindowMask|NSResizableWindowMask
						 backing: NSBackingStoreRetained
						 defer: NO];

  self = [super initWithWindow: messageViewWindow];

  [messageViewWindow layoutWindow];
  [messageViewWindow setDelegate: self];

  textView = [messageViewWindow textView];
  previousMessage = [messageViewWindow previousMessage];
  nextMessage = [messageViewWindow previousMessage];
  showOrHideAllHeaders = [messageViewWindow showOrHideAllHeaders];

  RELEASE(messageViewWindow);
#endif

  [[self window] setFrameAutosaveName: @"MessageViewWindow"];
  [[self window] setFrameUsingName: @"MessageViewWindow"];
  
  // We create our mail header cell
  mailHeaderCell = [[MailHeaderCell alloc] init];

  [[NSNotificationCenter defaultCenter] addObserver: mailHeaderCell
					selector: @selector (resize:) 
					name: @"NSViewFrameDidChangeNotification" 
					object: textView];

  // We load our accessory views
  [self _loadAccessoryViews];
  
  return self;
}


//
//
//
#ifdef MACOSX
- (void) awakeFromNib
{
  NSToolbar *toolbar;
  
  toolbar = [[NSToolbar alloc] initWithIdentifier: @"MessageViewWindowToolbar"];
  [toolbar setDelegate: self];
  [toolbar setAllowsUserCustomization: YES];
  [toolbar setAutosavesConfiguration: YES];
  [[self window] setToolbar: toolbar];
  RELEASE(toolbar);
}
#endif


//
//
//
- (void) dealloc
{
  NSDebugLog(@"MessageViewWindowController: dealloc called for message window: %@", [message subject]);
  
  // Cocoa bug? Does not look like it.
  //#ifdef MACOSX
  //[[self window] setToolbar: nil];
  //#endif
  
  [[NSNotificationCenter defaultCenter] removeObserver: mailHeaderCell
					name: @"NSViewFrameDidChangeNotification" 
					object: textView];

  RELEASE(mailHeaderCell);

  RELEASE(message);
  RELEASE(folder);

  [super dealloc];
}



//
// action methods
//
- (IBAction) deleteMessage: (id) sender
{
  // FIXME
  [self close];
}


//
//
//
- (IBAction) replyMessage: (id) sender
{
  // Invoke the message utility class method to reply to this message.
  [Utilities replyToSender: [self message]
	     folder: [self folder]];
}


//
//
//
- (IBAction) replyAllMessage: (id) sender
{
  
  // Invoke the message utility class method to reply to this message.
  [Utilities replyToAll: [self message]
	     folder: [self folder]];
}


//
//
//
- (IBAction) forwardMessage: (id) sender
{ 
  // Invoke the message utility class method to forward this message.
  [Utilities forwardMessage: [self message]];

}


//
//
//
- (IBAction) previousMessage: (id) sender
{
  Message *aMessage;
  
  if ( listIndex == 0 ) 
    {
      NSBeep();
      return;
    }
  
  aMessage = [[mailWindowController allMessages] objectAtIndex: listIndex-1];
  
  if ( aMessage )
    {
      [self setMessage: aMessage];
      [Utilities showMessage: [self message]
		 target: [self textView]
		 showAllHeaders: [self showAllHeaders]];
      listIndex--;
      [self windowDidBecomeKey:nil];
    }
}


//
//
//
- (IBAction) nextMessage: (id) sender
{
  Message *aMessage;
  
  NSDebugLog(@"number of rows = %d; listIndex = %d", [[mailWindowController tableView] numberOfRows], listIndex); 
  
  if ( listIndex >= ([[mailWindowController tableView] numberOfRows] - 1) ) 
    {
      NSBeep();
      return;
    }

  aMessage = [[mailWindowController allMessages] objectAtIndex: listIndex+1];
  
  if ( aMessage )
    {
      [self setMessage: aMessage];
      [Utilities showMessage: [self message]
		 target: [self textView]
		 showAllHeaders: [self showAllHeaders]];
      listIndex++;
      [self windowDidBecomeKey:nil];
    }
  
}


//
//
//
- (IBAction) firstMessage: (id) sender
{
  Message *aMessage;
  
  listIndex = 0;
  
  aMessage = [[mailWindowController allMessages] objectAtIndex: listIndex];
  
  if ( aMessage )
    {
      [self setMessage: aMessage];
      [Utilities showMessage: [self message]
		 target: [self textView]
		 showAllHeaders: [self showAllHeaders]];
      [self windowDidBecomeKey: nil];
    }
}


//
//
//
- (IBAction) lastMessage: (id) sender
{
  Message *aMessage;
  
  listIndex = [[mailWindowController tableView] numberOfRows] - 1;
  
  aMessage = [[mailWindowController allMessages] objectAtIndex: listIndex];
  
  if ( aMessage )
    {
      [self setMessage: aMessage];
      [Utilities showMessage: [self message]
		 target: [self textView]
		 showAllHeaders: [self showAllHeaders]];
      [self windowDidBecomeKey: nil];
    }
}


//
//
//
- (IBAction) showOrHideAllHeaders: (id) sender
{
  [(GNUMail *)[NSApp delegate] showAllHeaders: sender];
}


//
// accessor/mutator methods
//
- (NSTextView *) textView
{
  return textView;
}

- (Message *) message
{
  return message;
}


// FIXME
// This is the same as the above method. It exists just so that it matches the one in
// MailWindowController.
- (Message *) selectedMessage
{
  return message;
}


- (void) setMessage: (Message *) aMessage
{
  NSString *windowTitle = nil;
  
  if ( aMessage )
    {
      RETAIN(aMessage);
      RELEASE(message);
      message = aMessage;
      windowTitle = [message subject];
      
      if (windowTitle)
	{
	  [[self window] setTitle: windowTitle];
	}
    }
}


- (Folder *) folder
{
  return folder;
}


- (void) setFolder: (Folder *) aFolder
{
  if ( aFolder )
    {
      RETAIN(aFolder);
      RELEASE(folder);
      folder = aFolder;
    }
}


- (MailWindowController *) mailWindowController
{
  return mailWindowController;
}


- (void) setMailWindowController: (MailWindowController *) aMailWindowController
{
  if ( aMailWindowController )
    {
      mailWindowController = aMailWindowController;
    }
}

- (BOOL) showAllHeaders
{
  if ( [[NSUserDefaults standardUserDefaults] objectForKey: @"SHOWALLHEADERS"] )
    {
      return ([[[NSUserDefaults standardUserDefaults] objectForKey: @"SHOWALLHEADERS"] intValue] == NSOnState ? YES
	      : showAllHeaders);
    }
  
  return showAllHeaders;
}

- (void) setShowAllHeaders: (BOOL) aBool
{
  showAllHeaders = aBool;
}

- (BOOL) showRawSource
{
  return showRawSource;
}

- (void) setShowRawSource: (BOOL) aBool
{
  showRawSource = aBool;
}


- (int) listIndex
{
  return listIndex;
}

- (void) setListIndex: (int)anIndex
{
  listIndex = anIndex;
}


//
//
//
- (MailHeaderCell *) mailHeaderCell
{
  return mailHeaderCell;
}


//
// delegate methods
//
- (void) windowDidLoad
{
#ifdef MACOSX
  [[self window] setFrameAutosaveName: @"MessageViewWindow"];
  [[self window] setFrameUsingName: @"MessageViewWindow"];
#endif  

  // We set the last window on top
  [GNUMail setLastMailWindowOnTop: [self window]]; 
}


//
//
//
- (void) windowDidBecomeKey: (NSNotification *) aNotification
{  
  // We clear our 'Save Attachment' menu
  [(GNUMail *)[NSApp delegate] removeAllItemsFromMenu: [(GNUMail *)[NSApp delegate] saveAttachmentMenu]];
  
  // We set the last window on top
  [GNUMail setLastMailWindowOnTop: [self window]];
  
  // Post a notification to GNUMail.m so that the menu gets updated.
  [[NSNotificationCenter defaultCenter]
    postNotificationName: SelectionOfMessageHasChanged
    object: nil
    userInfo: nil];

  // We adjust our showOrHideAllHeaders button
  [showOrHideAllHeaders setTag: [[(GNUMail *)[NSApp delegate] showAllHeadersMenuItem] tag]];
  [showOrHideAllHeaders setTitle: [[(GNUMail *)[NSApp delegate] showAllHeadersMenuItem] title]];
}


//
//
//
- (void) windowWillClose: (NSNotification *) theNotification
{
  // We update our last mail window on top
  if ( [GNUMail lastMailWindowOnTop] == [self window] )
    {
      [GNUMail setLastMailWindowOnTop: nil];
    }

  // We remove our window controller from the list of opened window controllers
  // of the "parent" MailWindowController object.
  [[[self mailWindowController] allMessageViewWindowControllers] removeObject: self];
  
  AUTORELEASE(self);
}


//
//
//
-  (void) textView: (NSTextView *) aTextView
     clickedOnCell: (id <NSTextAttachmentCell>) attachmentCell
	    inRect: (NSRect) cellFrame
	   atIndex: (unsigned) charIndex
  
{
  [Utilities clickedOnCell: attachmentCell
	     inRect: cellFrame
	     atIndex: charIndex];
}


//
//
//
- (BOOL) textView: (NSTextView *) textView
    clickedOnLink: (id) link 
	  atIndex: (unsigned) charIndex
{
  NSDebugLog(@"Opening %@...", [link description]);
  return [[NSWorkspace sharedWorkspace] openURL: link];
}


@end


//
// Private interface
//
@implementation MessageViewWindowController (Private)

- (void) _loadAccessoryViews
{
  int i, index;

  index = 0;

  for (i = 0; i < [[GNUMail allBundles] count]; i++)
    {
      id<GNUMailBundle> aBundle;

      aBundle = [[GNUMail allBundles] objectAtIndex: i];
      
      if ( [aBundle hasViewingViewAccessory] )
	{
	  id aView;
	  
	  aView = [aBundle viewingViewAccessory];
	  
	  if ( [aBundle viewingViewAccessoryType] == ViewingViewTypeHeaderCell )
	    {
	      NSDebugLog(@"Adding ViewingViewTypeHeaderCell type of Bundle...");
	      [mailHeaderCell addView: aView];
	    }
	}

      // We also set the current superview
      [aBundle setCurrentSuperview: [[self window] contentView]];
    }
}

@end
