/*
**  Utilities.h
**
**  Copyright (c) 2001, 2002
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import <AppKit/AppKit.h>

@class Address;
@class Folder;
@class FolderNode;
@class FolderNodePopUpItem;
@class Message;
@class MimeBodyPart;
@class MimeMultipart;
@class NSView;
@class Part;
@class Store;
@class URLName;


@interface Utilities: NSObject
{
}

+ (NSFont *) fontFromFamilyName: (NSString *) theName
			  trait: (int) theTrait
                           size: (int) theSize;

+ (void) appendAddress: (Address *) theAddress
           toTextField: (NSTextField *) theTextField;

+ (NSAttributedString *) attributedStringFromContentForPart: (Part *) thePart;

+ (NSAttributedString *) attributedStringFromHeadersForMessage: (Message *) theMessage
                                                showAllHeaders: (BOOL) aBOOL;

+ (NSAttributedString *) attributedStringWithString: (NSString *) theString
                                         attributes: (NSDictionary *) theAttributes;


+ (NSAttributedString *) bestRepresentationFromMultipartAlternative: (MimeMultipart *) theMimeMultipart;

+ (NSAttributedString *) bestRepresentationFromMultipartAppleDouble: (MimeMultipart *) theMimeMultipart;
                                                  
+ (NSAttributedString *) formattedAttributedStringFromAttributedString: (NSAttributedString *) theAttributedString;


+ (NSString *) stringFromRecipients: (NSArray *) allRecipients
                               type: (int) recipientType;

+ (NSString *) encryptPassword: (NSString *) thePassword
                       withKey: (NSString *) theKey;
+ (NSString *) decryptPassword: (NSString *) thePassword
                       withKey: (NSString *) theKey;

+ (void) loadPersonalProfilesInPopUpButton: (NSPopUpButton *) thePopUpButton selectedProfile: (NSString*) profile;

+ (void) loadTransportMethodsInPopUpButton: (NSPopUpButton *) thePopUpButton;

+ (NSString *) profileNameForItemTitle: (NSString *) theString;

+ (id) windowForFolderName: (NSString *) theName
                     store: (Store *) theStore;

+ (FolderNode *) folderNodeForPath: (NSString *) thePath
                             using: (FolderNode *) rootNode;

+ (FolderNode *) folderNodesFromFolders: (NSEnumerator *) theFolders
                              separator: (NSString *) theSeparator;


+ (NSString *) completePathForFolderNode: (FolderNode *) theFolderNode
                               separator: (NSString *) theSeparator;

+ (NSString *) pathOfFolderFromFolderNode: (FolderNode *) theFolderNode
                                separator: (NSString *) theSeparator;

+ (NSString *) flattenPathFromString: (NSString *) theString
                           separator: (NSString *) theSeparator;


+ (NSString *) storeKeyForFolderNode: (FolderNode *) theFolderNode
                          serverName: (NSString **) theServerName
                            username: (NSString **) theUsername;

+ (BOOL) URLWithString: (NSString *) theString
           matchFolder: (Folder *) theFolder;

+ (id) storeForURLName: (URLName *) theURLName;


+ (NSString *) stringValueOfURLNameFromFolderNode: (FolderNode *) theFolderNode;

+ (FolderNode *) initializeLocalAndIMAPFolderNodes;

+ (void) addItemsToPopUpButton: (NSPopUpButton *) thePopUpButton
              usingFolderNodes: (FolderNode *) theFolderNodes;

+ (void) addItem: (FolderNode *) theFolderNode
	   level: (int) theLevel
   toPopUpButton: (NSPopUpButton *) thePopUpButton;

+ (FolderNodePopUpItem *) folderNodePopUpItemForFolderNode: (FolderNode *) theFolderNode
                                               popUpButton: (NSPopUpButton *) thePopUpButton;

+ (FolderNodePopUpItem *) folderNodePopUpItemForURLNameAsString: (NSString *) theString
                                               usingFolderNodes: (FolderNode *) theFolderNodes
                                                    popUpButton: (NSPopUpButton *) thePopUpButton;

+ (NSString *) passwordForKey: (NSString *) theKey
                       prompt: (BOOL) aBOOL;

+ (NSMutableDictionary *) passwordCache;

+ (void) replyToSender: (Message *) theMessage
                folder: (Folder *) theFolder;

+ (void) replyToAll: (Message *) theMessage
             folder: (Folder *) theFolder;

+ (void) forwardMessage: (Message *) theMessage;

+ (void) showMessage: (Message *) theMessage
              target: (NSTextView *) theTextView
      showAllHeaders: (BOOL) headersFlag;

+ (void) showMessageRawSource: (Message *)theMessage
                       target: (NSTextView *)theTextView;

+ (void) clickedOnCell: (id <NSTextAttachmentCell>) attachmentCell
	        inRect: (NSRect) cellFrame
               atIndex: (unsigned) charIndex;

@end

//
// Utilities private interface
//
@interface Utilities (Private)

+ (NSMutableAttributedString *) _quoteMessageContentFromAttributedString: (NSAttributedString *) theAttributedString;
+ (void) _highlightAndActivateURLs:(NSTextView *)targetView;

+ (NSString *) _guessPersonalProfile: (Message *)theMessage;

@end

NSString *GNUMailUserLibraryPath();
